#!/bin/bash
#            TheToolChain
#
#  Install script for basic extensions.
#
#  Script written by Gregor Rebel 2012 - 2014
# 
#  Feel free do distribute and change to your own needs!
#

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

setInstallDir "060_basic_extensions" "$1" # sets $Start_Path, $Install_Dir, $Install_Path
findFolderUpwards extensions; Dir_Extensions="$FoundFolder"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  
if [ ! -e OK.Documentation ]; then #{
  Get_Fails=""
  
  # download documentation files + link them into documentation folder
  # 
  # getDocumentation(URL_Prefix,File,OutFile,DocSubDir)     will download given URL into given subdirectory of documentation folder
  
  #getDocumentation http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library STM32F10x_Standard_Peripherals_Library.html uC/STM/StdPeripheralLibrary/
  #getDocumentation http://frama-c.com/download/frama-c-user-manual.pdf frama-c-user-manual.pdf Compiler/Debugging
  getDocumentation http://werner.dichler.at/sites/default/files/attachment/ prj21_CRC%20Einfuehrung.pdf CRC-Einfuehrung.pdf Algorithms
  
  if [ "$Get_Fails" == "" ]; then #{ all files downloaded successfully 
    touch OK.Documentation
  else
    echo "$ScriptName - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then #{
  
  if [ ! -e OK.Packages ]; then #{
    # nothing to install (yet)
    Arch=`uname -i`
    if [ "$Arch" == "x86_64" ]; then
      installPackage "ia32-libs"
    fi
    installPackageSafe "which xdotool" "xdotool"
    installPackageSafe "which kdbg"    "kdbg"
    installPackageSafe "which ddd"     "ddd"
    
    #?installPackageSafe "which frama-c" "frama-c"
    touch OK.Install
    touch OK.Packages
  fi #}
fi #}
if [ ! -d "$HOME/.ddd" ]; then #{ ddd configuration directory is missing; create default configuration file providing TTC color set
  dir "$HOME/.ddd"
  cat <<"END_OF_TEXT" >$HOME/.ddd/init
! New Color Set for The ToolChain
Ddd*foreground:                    yellow
Ddd*background:                    grey18
Ddd*XmText.background:             grey18
Ddd*XmTextField.background:        grey18
Ddd*GraphEdit.background:          grey18
Ddd*XmList.background:             grey18
Ddd*graph_edit_panner.background:  grey18
Ddd*graph_edit_panner.foreground:  grey96
Ddd*graph_edit_panner.shadowColor: grey


! DO NOT ADD ANYTHING BELOW THIS LINE -- DDD WILL OVERWRITE IT
Ddd*dddinitVersion: 3.3.12

! Debugger settings.
Ddd*autoDebugger: off
Ddd*debugger: gdb
Ddd*useSourcePath: off
! Ddd*bashSettings: 
! Ddd*dbgSettings: 
! Ddd*dbxSettings: 
Ddd*gdbSettings: \
set confirm off\n\
define hookpost-step\n\
mon cortex_m maskisr off\n\
end\n\
define hook-step\n\
mon cortex_m maskisr on\n\
end\n\
set confirm on\n
! Ddd*jdbSettings: 
! Ddd*perlSettings: 
! Ddd*pydbSettings: 
! Ddd*xdbSettings: 

! Source.
Ddd*findWordsOnly: on
Ddd*findCaseSensitive: on
! Ddd*tabWidth: 8
! Ddd*indentSource: 0
! Ddd*indentCode: 4
Ddd*cacheSourceFiles: on
Ddd*cacheMachineCode: on
Ddd*displayGlyphs: on
Ddd*displayLineNumbers: on
Ddd*disassemble: off
Ddd*allRegisters: off

! Undo Buffer.
! Ddd*maxUndoDepth: -1
! Ddd*maxUndoSize: 2000000

! Misc preferences.
Ddd*keyboardFocusPolicy: POINTER
Ddd*statusAtBottom: on
Ddd*suppressWarnings: off
Ddd*warnIfLocked: off
Ddd*checkGrabs: on
Ddd*saveHistoryOnExit: on
Ddd*paperSize: 210mm x 297mm
Ddd*blinkWhileBusy: on
Ddd*splashScreen: off
Ddd*startupTips: off

! Keys.
Ddd*globalTabCompletion: on
Ddd*cutCopyPasteBindings: Motif
Ddd*selectAllBindings: KDE

! Data.
Ddd*pannedGraphEditor: off
Ddd*graph_edit.showGrid: on
Ddd*graph_edit.snapToGrid: on
Ddd*graph_edit.showHints: on
Ddd*graph_edit.showAnnotations: on
Ddd*graph_edit.layoutMode: compact
Ddd*graph_edit.autoLayout: off
Ddd*showBaseDisplayTitles: on
Ddd*showDependentDisplayTitles: on
Ddd*autoCloseDataWindow: on
Ddd*graph_edit.GridSize: 11
Ddd*detectAliases: on
Ddd*clusterDisplays: on
Ddd*displayPlacement: XmHORIZONTAL
Ddd*align2dArrays: on
Ddd*arrayOrientation: XmVERTICAL
Ddd*structOrientation: XmVERTICAL
Ddd*showMemberNames: on

! Themes.
Ddd*themes: \
! green.vsl	*\n\
! red.vsl	*\n\
rednil.vsl	*\n\
! smalltitles.vsl	*\n\
smallvalues.vsl	*\n\
! suppress.vsl	\"(((t_ttc_usart_config *) 0x20000504)->BaseRegister)->DR\"\n\
! tinyvalues.vsl	*\n\
x86.vsl	$eax*;$ebx*;$ecx*;$edx*;($eflags &*\n

! Tips.
Ddd*buttonTips: on
Ddd*valueTips: on
Ddd*buttonDocs: on
Ddd*valueDocs: on

! Helpers.
! Ddd*editCommand: ${XEDITOR-false} +@LINE@ @FILE@ || xterm -e sensible-editor +@LINE@ @FILE@
! Ddd*getCoreCommand: gcore -o @FILE@ @PID@
! Ddd*psCommand: ps x 2> /dev/null || ps -ef 2> /dev/null || ps
! Ddd*termCommand: xterm -bg \'grey96\' -fg \'black\' -cr \'DarkGreen\' -fn \'@FONT@\' -title \'DDD: Execution Window\' -e /bin/sh -c
! Ddd*uncompressCommand: gzip -d -c
! Ddd*wwwCommand: firefox -remote \'openURL(@URL@)\' || mozilla -remote \'openURL(@URL@)\' || opera -remote \'openURL(@URL@)\' || ${WWWBROWSER-false} \'@URL@\' || konqueror \'openURL(@URL@)\' || galeon \'openURL(@URL@)\' || skipstone \'openURL(@URL@)\' || light \'openURL(@URL@)\' || netscape -remote \'openURL(@URL@)\' || mozilla \'@URL@\' || kfmbrowser \'@URL@\' || netscape \'@URL@\' || gnudoit \'(w3-fetch \"@URL@\")\' || mosaic \'@URL@\' || Mosaic \'@URL@\' || xterm -e lynx \'@URL@\'
! Ddd*plotCommand: gnuplot -bg \'grey96\' -font \'@FONT@\' -name \'@NAME@\' -geometry +5000+5000
Ddd*plotTermType: xlib
! Ddd*printCommand: lpr

! Tool Bars.
Ddd*toolbarsAtBottom: off
Ddd*buttonImages: on
Ddd*buttonCaptions: on
Ddd*FlatButtons: on
Ddd*buttonColorKey: g
Ddd*activeButtonColorKey: c

! Command Tool.
Ddd*commandToolBar: off
Ddd*toolRightOffset: 213
Ddd*toolTopOffset: -350

! Buttons.
Ddd*consoleButtons: 
Ddd*sourceButtons: 
Ddd*dataButtons: 
Ddd*verifyButtons: on

! Display shortcuts.
Ddd*bashDisplayShortcuts: 
Ddd*dbgDisplayShortcuts: 
! Ddd*dbxDisplayShortcuts: 
! Ddd*gdbDisplayShortcuts: \
! /t ()	// Convert to Bin\n\
! /d ()	// Convert to Dec\n\
! /x ()	// Convert to Hex\n\
! /o ()	// Convert to Oct
Ddd*makeDisplayShortcuts: 
! Ddd*jdbDisplayShortcuts: 
! Ddd*perlDisplayShortcuts: \
! sprintf(\"%#x\", ())	// Convert to Hex\n\
! sprintf(\"%#o\", ())	// Convert to Oct
! Ddd*pydbDisplayShortcuts: \
! /t ()	// Convert to Bin\n\
! /d ()	// Convert to Dec\n\
! /x ()	// Convert to Hex\n\
! /o ()	// Convert to Oct
! Ddd*xdbDisplayShortcuts: 

! Fonts.
! Ddd*defaultFont: helvetica-medium
! Ddd*variableWidthFont: helvetica-medium
Ddd*fixedWidthFont: -adobe-helvetica-medium-o-normal-*-11-80-100-100-p-57-iso8859
Ddd*dataFont: helvetica-medium
Ddd*FontSize: 100

! Windows.
Ddd*openDataWindow: on
Ddd*openSourceWindow: on
Ddd*openDebuggerConsole: on
Ddd*Separate: off
Ddd*separateExecWindow: off
Ddd*groupIconify: off
Ddd*uniconifyWhenReady: on

! Maintenance.
Ddd*dumpCore: on
Ddd*debugCoreDumps: off

! Window sizes.
Ddd*graph_edit.height: 559
Ddd*source_text_w.columns: 267
Ddd*source_text_w.rows: 19
Ddd*code_text_w.columns: 1
Ddd*code_text_w.rows: 1
Ddd*gdb_w.columns: 267
Ddd*gdb_w.rows: 7

END_OF_TEXT
fi #}

if [ -e OK.Install ]; then #{
  
  Name="${Install_Dir}_basic_setup"
  createExtensionMakefileHead ${Name} #{         create makefile for extension
  File="${Dir_Extensions}makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

# Makefile created by $ScriptName 

COMPILE_OPTS += -DEXTENSION_${Name}=1  

# avoid modern gcc to generate lots of useles warnings of type "warning: dereferencing type-punned pointer will break strict-aliasing rules [-Wstrict-aliasing]"
COMPILE_OPTS += -fno-strict-aliasing

# basic software interface of TheToolChain (ttc_XXX() functions)
INCLUDE_DIRS += -I ttc-lib/
INCLUDE_DIRS += -I ttc-lib/scheduler/
INCLUDE_DIRS += -I examples/
INCLUDE_DIRS += -I regressions/
INCLUDE_DIRS += -I extensions.active/

vpath %.c ttc-lib/ ttc-lib/interfaces/ ttc-lib/scheduler/ examples/ regressions/ extensions.active/

# add some basic object files which are always available
MAIN_OBJS += ttc_extensions.o ttc_extensions_active.o

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ${Dir_Extensions} "$ScriptName" "Basic toolchain settings required by all architectures and boards." #{ create activate script
  if [ "$Delete" != "" ]; then
    echo "rm 2>/dev/null \${Dir_ExtensionsActive}/${Install_Dir}" >>$ActivateScriptFile
  fi
  #X ../scripts/printBaseExtensions.sh >>$ActivateScriptFile
  cat <<END_OF_ACTTIVATE >>$ActivateScriptFile #{

createLink \${Dir_Extensions}/makefile.$Name \${Dir_ExtensionsActive}/makefile.$Name '' QUIET
#? activate.500_ttc_basic.sh QUIET "\$ScriptName"

END_OF_ACTTIVATE
#}
  createActivateScriptTail $Name ${Dir_Extensions} 
#}
  
  Name="${Install_Dir}_optimize_1_debug"
  createExtensionMakefileHead ${Name}
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{

DEBUG=1
OPTIMIZE+=-g3 # enable maximum debug informations (even macros)

END_OF_MAKEFILE
#}
  createExtensionMakefileTail ${Name}
  #createActivateScript "${Dir_Extensions}activate.${Name}.sh" $Name "makefile.${Install_Dir}_optimize_1_*" "$ScriptName" "compile option: do not optimize at all (best visibility during debugging)"
  createActivateScript $Name ${Dir_Extensions} "$ScriptName" "" "compile option: do not optimize at all (best visibility during debugging)"

  Name="${Install_Dir}_optimize_2_smallest"
  createExtensionMakefileHead ${Name}
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{
  
OPTIMIZE+=-Os 

END_OF_MAKEFILE
#}
  createExtensionMakefileTail ${Name}
  createActivateScript $Name ${Dir_Extensions} "$ScriptName" "makefile.${Install_Dir}_optimize_2_*" "compiler option: create small code (can degrade execution speed)"

  Name="${Install_Dir}_optimize_2_fastest"
  createExtensionMakefileHead ${Name}
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{

OPTIMIZE+=-O9

END_OF_MAKEFILE
#}
  createExtensionMakefileTail ${Name}
  createActivateScript $Name ${Dir_Extensions} "$ScriptName" "makefile.${Install_Dir}_optimize_2_*" "compiler option: create fast code (can increase code- and memory-usage)"
  
  Name="${Install_Dir}_optimize_3_inline"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

COMPILE_OPTS += -finline -finline-functions-called-once

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScript $Name ${Dir_Extensions} "$ScriptName" "" "compiler option: try to inline as many functions as possible (can increase code- and memory-usage)"

  Name="${Install_Dir}_optimize_3_remove_unused"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

COMPILE_OPTS += -fdata-sections -ffunction-sections
LDFLAGS += -Wl,--gc-sections

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScript $Name ${Dir_Extensions} "$ScriptName" "" "compiler option: remove all unused functions and data elements (reduces resulting code size but may remove functions used during debugging sessions)"

  Name="${Install_Dir}_disable_asserts"
  DynamicMakefile="makefile.000_disable_asserts"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

# including a makefile that has been created dynamically
include extensions.local/$DynamicMakefile

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ${Dir_Extensions} "$ScriptName 2" "Disables all Assert_XXX() macros (Makes code faster and binary smaller but more difficult to debug)" #{ (create activate script)
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \${Dir_Extensions}        -> extensions/
#   \${Dir_ExtensionsLocal}   -> extensions.local/
#   \${Dir_ExtensionsActive}  -> extensions.active/
#   \$Dir_Additionals       -> additionals/
#   \$Dir_Configs           -> configs/

# ACTIVATE_SECTION_A remove activated variants of same type
# rm 2>/dev/null \${Dir_ExtensionsActive}/*${Install_Dir}_*

# ACTIVATE_SECTION_B create link to your makefile into extensions.active/ to add makefile to your project
createLink \${Dir_Extensions}/makefile.$Name \${Dir_ExtensionsActive}/makefile.$Name '' QUIET

if [ ! -e "\${Dir_ExtensionsLocal}/$DynamicMakefile" ]; then
echo "creating file '\${Dir_ExtensionsLocal}/$DynamicMakefile'.."
cat <<END_OF_FILE >\${Dir_ExtensionsLocal}/$DynamicMakefile
#         The ToolChain
#
# This file contains all assert macros found in ttc-lib/ at date of creation.
# This file has been created by \$ScriptName on \`date\`
#
# Simply remove this file and rerun \$ScriptName to create an updated version.
#

END_OF_FILE
_/findAllAsserts.pl >>\${Dir_ExtensionsLocal}/$DynamicMakefile
else
echo "$ScriptName - reusing existing file '\${Dir_ExtensionsLocal}/$DynamicMakefile'"
fi

END_OF_ACTIVATE
  createActivateScriptTail $Name ${Dir_Extensions}
  #}

  Name="${Install_Dir}_disable_safe_arrays"
  DynamicMakefile="makefile.000_disable_safe_arrays"
  createExtensionMakefileHead ${Name} #{
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

# Disabling safe array index checking (interpreted in ttc_basic_types.h)
COMPILE_OPTS += -DTTC_TUNABLE_BASIC_SAFE_ARRAYS=0

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ${Dir_Extensions} "$ScriptName 2" "Disables index checking on safe array macros A*() for faster and smaller code (see ttc_basic_types.h for a description of safe arrays))" #{ (create activate script)
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \${Dir_Extensions}        -> extensions/
#   \${Dir_ExtensionsLocal}   -> extensions.local/
#   \${Dir_ExtensionsActive}  -> extensions.active/
#   \$Dir_Additionals       -> additionals/
#   \$Dir_Configs           -> configs/

# ACTIVATE_SECTION_A remove activated variants of same type
# rm 2>/dev/null \${Dir_ExtensionsActive}/*${Install_Dir}_*

# ACTIVATE_SECTION_B create link to your makefile into extensions.active/ to add makefile to your project
createLink \${Dir_Extensions}/makefile.$Name \${Dir_ExtensionsActive}/makefile.$Name '' QUIET

if [ ! -e "\${Dir_ExtensionsLocal}/$DynamicMakefile" ]; then
echo "creating file '\${Dir_ExtensionsLocal}/$DynamicMakefile'.."
cat <<END_OF_FILE >\${Dir_ExtensionsLocal}/$DynamicMakefile
#         The ToolChain
#
# This file contains all assert macros found in ttc-lib/ at date of creation.
# This file has been created by \$ScriptName on \`date\`
#
# Simply remove this file and rerun \$ScriptName to create an updated version.
#

END_OF_FILE
echo "$ScriptName - disabling all safe arrays A*(). Will use plain C arrays instead."
fi

END_OF_ACTIVATE
  createActivateScriptTail $Name ${Dir_Extensions}
  #}

  echo "Installed successfully: $Install_Dir"
fi #}
cd ..
rmdir 2>/dev/null $Install_Dir  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$Start_Path"
applyTargetUserAndGroup "$Install_Dir"

exit 0
