#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Initial Script written by Gregor Rebel 2010-2019.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

RANK="510"
EXTENSION_SHORT="font"
EXTENSION_PREFIX="ttc"
EXTENSION_NAME="${EXTENSION_PREFIX}_$EXTENSION_SHORT"

setInstallDir "${RANK}_${EXTENSION_NAME}" "$1" # sets $Start_Path, $Install_Dir, $Install_Path
findFolderUpwards extensions; Dir_Extensions="$FoundFolder"


#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Documentation ]; then #{ (download documentation)

  Get_Fails=""
  
  # download documentation files + link them into documentation folder
  # 
  # getDocumentation(URL_Prefix,File,OutFile,DocSubDir)     will download given URL into given subdirectory of documentation folder
  
  #getDocumentation http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library STM32F10x_Standard_Peripherals_Library.html uC/STM/StdPeripheralLibrary/
  
  if [ "$Get_Fails" == "" ]; then #{ all files downloaded successfully 
    touch OK.Documentation
  else
    echo "$ScriptName - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then    #{ download + install software
  if [ ! -e OK.Packages ]; then #{ install packages
    # installPackageSafe 'which autoreconf' autoreconf
    
    touch OK.Packages
  fi #}
  if [ -e OK.Packages ];   then #{ install software
    echo "installing in $Install_Dir ..."
    Install_Path=`pwd`
    
    # getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ stm32_i2c_cpal.zip stm32_i2c_cpal.zip
    # unZIP "UNKNOWN" stm32_i2c_cpal.zip
    
    #if [ -e "SOME_FILE" ]; then
      echo "" >OK.Install
    #fi
  else
    echo "$ScriptName - ERROR: Packages were not installed correctly!"
  fi #}OK.Packages
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then    #{ create makefiles and activate scripts

  Name="${Install_Dir}"    
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead() 

INCLUDE_DIRS += -I ttc-lib/font
vpath %.c ttc-lib/font

MAIN_OBJS += ttc_font.o

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}

  DefaultFont=""  
  for Font in `ls ../../TTC-Library/font/font_*.c`; do #{ create makefile+activate for every found font
    FontName=`perl -e "my \\\$P = rindex('$Font', '/'); print substr('$Font', \\\$P+1, -2);"`
    if [ "$DefaultFont" == "" ]; then
      DefaultFont="$FontName"         # using first font as default font
    fi
    ObjectFile="${FontName}.o"
    echo "installing $FontName ..."
    
    Name2="${Install_Dir}__$FontName"    
    createExtensionMakefileHead ${Name2}      #{ create makefile
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead()

MAIN_OBJS += $ObjectFile

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name2} #}
    createActivateScriptHead $Name2 ${Dir_Extensions} "$ScriptName 2" "Device independent text font $Name2" #{
    cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \${Dir_Extensions}        -> extensions/
#   \${Dir_ExtensionsLocal}   -> extensions.local/
#   \${Dir_ExtensionsActive}  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_Additionals/$Install_Dir_*


activate.510_ttc_font.sh QUIET "\$ScriptName"

Driver="\${Dir_ExtensionsActive}/makefile.510_ttc_font"
if [ "\`ls \$Driver 2>/dev/null\`" != "" ]; then
  createLink \${Dir_Extensions}/makefile.$Name2 \${Dir_ExtensionsActive}/makefile.$Name2 '' QUIET
else
  echo "$ScriptName - missing dependency (\$Driver)"
fi

END_OF_ACTIVATE
    createActivateScriptTail $Name2 ${Dir_Extensions}
    #}

  done #}

  createActivateScriptHead $Name ${Dir_Extensions} "$ScriptName 2" "Device independent graphic output + input" #{
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \${Dir_Extensions}        -> extensions/
#   \${Dir_ExtensionsLocal}   -> extensions.local/
#   \${Dir_ExtensionsActive}  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# remove activated variants of same type
# rm 2>/dev/null \$Dir_Additionals/$Install_Dir_*

if [ "\`ls \${Dir_ExtensionsActive}/*450_font_* 2>/dev/null\`" == "" ]; then 
  echo "\$ScriptName: No font enabled: enable default font '$DefaultFont'"
  enableFeature 450_$DefaultFont
fi
activate.500_ttc_gfx.sh QUIET "\$ScriptName"

Driver="\${Dir_ExtensionsActive}/makefile.500_ttc_gfx"
if [ "\`ls \$Driver 2>/dev/null\`" != "" ]; then
  createLink \${Dir_Extensions}/makefile.$Name \${Dir_ExtensionsActive}/makefile.$Name '' QUIET
else
  echo "$ScriptName - missing dependency (\$Driver)"
fi


END_OF_ACTIVATE
  createActivateScriptTail $Name ${Dir_Extensions}
  #}

  cd "$OldPWD"
  echo "Installed successfully: $Install_Dir"
#}
else                            #{ Install failed
  echo "failed to install $Install_Dir"
  exit 10
fi #}

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$Start_Path"
applyTargetUserAndGroup "$Install_Dir"

exit 0
