#!/bin/bash

#                         The ToolChain
#
#  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
#
#  Install script for a single Low-Level Driver
#  for board devices.
#
#  Created from template _install_TTC_DEVICE_ARCHITECTURE.sh revision 39 at 20191125 14:30:13 UTC
#
#  Supported Architecture: stm32f103c8t6_blue_pill
#
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
#  Authors: Gregor Rebel
#

if [ "$EXTENSION_NAME" == "" ]; then #{ ERROR: globals must be defined by high-level install script!
  echo "`pwd`/$ScriptName - ERROR: Script must be called from high-level install script!"
  return 10
fi #}

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

# create sub-directory for each low-level driver
MyStartDir=`pwd`
ThisArchitecture="stm32f103c8t6_blue_pill"
dir "$ThisArchitecture"
cd "$ThisArchitecture"
findFolderUpwards "extensions"
findFolderUpwards extensions; DirLL_Extensions="$MyStartDir/$ThisArchitecture/$FoundFolder"
findFolderUpwards scripts;    Start_Path="$MyStartDir/$ThisArchitecture/${FoundFolder}.."

#{ look for newest versions on download pages
NewestDownloads="../${0}_downloads"
if [ "$1" == "FINDNEWEST" ] || [ ! -e "$NewestDownloads" ]; then #{ find newest downloads
  # echo -n "$ScriptName: looking for new GCC... "
  # updateNewest GCC https://launchpad.net/gcc-arm-embedded/+download 'gcc-arm-none-eabi-.+-linux\.tar\.bz2' $NewestDownloads
  if [ "$return" != "" ]; then
    # prepare folder for new version
    rm -Rf *
  fi
fi #}
if [ -e "$NewestDownloads" ]; then
  source $NewestDownloads
fi
# if [ "$GCC_URL" == "" ] || [ "$GCC_File" == "" ]; then #{ check if at least one version has been found 
#   echo "$ScriptName - ERROR: Cannot find newest GCC version!"
#   return 10
# fi #}
#}look for newest versions

if [ ! -e OK.Documentation ]; then #{ (download documentation)

  Get_Fails=""
  
  # download documentation files + link them into documentation folder
  # 
  # getDocumentation(URL_Prefix,File,OutFile,DocSubDir)     will download given URL into given subdirectory of documentation folder
  
  #  getDocumentation http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library STM32F10x_Standard_Peripherals_Library.html uC/STM/StdPeripheralLibrary/
  getDocumentation https://raw.githubusercontent.com/dhylands/stm32-test/master/board-STM32F103-Mini/ STM32F103C8T6-Schematic.pdf STM32F103C8T6_Minimum_Schematic.pdf Boards/
  
  if [ "$Get_Fails" == "" ]; then #{ all files downloaded successfully 
    touch OK.Documentation
  else
    echo "$ScriptName - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then       #{ (download + install third party software)
  if [ ! -e OK.Packages ]; then #{ install packages
    # installPackageSafe which
    
    touch OK.Packages
  fi #}
  if [ -e OK.Packages ];   then #{ install software
    echo "installing in $Install_Dir/$ThisArchitecture ..."
    Install_Path=`pwd`
    
    # getFile http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ stm32_i2c_cpal.zip stm32_i2c_cpal.zip
    # unZIP "UNKNOWN" stm32_i2c_cpal.zip
    
    #if [ -e "SOME_FILE" ]; then
      echo "" >OK.Install
    #fi
  else
    echo "$ScriptName - ERROR: Packages were not installed correctly!"
  fi #}OK.Packages
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then       #{ create makefiles and activate scripts

  if [ "1" == "1" ]; then #{ low-level driver for board on stm32f103c8t6_blue_pill
  
    DriverNameBase="${RANK_LOWLEVEL}_board_stm32f103c8t6_blue_pill"
    DriverRevision=""                            # you may add a suffix to the driver name (e.g. "_RevA")
    DriverName="${DriverNameBase}${DriverRevision}"
    createExtensionSourcefileHead ${DriverName}  #{ create initializer code for your extension
  
    # Below you may write C-code that will automatically being placed inside extensions.active/ttc_extensions_active.c during _/compile.sh
    # Every #include directive will automatically being added to extensions.active/ttc_extensions_active.h
    #
    cat <<END_OF_SOURCEFILE >>$ExtensionSourceFile


END_OF_SOURCEFILE
    createExtensionSourcefileTail ${DriverName} #}    
    createExtensionMakefileHead ${DriverName} "" ${DriverNameBase}   #{ (create makefile)
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead() 

MAIN_OBJS += board_stm32f103c8t6_blue_pill.o  # ttc-lib/board/board_stm32f103c8t6_blue_pill.c

# define board type
COMPILE_OPTS += -DTTC_BOARD1=E_ttc_board_architecture_stm32f103c8t6_blue_pill # Line generated automatically. Remove if not wanted!

# system clock setup (-> ttc_sysclock_types.h)
COMPILE_OPTS += -DTTC_SYSCLOCK_EXTERNAL_HIGHSPEED_CRYSTAL=8000000

# Enable exactly one cpu variant matching those on your board!
# Check Documentation/TheToolChain-Manual/chapter_AddingNewCPU.tml to learn how to add new cpu variants!
# TTC_CPU_VARIANT_stm32f051r8t6   = 1
# TTC_CPU_VARIANT_stm32f100rbt6   = 1
# TTC_CPU_VARIANT_stm32f103_end   = 1
# TTC_CPU_VARIANT_stm32f103_start = 1
TTC_CPU_VARIANT_stm32f103c8t6   = 1
# TTC_CPU_VARIANT_stm32f103r6t6a  = 1
# TTC_CPU_VARIANT_stm32f103r8t6   = 1
# TTC_CPU_VARIANT_stm32f103rbt6   = 1
# TTC_CPU_VARIANT_stm32f103rct6   = 1
# TTC_CPU_VARIANT_stm32f103rdy6tr = 1
# TTC_CPU_VARIANT_stm32f103zet6   = 1
# TTC_CPU_VARIANT_stm32f105rct6   = 1
# TTC_CPU_VARIANT_stm32f107vct6   = 1
# TTC_CPU_VARIANT_stm32f302r8t6   = 1
# TTC_CPU_VARIANT_stm32l053r8t6   = 1
# TTC_CPU_VARIANT_stm32l100rbt6   = 1
# TTC_CPU_VARIANT_stm32l100rct6   = 1
# TTC_CPU_VARIANT_stm32l152rbt6   = 1
# TTC_CPU_VARIANT_stm32w108ccu    = 1

#{ Define amount of usable physical device units for this board
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_SPI=3
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_USART=5
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_ETH=0
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_I2C=2
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_I2S=2
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_CAN=1
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_SDIO=1
  COMPILE_OPTS += -DTTC_CPU_AMOUNT_USB=0
#}
#{ TTC_LIGHT               _PIN- LIGHTs and similar signal lights

COMPILE_OPTS += -DTTC_LIGHT1_GPIO1=E_ttc_gpio_pin_c13
COMPILE_OPTS += -DTTC_LIGHT1_LOWACTIVE=1          # ==1: gpio pin must be pulled low (cleared) to activate LED; ==0: pushed high (set)
COMPILE_OPTS += -DTTC_LIGHT1=E_ttc_light_architecture_gpio

#}
#{ USART1
  COMPILE_OPTS += -DTTC_USART1=INDEX_USART1
  COMPILE_OPTS += -DTTC_USART1_TX=E_ttc_gpio_pin_b6   
  COMPILE_OPTS += -DTTC_USART1_RX=E_ttc_gpio_pin_b7   
  COMPILE_OPTS += -DTTC_USART1_RTS=E_ttc_gpio_pin_a12  
  COMPILE_OPTS += -DTTC_USART1_CTS=E_ttc_gpio_pin_a11 
  COMPILE_OPTS += -DTTC_USART1_CK=E_ttc_gpio_pin_a8
#}
#{ USART2
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART2
  COMPILE_OPTS += -DTTC_USART2_TX=E_ttc_gpio_pin_a2  
  COMPILE_OPTS += -DTTC_USART2_RX=E_ttc_gpio_pin_a3   
  COMPILE_OPTS += -DTTC_USART2_RTS=E_ttc_gpio_pin_a1  
  COMPILE_OPTS += -DTTC_USART2_CTS=E_ttc_gpio_pin_a0 
  COMPILE_OPTS += -DTTC_USART2_CK=E_ttc_gpio_pin_a4 
#}
#{ USART3
  COMPILE_OPTS += -DTTC_USART3=INDEX_USART3
  COMPILE_OPTS += -DTTC_USART3_TX=E_ttc_gpio_pin_b10  
  COMPILE_OPTS += -DTTC_USART3_RX=E_ttc_gpio_pin_b11   
  COMPILE_OPTS += -DTTC_USART3_RTS=E_ttc_gpio_pin_b14  
  COMPILE_OPTS += -DTTC_USART3_CTS=E_ttc_gpio_pin_b13 
  COMPILE_OPTS += -DTTC_USART3_CK=E_ttc_gpio_pin_b12 
#}

#{ SPI1 - define serial peripheral interface #1
  COMPILE_OPTS += -DTTC_SPI1=ttc_device_1          # 
  COMPILE_OPTS += -DTTC_SPI1_MOSI=E_ttc_gpio_pin_a12  # P2-19
  COMPILE_OPTS += -DTTC_SPI1_MISO=E_ttc_gpio_pin_a11  # P2-20
  COMPILE_OPTS += -DTTC_SPI1_SCK=E_ttc_gpio_pin_b3   # P2-11
  COMPILE_OPTS += -DTTC_SPI1_NSS=E_ttc_gpio_pin_a15   # P2-16
#}
#{ SPI2 - define serial peripheral interface #2
  COMPILE_OPTS += -DTTC_SPI2=ttc_device_2           # 
  COMPILE_OPTS += -DTTC_SPI2_MOSI=E_ttc_gpio_pin_b15  # P1-27
  COMPILE_OPTS += -DTTC_SPI2_MISO=E_ttc_gpio_pin_b14  # P1-26
  COMPILE_OPTS += -DTTC_SPI2_SCK=E_ttc_gpio_pin_b13   # P1-25
  COMPILE_OPTS += -DTTC_SPI2_NSS=E_ttc_gpio_pin_b12   # P1-24
#}

COMPILE_OPTS += -DTTC_I2C_AMOUNT=2
#{  I2C1    - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=ttc_device_1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=E_ttc_gpio_pin_b7    # P2-7
  COMPILE_OPTS += -DTTC_I2C1_SCL=E_ttc_gpio_pin_b6    # P2-8
  COMPILE_OPTS += -DTTC_I2C1_SMBAL=E_ttc_gpio_pin_b5  # P2-9
#}I2C1
#{  I2C2    - define inter-integrated circuit interface #2
  COMPILE_OPTS += -DTTC_I2C2=ttc_device_2           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C2_SDA=E_ttc_gpio_pin_b11   # P1-23
  COMPILE_OPTS += -DTTC_I2C2_SCL=E_ttc_gpio_pin_b10   # P1-22
  COMPILE_OPTS += -DTTC_I2C2_SMBAL=E_ttc_gpio_pin_b12 # P1-24
#}I2C2

#{ CAN
#}

#{ DMA
  COMPILE_OPTS += -DTTC_DMA1=ttc_device_1
  COMPILE_OPTS += -DTTC_NCHANNELS=TTC_NUMBER_OF_CHANNELS 
#}

#{ USB
#}

END_OF_MAKEFILE
    createExtensionMakefileTail ${DriverName} #}
    createActivateScriptHead $DriverName $DirLL_Extensions/ "$ScriptName 2" "minimalistic header board for stm32f103c8t6" #{ (create activate script)
    cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/
#   \$Dir_Configs           -> configs/

# Low-Level driver has to check if required architecture driver has been activated before.
# This is done by checking existence of a corresponding makefile inside \$Dir_Extensions/.
#
checkFeature "${DriverName}" "$cASH_InfoText"
if [ "\$?" == "1" ]; then #{ architecture for this driver has been activated: activate low-level driver 
  #{ ACTIVATE_SECTION_A remove activated variants of same type
  
  rm 2>/dev/null \$Dir_ExtensionsActive/*${Install_Dir}_*
  
  #}
  #  ACTIVATE_SECTION_B no low-level drivers available
  #{ ACTIVATE_SECTION_C create link to your makefile into extensions.active/ to add makefile to your project
  
  createLink \$Dir_Extensions/makefile.$DriverName \$Dir_ExtensionsActive/makefile.$DriverName '' QUIET
  
  #}
  #{ ACTIVATE_SECTION_C2: create symbolic link from extension folder into every PROJECT_FOLDER/additionals/ to provide extra source code and headers
    # Activate line below and change <SUBFOLDER> to your needs
    Link="additionals/${Install_Dir}_stm32f103c8t6_blue_pill"
    #addLine createLinks.sh "rm 2>/dev/null \$Link; ln -sv \\\$HOME/Source/TheToolChain/InstallData/${Install_Dir}/<SUBFOLDER> \$Link"
  #}
  #{ ACTIVATE_SECTION_D activate initialization source-code for this extension 

  # board_stm32f103c8t6_blue_pill_prepare() typically ist called ttc_board_prepare() during preparation of all extensions.
  # It should not be required to activate source.${RANK_LOWLEVEL}_*.c here.

  #X if [ -e \$Dir_Extensions/source.${DriverName}.c ]; then
  #X   createLink \$Dir_Extensions/source.${DriverName}.c \$Dir_ExtensionsActive/ '' QUIET
  #X fi

  #}
  #{ ACTIVATE_SECTION_E enable features provided by this extension
  #
  # Most low-level drivers activate only if a certain file exists inside extensions.active/.
  # This can be makefile.* or feature.* files. Feature files are empty files that simply allow to activate 
  # a corresponding extension.
  # The name of the feature file shall match the name of its corresponding extension.
  # E.g.: feature.${RANK_LOWLEVEL}_ethernet_ste101p -> allows activate.500_ethernet.sh to enable its low-level driver ${RANK_LOWLEVEL}_ethernet_ste101p.
  # enableFeature NAME  # creates extensions.active/feature.NAME
  # See activate_project.sh of a fresh created project for a list of all available features! 
  
  # enableFeature 450_ethernet_ste101p
  enableFeature 450_cpu_stm32f1xx

  #}
  #{ ACTIVATE_SECTION_F call other activate-scripts
  
  # activate.500_ttc_memory.sh QUIET "\$ScriptName"
  
  #}
fi #}

END_OF_ACTIVATE
    createActivateScriptTail $DriverName $DirLL_Extensions/
  #}
  
  fi #}low-level driver
  if [ "1" == "0" ]; then #{ optional: create symbolic link from extension folder into every PROJECT_FOLDER/additionals/ to provide extra source code and headers
    # Activate line below and change SUBFOLDER to your needs
    # addLine ../../scripts/createLinks.sh "rm 2>/dev/null $Install_Dir; ln -sv \$Source/TheToolChain/InstallData/${Install_Dir}/SUBFOLDER $Install_Dir"
    echo ""
  fi #} 
  if [ "1" == "0" ]; then #{ optional: create regression test for your extension
  
    Name="650_${EXTENSION_NAME}_regression_${ThisArchitecture}"  
    createExtensionSourcefileHead ${Name}    #{ create initializer code for your extension
  
    # Below you may write C-code that will automatically being placed inside extensions.active/ttc_extensions_active.c during _/compile.sh
    # Every #include directive will automatically being added to extensions.active/ttc_extensions_active.h
    #
    cat <<END_OF_SOURCEFILE >>$ExtensionSourceFile

#include "${Name}.h"
${Name}_prepare();

END_OF_SOURCEFILE
    createExtensionSourcefileTail ${Name} #}    
    createExtensionMakefileHead ${Name}      #{ create makefile
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead() 

# Every regression test should set this to enable extra safety checks
COMPILE_OPTS += -DTTC_REGRESSION

# additional constant defines
#COMPILE_OPTS += -D

# additional directories to search for header files
#INCLUDE_DIRS += -I additionals/$Name/
#INCLUDE_DIRS += -I

# additional directories to search for C-Sources
#vpath %.c 

# additional object files to be compiled
MAIN_OBJS += regression_${EXTENSION_NAME}.o

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    createActivateScriptHead $Name $DirLL_Extensions/ "$ScriptName 2" "PLACE YOUR INFO HERE FOR $ScriptName" #{ create activate script
    cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

#{ ACTIVATE_SECTION_A remove activated variants of same type

# rm 2>/dev/null \$Dir_ExtensionsActive/*${Install_Dir}_*

#}
#  ACTIVATE_SECTION_B no low-level drivers available
#{ ACTIVATE_SECTION_C create link to your makefile into extensions.active/ to add makefile to your project

createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

#}
#{ ACTIVATE_SECTION_D activate initialization source-code for this extension 

  if [ -e \$Dir_Extensions/source.${Name}.c ]; then
    createLink \$Dir_Extensions/source.${Name}.c \$Dir_ExtensionsActive/ '' QUIET
  fi

#}
#{ ACTIVATE_SECTION_E provide default features required to compile this example

  # Read Documentation/TheToolChain-Manual/chapter_Extensions.tml for a description of feature enabling.  

  # enableFeature 450_math_software_float   # enable math driver (single precision)
  # enableFeature 450_math_software_double  # enable math driver (double precision)

#}
#{ ACTIVATE_SECTION_F call other activate-scripts

#activate.500_ttc_gpio.sh QUIET "\$ScriptName"

#}
#{ ACTIVATE_SECTION_G copy source code into project folder if required

if [ ! -d regressions ]; then
  mkdir regressions
fi
ExampleFound=\`ls regressions/regression_${EXTENSION_NAME}_${ThisArchitecture}*\`
if [ "\$ExampleFound" == "" ]; then
  echo "copying missing regression_${EXTENSION_NAME}_${ThisArchitecture}*.."
  cp -v ~/Source/TheToolChain/Template/regressions/regression_${EXTENSION_NAME}_${ThisArchitecture}* regressions/
fi
#}

END_OF_ACTIVATE
    createActivateScriptTail $Name $DirLL_Extensions/
    #}

  fi #}
  if [ "1" == "0" ]; then #{ recommended: create usage example for your extension
  
    Name="600_example_${EXTENSION_NAME}_${ThisArchitecture}_${ThisArchitecture}"  
    createExtensionSourcefileHead ${Name}    #{ create initializer code for your extension
  
    # Below you may write C-code that will automatically being placed inside extensions.active/ttc_extensions_active.c during _/compile.sh
    # Every #include directive will automatically being added to extensions.active/ttc_extensions_active.h
    #
    cat <<END_OF_SOURCEFILE >>$ExtensionSourceFile

#include "examples/example_${EXTENSION_NAME}_${ThisArchitecture}.h"
example_${EXTENSION_NAME}_${ThisArchitecture}_prepare();

END_OF_SOURCEFILE
    createExtensionSourcefileTail ${Name} #}    
    createExtensionMakefileHead ${Name}      #{ create makefile
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead()

# additional constant defines
#COMPILE_OPTS += -D

# define devices to use in this example
COMPILE_OPTS += -DTTC_BOARD1=E_ttc_board_architecture_stm32f103c8t6_blue_pill # Line generated automatically. Remove if not wanted!

# additional directories to search for header files
#INCLUDE_DIRS += -I additionals/$Name/

# additional directories to search for C-Sources
#vpath %.c 

# additional object files to be compiled
MAIN_OBJS += example_${EXTENSION_NAME}_${ThisArchitecture}.o

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    createActivateScriptHead $Name $DirLL_Extensions/ "$ScriptName 2" "Usage example for extension ${EXTENSION_NAME}_${ThisArchitecture}" #{ create activate script
    cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

#{ ACTIVATE_SECTION_A remove activated variants of same type

# rm 2>/dev/null \$Dir_ExtensionsActive/*${Install_Dir}_*

#}
#  ACTIVATE_SECTION_B no low-level drivers available
#{ ACTIVATE_SECTION_C create link to your makefile into extensions.active/ to add makefile to your project

createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

#}
#{ ACTIVATE_SECTION_D activate initialization source-code for this extension 

if [ -e \$Dir_Extensions/source.${Name}.c ]; then
  createLink \$Dir_Extensions/source.${Name}.c \$Dir_ExtensionsActive/ '' QUIET
fi

#}
#{ ACTIVATE_SECTION_E provide default features required to compile this example

  # Read Documentation/TheToolChain-Manual/chapter_Extensions.tml for a description of feature enabling.
  enableFeature $DriverName  # enable low-level driver to be tested

  # enableFeature 450_math_software_float   # enable math driver (single precision)
  # enableFeature 450_math_software_double  # enable math driver (double precision)

#}
#{ ACTIVATE_SECTION_F call other activate-scripts

activate.${Install_Dir}.sh QUIET "\$ScriptName" # activate high-level driver

#}
#{ ACTIVATE_SECTION_G copy source code into project folder if required

if [ ! -d examples ]; then
  mkdir examples
fi
ExampleFound=\`ls examples/example_${EXTENSION_NAME}_${ThisArchitecture}*\`
if [ "\$ExampleFound" == "" ]; then
  echo "copying missing example_${EXTENSION_NAME}.."
  cp -v ~/Source/TheToolChain/Template/examples/example_${EXTENSION_NAME}_${ThisArchitecture}* examples/
fi
#}

END_OF_ACTIVATE
    createActivateScriptTail $Name $DirLL_Extensions/
    #}

  fi #}

  cd "$OldPWD"
  echo "Installed successfully: $DriverName"
#}
else                               #{ Install failed
  #echo "failed to install $DriverName"
  return 10
fi #}

cd "$MyStartDir"
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------
