/** { memory_basic.ld **********************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Architecture independend memory section definitions.
 *
 *  This file defines all memory sections being required by TheToolChain.
 *  If your project requires additional sections, you may copy this script
 *  as a template into your project specific linker script configs/memory_project.ld.
 *
 */

/* define start + end of RAM area */
_ttc_ram_start = ORIGIN(ram);
_ttc_ram_end = ORIGIN(ram) + LENGTH(ram) - 1;

/* Define end of stack as end of ram region */
__Stack_Size = _ttc_size_stack;
_estack = _ttc_ram_end;
__Stack_Init = _estack - __Stack_Size;
_Stack_Init = __Stack_Init; /*"PROVIDE" allows to easily override these values from an object file or the commmand line.*/

/* define end of memory heap before start of stack */
_ttc_heap_end = __Stack_Init - 1;

/* define first valid address of main stack */
_ttc_memory_stack_start = _estack;

/* define last valid address of main stack */
_ttc_memory_stack_end   = __Stack_Init;


/* define memory sections being required by gcc and dynamic memory (heap) */
SECTIONS
{
     .data : AT(__sidata)
     {
     . = ALIGN(4);
         __sdata = .;
         *(.data)        /* Data memory */
         *(.data.*)
         . = ALIGN(4);
         __edata = .;
     } >ram

     /* Section for variables mapped onto registers */
     .peripherals (OVERLAY) :
     {
        . = ALIGN(4);
        *(.peripherals)
     } >peripheral

     .private_peripherals (OVERLAY) :
     {
        . = ALIGN(4);
        *(.private_peripherals)
     } >private_peripheral


    .isr_vector :
    {
        . = ALIGN(4);
        KEEP(*(.isr_vector))
        . = ALIGN(4);
    } >rom
    
    .text : 
    {
        . = ALIGN(4);
        *(.text .text.*)        /* Program code */
        *(.rodata)              /* Read only data */
        *(.rodata.*)
        *(.glue_7)
        *(.glue_7t)
        . = ALIGN(4);
        
        _etext = .;
        __sidata = _etext;
    } >rom


    .bss : 
    {
        . = ALIGN(4);
        __sbss = .;
        *(.bss)         /* Zero-filled run time allocate data memory */
        *(.bss.*)
        . = ALIGN(4);
        __ebss = .;
    } >ram

    .heap :
    {
        . = ALIGN(4);
        .heap = .;
        _ttc_heap_start = .;
        . = ALIGN(4);
    } >ram
}

_sdata  = __sdata;
_edata  = __edata;
_sidata = __sidata;
_sbss   = __sbss;
_ebss   = __ebss;
