/**  memory_stm32w1xx.ld **********************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Architecture dependend memory layout for stm32w1xx.
 *
 */
 
/* Size of IAP bootloader (ToDo: Required?) */
_BOOTLOADER_SIZE = 0x3000 ;

/* This value needs to be fine tuned if the NVM section grows */
_NVM_size = 0x800;

/* memory.ld will define memory spaces according to definitions in board-makefiles */
INCLUDE configs/memory_current.ld
/*INCLUDE ttc-lib/_linker/memory_cortexm3.ld*/

/* moving register variables to fixed addresses
 *
 * Compare with registers_stm32w1xx.c
 */

/* STM32W1xx registers as defined in EM357_Manual.pdf */
register_stm32w1xx_Peripherals         = 0x40000000;
register_stm32w1xx_OSC24M_CTRL         = 0x4000401C;
register_stm32w1xx_PERIPHERAL_DISABLE  = 0x40004038;
register_stm32w1xx_TIM1_INT            = 0x4000A800;
register_stm32w1xx_TIM2_INT            = 0x4000A804;
register_stm32w1xx_INT_SC1FLAG         = 0x4000a808;
register_stm32w1xx_INT_SC2FLAG         = 0x4000a80c;
register_stm32w1xx_INT_ADCFLAG         = 0x4000A810;
register_stm32w1xx_INT_GPIOFLAG        = 0x4000a814;
register_stm32w1xx_INT_TIM1MISS        = 0x4000A818;
register_stm32w1xx_INT_TIM2MISS        = 0x4000A81C;
register_stm32w1xx_INT_MISS            = 0x4000a820;
register_stm32w1xx_INT_TIM1CFG         = 0x4000A840;
register_stm32w1xx_INT_TIM2CFG         = 0x4000A844;
register_stm32w1xx_INT_SC1CFG          = 0x4000a848;
register_stm32w1xx_INT_SC2CFG          = 0x4000a84c;
register_stm32w1xx_INT_ADCCFG          = 0x4000A850;
register_stm32w1xx_SC1_INTMODE         = 0x4000a854;
register_stm32w1xx_SC2_INTMODE         = 0x4000a858;
register_stm32w1xx_GPIO_INT_CFG        = 0x4000a860;
register_stm32w1xx_GPIOA               = 0x4000b000;
register_stm32w1xx_GPIOA_CFGL          = 0x4000b000;
register_stm32w1xx_GPIOA_CFGH          = 0x4000b004;
register_stm32w1xx_GPIOA_IN            = 0x4000b008;
register_stm32w1xx_GPIOA_OUT           = 0x4000b00c;
register_stm32w1xx_GPIOA_SET           = 0x4000b010;
register_stm32w1xx_GPIOA_CLR           = 0x4000b014;
register_stm32w1xx_GPIOB               = 0x4000b400;
register_stm32w1xx_GPIOOB_CFGL         = 0x4000b400;
register_stm32w1xx_GPIOOB_CFGH         = 0x4000b404;
register_stm32w1xx_GPIOOB_IN           = 0x4000b408;
register_stm32w1xx_GPIOOB_OUT          = 0x4000b40c;
register_stm32w1xx_GPIOOB_SET          = 0x4000b410;
register_stm32w1xx_GPIOOB_CLR          = 0x4000b414;
register_stm32w1xx_GPIOC               = 0x4000b800;
register_stm32w1xx_GPIOC_CFGL          = 0x4000b800;
register_stm32w1xx_GPIOC_CFGH          = 0x4000b804;
register_stm32w1xx_GPIOC_IN            = 0x4000b808;
register_stm32w1xx_GPIOC_OUT           = 0x4000b80c;
register_stm32w1xx_GPIOC_SET           = 0x4000b810;
register_stm32w1xx_GPIOC_CLR           = 0x4000b814;
register_stm32w1xx_GPIO_DB_WAKE_IRQ    = 0x4000bc00;
register_stm32w1xx_GPIO_DBGCFG         = 0x4000bc00;
register_stm32w1xx_GPIO_DBGSTAT        = 0x4000bc04;
register_stm32w1xx_GPIO_AWAKE          = 0x4000bc08;
register_stm32w1xx_GPIO_BWAKE          = 0x4000bc0c;
register_stm32w1xx_GPIO_CWAKE          = 0x4000bc10;
register_stm32w1xx_GPIO_IRQCSEL        = 0x4000bc14;
register_stm32w1xx_GPIO_IRQDSEL        = 0x4000bc18;
register_stm32w1xx_GPIO_WAKEFILT       = 0x4000bc1c;
register_stm32w1xx_SC2                 = 0x4000c000;
register_stm32w1xx_SC2_RXBEGA          = 0x4000c000;
register_stm32w1xx_SC2_RXENDA          = 0x4000c004;
register_stm32w1xx_SC2_RXBEGB          = 0x4000c008;
register_stm32w1xx_SC2_RXENDB          = 0x4000c00c;
register_stm32w1xx_SC2_TXBEGA          = 0x4000c010;
register_stm32w1xx_SC2_TXENDA          = 0x4000c014;
register_stm32w1xx_SC2_TXBEGB          = 0x4000c018;
register_stm32w1xx_SC2_TXENDB          = 0x4000c01c;
register_stm32w1xx_SC2_RXCNTA          = 0x4000c020;
register_stm32w1xx_SC2_RXCNTB          = 0x4000c024;
register_stm32w1xx_SC2_TXCNT           = 0x4000c028;
register_stm32w1xx_SC2_DMASTAT         = 0x4000c02c;
register_stm32w1xx_SC2_DMACTRL         = 0x4000c030;
register_stm32w1xx_SC2_RXERRA          = 0x4000c034;
register_stm32w1xx_SC2_RXERRB          = 0x4000c038;
register_stm32w1xx_SC2_DATA            = 0x4000c03c;
register_stm32w1xx_SC2_SPISTAT         = 0x4000c040;
register_stm32w1xx_SC2_TWISTAT         = 0x4000c044;
register_stm32w1xx_SC2_UARTSTAT        = 0x4000c048;
register_stm32w1xx_SC2_TWICTRL1        = 0x4000c04c;
register_stm32w1xx_SC2_TWICTRL2        = 0x4000c050;
register_stm32w1xx_SC2_MODE            = 0x4000c054;
register_stm32w1xx_SC2_SPICFG          = 0x4000c058;
register_stm32w1xx_SC2_UARTCFG         = 0x4000c05c;
register_stm32w1xx_SC2_RATELIN         = 0x4000c060;
register_stm32w1xx_SC2_RATEEXP         = 0x4000c064;
register_stm32w1xx_SC2_UARTPER         = 0x4000c068;
register_stm32w1xx_SC2_UARTFRAC        = 0x4000c06c;
register_stm32w1xx_SC2_RXCNTSAVED      = 0x4000c070;
register_stm32w1xx_SC1                 = 0x4000c800;
register_stm32w1xx_SC1_RXBEGA          = 0x4000c800;
register_stm32w1xx_SC1_RXENDA          = 0x4000c804;
register_stm32w1xx_SC1_RXBEGB          = 0x4000c808;
register_stm32w1xx_SC1_RXENDB          = 0x4000c80c;
register_stm32w1xx_SC1_TXBEGA          = 0x4000c810;
register_stm32w1xx_SC1_TXENDA          = 0x4000c814;
register_stm32w1xx_SC1_TXBEGB          = 0x4000c818;
register_stm32w1xx_SC1_TXENDB          = 0x4000c81c;
register_stm32w1xx_SC1_RXCNTA          = 0x4000c820;
register_stm32w1xx_SC1_RXCNTB          = 0x4000c824;
register_stm32w1xx_SC1_TXCNT           = 0x4000c828;
register_stm32w1xx_SC1_DMASTAT         = 0x4000c82c;
register_stm32w1xx_SC1_DMACTRL         = 0x4000c830;
register_stm32w1xx_SC1_RXERRA          = 0x4000c834;
register_stm32w1xx_SC1_RXERRB          = 0x4000c838;
register_stm32w1xx_SC1_DATA            = 0x4000c83c;
register_stm32w1xx_SC1_SPISTAT         = 0x4000c840;
register_stm32w1xx_SC1_TWISTAT         = 0x4000c844;
register_stm32w1xx_SC1_UARTSTAT        = 0x4000c848;
register_stm32w1xx_SC1_TWICTRL1        = 0x4000c84c;
register_stm32w1xx_SC1_TWICTRL2        = 0x4000c850;
register_stm32w1xx_SC1_MODE            = 0x4000c854;
register_stm32w1xx_SC1_SPICFG          = 0x4000c858;
register_stm32w1xx_SC1_UARTCFG         = 0x4000c85c;
register_stm32w1xx_SC1_RATELIN         = 0x4000c860;
register_stm32w1xx_SC1_RATEEXP         = 0x4000c864;
register_stm32w1xx_SC1_UARTPER         = 0x4000c868;
register_stm32w1xx_SC1_UARTFRAC        = 0x4000c86c;
register_stm32w1xx_SC1_RXCNTSAVED      = 0x4000c870;

register_stm32w1xx_ADC_DATA            = 0x4000D000;
register_stm32w1xx_ADC_CFG             = 0x4000D004;
register_stm32w1xx_ADC_OFFSET          = 0x4000D008;
register_stm32w1xx_ADC_GAIN            = 0x4000D00C;
register_stm32w1xx_ADC_DMACFG          = 0x4000D010;
register_stm32w1xx_ADC_DMASTAT         = 0x4000D014;
register_stm32w1xx_ADC_DMABEG          = 0x4000D018;
register_stm32w1xx_ADC_DMASIZE         = 0x4000D01C;
register_stm32w1xx_ADC_DMACUR          = 0x4000D020;
register_stm32w1xx_ADC_DMACNT          = 0x4000D024;
register_stm32w1xx_TIM1_CR1            = 0x4000E000;
register_stm32w1xx_TIM1_CR2            = 0x4000E004;
register_stm32w1xx_TIM1_SMCR           = 0x4000E008;
register_stm32w1xx_TIM1_EGR            = 0x4000E014;
register_stm32w1xx_TIM1_CCMR1          = 0x4000E018;
register_stm32w1xx_TIM1_CCMR2          = 0x4000E01C;
register_stm32w1xx_TIM1_CCER           = 0x4000E020;
register_stm32w1xx_TIM1_CNT            = 0x4000E024;
register_stm32w1xx_TIM1_PSC            = 0x4000E028;
register_stm32w1xx_TIM1_ARR            = 0x4000E02C;
register_stm32w1xx_TIM1_CCR1           = 0x4000E034;
register_stm32w1xx_TIM1_CCR2           = 0x4000E038;
register_stm32w1xx_TIM1_CCR3           = 0x4000E03C;
register_stm32w1xx_TIM1_CCR4           = 0x4000E040;
register_stm32w1xx_TIM1_OR             = 0x4000E050;
register_stm32w1xx_TIM2_CR1            = 0x4000F000;
register_stm32w1xx_TIM2_CR2            = 0x4000F004;
register_stm32w1xx_TIM2_SMCR           = 0x4000F008;
register_stm32w1xx_TIM2_EGR            = 0x4000F014;
register_stm32w1xx_TIM2_CCMR1          = 0x4000F018;
register_stm32w1xx_TIM2_CCMR2          = 0x4000F01C;
register_stm32w1xx_TIM2_CCER           = 0x4000F020;
register_stm32w1xx_TIM2_CNT            = 0x4000F024;
register_stm32w1xx_TIM2_PSC            = 0x4000F028;
register_stm32w1xx_TIM2_ARR            = 0x4000F02C;
register_stm32w1xx_TIM2_CCR1           = 0x4000F034;
register_stm32w1xx_TIM2_CCR2           = 0x4000F038;
register_stm32w1xx_TIM2_CCR3           = 0x4000F03C;
register_stm32w1xx_TIM2_CCR4           = 0x4000F040;
register_stm32w1xx_TIM2_OR             = 0x4000F050;

register_stm32w1xx_Private_Peripherals = 0xe0000000;
register_stm32w1xx_INT_CFGSET          = 0xe000e100;
register_stm32w1xx_INT_CFGCLR          = 0xe000e180;
register_stm32w1xx_INT_PENDSET         = 0xe000e200;
register_stm32w1xx_INT_PENDCLR         = 0xe000e280;
register_stm32w1xx_INT_ACTIVE          = 0xe000e300;
register_stm32w1xx_BUS_AFSR            = 0xe000ed3c;

/* registers as defined in Ember AN710 */
register_stm32w1xx_TOKEN_MFG_CIB_OBS             = 0x08040800;
register_stm32w1xx_TOKEN_MFG_CUSTOM_VERSION      = 0x08040810;
register_stm32w1xx_TOKEN_MFG_CUSTOM_EUI_64       = 0x08040812;
register_stm32w1xx_TOKEN_MFG_STRING              = 0x0804081A;
register_stm32w1xx_TOKEN_MFG_BOARD_NAME          = 0x0804082A;
register_stm32w1xx_TOKEN_MFG_MANUF_ID            = 0x0804083A;
register_stm32w1xx_TOKEN_MFG_PHY_CONFIG          = 0x0804083C;
register_stm32w1xx_TOKEN_MFG_BOOTLOAD_AES_KEY    = 0x0804083E;
register_stm32w1xx_TOKEN_MFG_EZSP_STORAGE        = 0x0804084E;
register_stm32w1xx_TOKEN_MFG_CBKE_DATA           = 0x0804087E;
register_stm32w1xx_TOKEN_MFG_INSTALLATION_CODE   = 0x080408DA;
register_stm32w1xx_TOKEN_MFG_OSC24M_BIAS_TRIM    = 0x080408EE;
register_stm32w1xx_TOKEN_MFG_SYNTH_FREQ_OFFSET   = 0x080408F0;
register_stm32w1xx_TOKEN_MFG_OSC24M_SETTLE_DELAY = 0x080408F2;
register_stm32w1xx_TOKEN_MFG_SECURITY_CONFIG     = 0x080408F4;
register_stm32w1xx_TOKEN_MFG_CCA_THRESHOLD       = 0x080408F6;

/* DEPRECATED
__Stack_Size = 0x400;

PROVIDE ( _Stack_Size = __Stack_Size ) ;

__Stack_Init = 0x20000000;
_estack = __Stack_Init + __Stack_Size;
*/


/* define start + end of RAM area */
PROVIDE ( _ttc_ram_start = ORIGIN(ram) );
PROVIDE ( _ttc_ram_end = ORIGIN(ram) + LENGTH(ram) - 1 );

/* Define end of stack as end of ram region */
__Stack_Size = _ttc_size_stack;
_estack = _ttc_ram_end;
__Stack_Init = _estack - __Stack_Size;
PROVIDE ( _Stack_Init = __Stack_Init ); /*"PROVIDE" allows to easily override these values from an object file or the commmand line.*/

/* define end of memory heap before start of stack */
PROVIDE ( _ttc_heap_end = __Stack_Init - 1 );

/* define first valid address of main stack */
PROVIDE ( _ttc_memory_stack_start = _estack );

/* define last valid address of main stack */
PROVIDE ( _ttc_memory_stack_end   = __Stack_Init );

/* Sections management for FLASH mode */
/* Sections Definitions */

SECTIONS
{
    /* Section for variables mapped onto registers */
    .peripherals (OVERLAY) :
    {
       . = ALIGN(4);
       *(.peripherals)
    } >peripheral

    .private_peripherals (OVERLAY) :
    {
       . = ALIGN(4);
       *(.private_peripherals)
    } >private_peripheral

    /* for Cortex devices, the beginning of the startup code is stored in the .isr_vector section, which goes to FLASH */
    .isr_vector (DEFINED(ST_BTL) ? (0x08000000 + _BOOTLOADER_SIZE) : 0x08000000) :
    {
	. = ALIGN(4);
	__ApplicationFlashStart = . ;
	/*	. = . + _ROM_base ;*/
        KEEP(*(.isr_vector))            /* Startup code */
	. = ALIGN(4);
    } >rom
 
    /* for some STRx devices, the beginning of the startup code is stored in the .flashtext section, which goes to FLASH */
    .flashtext :
    {
	. = ALIGN(4);
        *(.flashtext)            /* Startup code */
	. = ALIGN(4);
    } >rom
 
    /* the program code is stored in the .text section, which goes to Flash */
    .text :
    {
        . = ALIGN(4);
	    
        *(.text)                   /* remaining code */
        *(.text.*)                   /* remaining code */
        *(.rodata)                 /* read-only data (constants) */
        *(.rodata*)
        *(.glue_7)
        *(.glue_7t)

         . = ALIGN(4);
   	 _etext = .;

         /* This is used by the startup in order to initialize the .data secion */
   	 _sidata = _etext;
    } >rom
    /*
    NVM (NOLOAD):
    {
      . = ALIGN(1024);
      *(NVM)
      . = ALIGN(4);
    } > NVM_region
    */
    /* This is to emulate place at end of IAR linker */
    NVM (ORIGIN(rom) + LENGTH(rom) - _NVM_size) (NOLOAD):
    {
     *(NVM)
    } > rom
    FAT (NOLOAD) :
    {
      . = ALIGN(4);
      KEEP(*(FAT))
      . = ALIGN(4);
    } > fib

/*
    .FAT (NOLOAD):
    {
      KEEP(*(.FAT))
    } > fib
*/
    /* after that it's only debugging information. */
    

    /* This is the initialized data section
    The program executes knowing that the data is in the RAM
    but the loader puts the initial values in the FLASH (inidata).
    It is one task of the startup to copy the initial values from FLASH to RAM. */
    .data  : AT ( _sidata )
    {
	    . = ALIGN(4);
        /* This is used by the startup in order to initialize the .data secion */
        _sdata = . ;
        
        *(.data)
        *(.data.*)

	    . = ALIGN(4);
	    /* This is used by the startup in order to initialize the .data secion */
   	 _edata = . ;
    } >ram
    
    

    /* This is the uninitialized data section */
    .bss :
    {
        . = ALIGN(4);

        /* This is used by the startup in order to initialize the .bss secion */
        _sbss = .;
        
        *(.bss)
        *(.bss.*)
        *(COMMON)
        
        . = ALIGN(4);

         /* This is used by the startup in order to initialize the .bss secion */
   	 _ebss = . ;
    } >ram
    
    PROVIDE ( end = _ebss );
    PROVIDE ( _end = _ebss );
       
    .heap :
    {
        . = ALIGN(4);
        .heap = .;
        __ttc_heap_start = .;
        . = ALIGN(4);
    } >ram

    __exidx_start = .;
    __exidx_end = .;

    /* remove the debugging information from the standard libraries */
    /DISCARD/ :
    {
     libc.a ( * )
     libm.a ( * )
     libgcc.a ( * )
     }

    /* Stabs debugging sections.  */
    .stab          0 : { *(.stab) }
    .stabstr       0 : { *(.stabstr) }
    .stab.excl     0 : { *(.stab.excl) }
    .stab.exclstr  0 : { *(.stab.exclstr) }
    .stab.index    0 : { *(.stab.index) }
    .stab.indexstr 0 : { *(.stab.indexstr) }
    .comment       0 : { *(.comment) }
    /* DWARF debug sections.
       Symbols in the DWARF debugging sections are relative to the beginning
       of the section so we begin them at 0.  */
    /* DWARF 1 */
    .debug          0 : { *(.debug) }
    .line           0 : { *(.line) }
    /* GNU DWARF 1 extensions */
    .debug_srcinfo  0 : { *(.debug_srcinfo) }
    .debug_sfnames  0 : { *(.debug_sfnames) }
    /* DWARF 1.1 and DWARF 2 */
    .debug_aranges  0 : { *(.debug_aranges) }
    .debug_pubnames 0 : { *(.debug_pubnames) }
    /* DWARF 2 */
    .debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
    .debug_abbrev   0 : { *(.debug_abbrev) }
    .debug_line     0 : { *(.debug_line) }
    .debug_frame    0 : { *(.debug_frame) }
    .debug_str      0 : { *(.debug_str) }
    .debug_loc      0 : { *(.debug_loc) }
    .debug_macinfo  0 : { *(.debug_macinfo) }
    /* SGI/MIPS DWARF 2 extensions */
    .debug_weaknames 0 : { *(.debug_weaknames) }
    .debug_funcnames 0 : { *(.debug_funcnames) }
    .debug_typenames 0 : { *(.debug_typenames) }
    .debug_varnames  0 : { *(.debug_varnames) }
}

__ApplicationFlashEnd = _sidata + (_edata - _sdata);
__RAMStart = __Stack_Init;
__RAMEnd = _ebss;

PROVIDE( _ttc_heap_start = __ttc_heap_start );
