#ifndef CPU_STM32F1XX_TYPES_H
#define CPU_STM32F1XX_TYPES_H

/** { cpu_stm32f1xx.h ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level datatypes for CPU devices on stm32f1xx architectures.
 *  Structures, Enums and Defines being required by ttc_cpu_types.h
 *
 *  Created from template device_architecture_types.h revision 21 at 20150318 12:33:43 UTC
 *
 *  Note: See ttc_cpu.h for description of architecture independent CPU implementation.
 *
 *  Authors: Gregor Rebel
 *
}*/

//{ Defines/ TypeDefs **********************************************************

/* Excerpt from stm32f10x.h about density definition

  Exactky one of the lines below must be defined before including stm32f10x.h.
  Select the right one for your CPU variant and copy it into each "#if (TTC_CPU_VARIANT ==..." block!

    #define STM32F10X_LD      // STM32F10X_LD: STM32 Low density devices
    #define STM32F10X_LD_VL   // STM32F10X_LD_VL: STM32 Low density Value Line devices
    #define STM32F10X_MD      // STM32F10X_MD: STM32 Medium density devices
    #define STM32F10X_MD_VL   // STM32F10X_MD_VL: STM32 Medium density Value Line devices
    #define STM32F10X_HD      // STM32F10X_HD: STM32 High density devices
    #define STM32F10X_HD_VL   // STM32F10X_HD_VL: STM32 High density value line devices
    #define STM32F10X_XL      // STM32F10X_XL: STM32 XL-density devices
    #define STM32F10X_CL      // STM32F10X_CL: STM32 Connectivity line devices

 - Low-density devices are STM32F101xx, STM32F102xx and STM32F103xx microcontrollers
   where the Flash memory density ranges between 16 and 32 Kbytes.
 - Low-density value line devices are STM32F100xx microcontrollers where the Flash
   memory density ranges between 16 and 32 Kbytes.
 - Medium-density devices are STM32F101xx, STM32F102xx and STM32F103xx microcontrollers
   where the Flash memory density ranges between 64 and 128 Kbytes.
 - Medium-density value line devices are STM32F100xx microcontrollers where the
   Flash memory density ranges between 64 and 128 Kbytes.
 - High-density devices are STM32F101xx and STM32F103xx microcontrollers where
   the Flash memory density ranges between 256 and 512 Kbytes.
 - High-density value line devices are STM32F100xx microcontrollers where the
   Flash memory density ranges between 256 and 512 Kbytes.
 - XL-density devices are STM32F101xx and STM32F103xx microcontrollers where
   the Flash memory density ranges between 512 and 1024 Kbytes.
 - Connectivity line devices are STM32F105xx and STM32F107xx microcontrollers.
  */

// these are defined in CPU makefile
#  define TTC_CPU_SIZE_RAM     TTC_MEMORY_REGION_RAM_SIZEK    * 1024
#  define TTC_CPU_SIZE_FLASH   TTC_MEMORY_REGION_ROM_SIZEK    * 1024
#  define TTC_CPU_SIZE_EEPROM  0

// identify CPU architecture and its features from given variant



//InsertDefines above (DO NOT DELETE THIS LINE!)

//InsertEnums above (DO NOT DELETE THIS LINE!)

//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Includes *******************************************************************

#include "../ttc_basic_types.h"
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Structures/ Enums required by ttc_cpu_types.h *************************


typedef struct {  // stm32f1xx specific configuration data of single CPU device
    t_u8 Unused;
} __attribute__( ( __packed__ ) ) t_cpu_stm32f1xx_config;

// t_ttc_cpu_architecture is required by ttc_cpu_types.h
#define t_ttc_cpu_architecture t_cpu_stm32f1xx_config

//} Structures/ Enums


#endif //CPU_STM32F1XX_TYPES_H
