/** { ttc_light_interface.c ************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for LED device.
 *
 *  The functions implemented here are used when additional runtime
 *  computations arerequired to call the corresponding low-level driver.
 *  These computations should be held very simple and fast. Every additional
 *
 *  Adding a new low-level implementation may require changes in this file.
 *
 *  Functions in this file may not call ttc_light_*() functions.
 *  Instead they can call _driver_light_*() pendants.
 *  E.g.: ttc_light_reset() -> _driver_light_reset()
 *
 *  Created from template ttc_device_interface.c revision 25 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

//{ Includes *************************************************************
//
// C-Sources include their corresponding header file and all other header files
// that are required to compile this source file.
// Includes that provide datatypes being used by function prototypes belong into
// "ttc_light_interface.h".
//
#include "ttc_light_interface.h"
#include "../ttc_task.h"             // provides macro TTC_RETURN()
//InsertIncludes above (DO NOT DELETE THIS LINE!)

// interface will include low-driver for us. Check if at least one low-level driver could be activated
#ifndef EXTENSION_LED_DRIVER_AVAILABLE
    #  error Missing low-level driver for TTC_<DRIVER>! Did you forget to provide a feature? Check your activation stage: ./clean.sh ; ./activate_project.sh 2>&1 | grep _light_
#endif

//}Includes
/** Global Variables ****************************************************{
 *
 * Interfaces ideally do not use any extra memory.
 * Try to use macros and definitions instead.
 *
 */

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//} Global Variables
//{ Private Function declarations ****************************************

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}Private Function declarations
//{ Function definitions *************************************************

#ifndef ttc_driver_light_configuration_check  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_light_configuration_check() (using default implementation)!

void ttc_light_interface_configuration_check( t_ttc_light_config* Config ) {
    Assert_LIGHT_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function ttc_light_interface_configuration_check() is empty.


}

#endif
e_ttc_light_errorcode ttc_light_interface_load_defaults( t_ttc_light_config* Config ) {

    switch ( Config->Architecture ) { // physical index of led device from static configuration
#ifdef EXTENSION_light_gpio
        case E_ttc_light_architecture_gpio:  return light_gpio_load_defaults( Config ); break;
#endif
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // unknown architecture driver (maybe you missed to activate a low-level driver?
    }

    TTC_RETURN( 0 );
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
//{ private Function definitions *****************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_light_interface_foo(t_ttc_light_config* Config) { ... }

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
