/** { ttc_touchpad_interface.c ************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for TOUCHPAD device.
 *
 *  The functions implemented here are used when additional runtime
 *  computations arerequired to call the corresponding low-level driver.
 *  These computations should be held very simple and fast. Every additional
 *
 *  Adding a new low-level implementation may require changes in this file.
 *
 *  Functions in this file may not call ttc_touchpad_*() functions.
 *  Instead they can call _driver_touchpad_*() pendants.
 *  E.g.: ttc_touchpad_reset() -> _driver_touchpad_reset()
 *
 *  Created from template ttc_device_interface.c revision 20 at 20150316 07:48:52 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

//{ Includes *******************************************************************
//
// C-Sources include their corresponding header file and all other header files
// that are required to compile this source file.
// Includes that provide datatypes being used by function prototypes belong into
// "ttc_touchpad_interface.h".
//
#include "ttc_touchpad_interface.h"
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//}Includes
/** Global Variables ****************************************************{
 *
 * Interfaces ideally do not use any extra memory.
 * Try to use macros and definitions instead.
 *
 */

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//} Global Variables
//{ Function definitions *************************************************

#ifndef ttc_driver_touchpad_prepare  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_prepare() (using default implementation)!

void ttc_touchpad_interface_prepare() {


#warning Function ttc_touchpad_interface_prepare() is empty.


}

#endif
#ifndef ttc_driver_touchpad_get_features  // no low-level implementation: use default implementation

t_ttc_touchpad_config* ttc_touchpad_interface_get_features( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

    return ( t_ttc_touchpad_config* ) 0;
}

#endif
#ifndef ttc_driver_touchpad_init  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_init() (using default implementation)!

e_ttc_touchpad_errorcode ttc_touchpad_interface_init( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_init() is empty.

    return ( e_ttc_touchpad_errorcode ) 0;
}

#endif
#ifndef ttc_driver_touchpad_reset  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_reset() (using default implementation)!

void ttc_touchpad_interface_reset( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_reset() is empty.


}

#endif
#ifndef ttc_driver_touchpad_load_defaults  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_load_defaults() (using default implementation)!

e_ttc_touchpad_errorcode ttc_touchpad_interface_load_defaults( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_load_defaults() is empty.

    return ( e_ttc_touchpad_errorcode ) 0;
}

#endif
#ifndef ttc_driver_touchpad_deinit  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_deinit() (using default implementation)!

e_ttc_touchpad_errorcode ttc_touchpad_interface_deinit( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_deinit() is empty.

    return ( e_ttc_touchpad_errorcode ) 0;
}

#endif
#ifndef ttc_driver_touchpad_check  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_check() (using default implementation)!

void ttc_touchpad_interface_read( t_ttc_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_read() is empty.


}

#endif
#ifndef ttc_driver_touchpad_calibrate  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_touchpad_calibrate() (using default implementation)!

void ttc_touchpad_interface_calibrate( t_touchpad_config* Config ) {
    Assert_TOUCHPAD( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_touchpad_interface_calibrate() is empty.


}

#endif
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
//{ private Function definitions *****************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_touchpad_interface_foo(t_ttc_touchpad_config* Config) { ... }

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
