/** { ttc_usb_interface.c ************************************************
 *
 *                           The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Interface layer between high- and low-level driver for USB device.
 *
 *  The functions implemented here are used when additional runtime
 *  computations arerequired to call the corresponding low-level driver.
 *  These computations should be held very simple and fast. Every additional
 *
 *  Adding a new low-level implementation may require changes in this file.
 *
 *  Functions in this file may not call ttc_usb_*() functions.
 *  Instead they can call _driver_usb_*() pendants.
 *  E.g.: ttc_usb_reset() -> _driver_usb_reset()
 *
 *  Created from template ttc_device_interface.c revision 20 at 20140411 07:54:11 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#include "ttc_usb_interface.h"

/** Global Variables ****************************************************{
 *
 * Interfaces ideally do not use any extra memory.
 * Try to use macros and definitions instead.
 *
 */

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//} Global Variables
//{ Function definitions *************************************************

#ifndef ttc_driver_usb_load_defaults  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_load_defaults() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_load_defaults( t_ttc_usb_config* Config ) {
    Assert_USB( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usb_errorcode ) 0;
}

#endif
#ifndef ttc_driver_usb_prepare  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_prepare() (using default implementation)!

void ttc_usb_interface_prepare() {




}

#endif
#ifndef ttc_driver_usb_reset  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_reset() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_reset( t_ttc_usb_config* Config ) {
    Assert_USB( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usb_errorcode ) 0;
}

#endif
#ifndef ttc_driver_usb_init  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_init() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_init( t_ttc_usb_config* Config ) {
    Assert_USB( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usb_errorcode ) 0;
}

#endif
#ifndef ttc_driver_usb_deinit  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_deinit() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_deinit( t_ttc_usb_config* Config ) {
    Assert_USB( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usb_errorcode ) 0;
}

#endif
#ifndef ttc_driver_usb_get_features  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_get_features() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_get_features( t_ttc_usb_config* Config ) {
    Assert_USB( Config, E_ttc_assert_origin_auto ); // pointers must not be NULL


    return ( e_ttc_usb_errorcode ) 0;
}

#endif
#ifndef ttc_driver_usb_register_rx_function  // no low-level implementation: use default implementation
#  warning: missing low-level implementation for ttc_driver_usb_register_rx_function() (using default implementation)!

e_ttc_usb_errorcode ttc_usb_interface_register_rx_function( t_u8 LogicalIndex, void ( *TxFunction )( t_u8* Data_Buffer, t_u8Amount e_ttc_usb_errorcode ) {
    Assert_USB( Data_Buffer, E_ttc_assert_origin_auto ); // pointers must not be NULL

#warning Function ttc_usb_interface_register_rx_function() is empty.

    return ( e_ttc_usb_errorcode ) 0;
}

#endif
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}FunctionDefinitions
//{ private Function definitions *****************************************
//
// Convention: Private functions start with an underscore "_".
// Example:    void _ttc_usb_interface_foo(t_ttc_usb_config* Config) { ... }

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}PrivateFunctions
