/** { interrupt_stm32f1xx.c ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level driver for interrupt devices on stm32f1xx architectures.
 *  Implementation of low-level driver.
 *
 *  Created from template device_architecture.c revision 20 at 20140224 21:37:20 UTC
 *
 *  Note: See ttc_interrupt.h for description of stm32f1xx independent INTERRUPT implementation.
 *
 *  Authors: Gregor Rebel
}*/

#include "interrupt_stm32f1xx.h"
#include "../ttc_heap.h"

/* Note: The included files below are part of CPAL
 *
 * The files can be found by issuing the following line in your project folder:
 * _/findFile.sh core_cm3.h | grep CPAL
 * ...
 *
 * After a successfull compilation, they should also appear in <PROJECT_FOLDER>/additionals/current/.
 */
#include "additionals/270_CPAL_CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h" // (must be included BEFORE core_cm3.h!)
#include "additionals/270_CPAL_CMSIS/CM3/CoreSupport/core_cm3.h"

/* DEPRECATED
// @brief Bit definitions for register SYS_STATUS

#define SYS_STATUS_ID           0x0F            // System event Status Register
#define SYS_STATUS_LEN          (5)             // Note 40 bit register
//masks
#define SYS_STATUS_MASK_32      0xFFF7FFFFUL    // System event Status Register access mask (all unused fields should always be writen as zero)
//offset 0
#define SYS_STATUS_IRQS         0x00000001UL    // Interrupt Request Status READ ONLY
#define SYS_STATUS_CPLOCK       0x00000002UL    // Clock PLL Lock
#define SYS_STATUS_ESYNCR       0x00000004UL    // External Sync Clock Reset
#define SYS_STATUS_AAT          0x00000008UL    // Automatic Acknowledge Trigger
#define SYS_STATUS_TXFRB        0x00000010UL    // Transmit Frame Begins
#define SYS_STATUS_TXPRS        0x00000020UL    // Transmit Preamble Sent
#define SYS_STATUS_TXPHS        0x00000040UL    // Transmit PHY Header Sent
#define SYS_STATUS_TXFRS        0x00000080UL    // Transmit Frame Sent: This is set when the transmitter has completed the sending of a frame
//offset 8
#define SYS_STATUS_RXPRD        0x00000100UL    // Receiver Preamble Detected status
#define SYS_STATUS_RXSFDD       0x00000200UL    // Receiver Start Frame Delimiter Detected.
#define SYS_STATUS_LDEDONE      0x00000400UL    // LDE processing done
#define SYS_STATUS_RXPHD        0x00000800UL    // Receiver PHY Header Detect
#define SYS_STATUS_RXPHE        0x00001000UL    // Receiver PHY Header Error
#define SYS_STATUS_RXDFR        0x00002000UL    // Receiver Data Frame Ready
#define SYS_STATUS_RXFCG        0x00004000UL    // Receiver FCS Good
#define SYS_STATUS_RXFCE        0x00008000UL    // Receiver FCS Error
//offset 16
#define SYS_STATUS_RXRFSL       0x00010000UL    // Receiver Reed Solomon Frame Sync Loss
#define SYS_STATUS_RXRFTO       0x00020000UL    // Receive Frame Wait Timeout
#define SYS_STATUS_LDEERR       0x00040000UL    // Leading edge detection processing error
#define SYS_STATUS_reserved     0x00080000UL    // bit19 reserved
#define SYS_STATUS_RXOVRR       0x00100000UL    // Receiver Overrun
#define SYS_STATUS_RXPTO        0x00200000UL    // Preamble detection timeout
#define SYS_STATUS_GPIOIRQ      0x00400000UL    // GPIO interrupt
#define SYS_STATUS_SLP2INIT     0x00800000UL    // SLEEP to INIT
//offset 24
#define SYS_STATUS_RFPLL_LL     0x01000000UL    // RF PLL Losing Lock
#define SYS_STATUS_CLKPLL_LL    0x02000000UL    // Clock PLL Losing Lock
#define SYS_STATUS_RXSFDTO      0x04000000UL    // Receive SFD timeout
#define SYS_STATUS_HPDWARN      0x08000000UL    // Half Period Delay Warning
#define SYS_STATUS_TXBERR       0x10000000UL    // Transmit Buffer Error
#define SYS_STATUS_AFFREJ       0x20000000UL    // Automatic Frame Filtering rejection
#define SYS_STATUS_HSRBP        0x40000000UL    // Host Side Receive Buffer Pointer
#define SYS_STATUS_ICRBP        0x80000000UL    // IC side Receive Buffer Pointer READ ONLY
//offset 32
#define SYS_STATUS_RXRSCS       0x0100000000ULL // Receiver Reed-Solomon Correction Status
#define SYS_STATUS_RXPREJ       0x0200000000ULL // Receiver Preamble Rejection
#define SYS_STATUS_TXPUTE       0x0400000000ULL // Transmit power up time error

#define CLEAR_ALLRXGOOD_EVENTS  (SYS_STATUS_RXDFR | SYS_STATUS_RXFCG | SYS_STATUS_RXPRD |                                  SYS_STATUS_RXSFDD | SYS_STATUS_RXPHD | SYS_STATUS_LDEDONE) // Clear all RX event flags after a packet reception
#define CLEAR_DBLBUFF_EVENTS    (SYS_STATUS_RXDFR | SYS_STATUS_RXFCG) //| SYS_STATUS_LDEDONE)

#define CLEAR_ALLRXERROR_EVENTS (SYS_STATUS_RXPHE | SYS_STATUS_RXFCE | SYS_STATUS_RXRFSL |                                  SYS_STATUS_RXSFDTO | SYS_STATUS_RXRFTO | SYS_STATUS_RXPTO |                                 SYS_STATUS_AFFREJ | SYS_STATUS_LDEERR) // Clear all RX event flags after an rx error

#define CLEAR_ALLTX_EVENTS      (SYS_STATUS_AAT | SYS_STATUS_TXFRB | SYS_STATUS_TXPRS |                                  SYS_STATUS_TXPHS | SYS_STATUS_TXFRS    ) // Clear all TX event flags

*/

//{ Global Variables ***********************************************************

// All initializers use same configuration variable to safe memory and speed up initialization.
// Note: This makes interrupt initialization thread unsafe! Always enclose your initialisation calls in critical sections!
//DEPRECATED: NVIC_InitTypeDef interrupt_stm32f1xx_NVIC_Cfg;

// provided by ttc_interrupt.c
#ifdef EXTENSION_ttc_usart
const t_u8 interrupt_stm32f1xx_IRQChannel_USART[TTC_INTERRUPT_USART_AMOUNT] = { USART1_IRQn, USART2_IRQn, USART3_IRQn, 52, 53};
volatile t_register_stm32f1xx_usart* const interrupt_stm32f1xx_BaseRegister_USART[TTC_INTERRUPT_USART_AMOUNT] = {
    ( volatile t_register_stm32f1xx_usart* ) USART1,
    ( volatile t_register_stm32f1xx_usart* ) USART2,
    ( volatile t_register_stm32f1xx_usart* ) USART3,
    ( volatile t_register_stm32f1xx_usart* ) UART4,
    ( volatile t_register_stm32f1xx_usart* ) UART5,
};
A_dynamic_extern( t_ttc_interrupt_config_usart*, ttc_interrupt_config_usart );
#endif
#ifdef EXTENSION_ttc_gpio
    A_dynamic_extern( t_ttc_interrupt_config_gpio*,  ttc_interrupt_config_gpio );
#endif
#ifdef EXTENSION_ttc_timer
    A_dynamic_extern( t_ttc_interrupt_config_timer*,  ttc_interrupt_config_timer );
#endif
#ifdef EXTENSION_ttc_can
    A_dynamic_extern( t_ttc_interrupt_config_can*,  ttc_interrupt_config_can );
#endif

//}
//{ Function definitions *******************************************************

void interrupt_stm32f1xx_prepare() {

}
#ifdef EXTENSION_ttc_gpio //{
const t_u8 interrupt_stm32f1xx_IRQChannel_GPIO[TTC_INTERRUPT_GPIO_AMOUNT] = { EXTI0_IRQn,
                                                                              EXTI1_IRQn,
                                                                              EXTI2_IRQn,
                                                                              EXTI3_IRQn,
                                                                              EXTI4_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI9_5_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn,
                                                                              EXTI15_10_IRQn
                                                                            };

e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_gpio( t_ttc_interrupt_config_gpio* Config ) {
    t_ttc_gpio_bank GPIOx;
    t_u8 Pin;

    ttc_gpio_from_index( Config->PhysicalIndex, &GPIOx, &Pin );
    Assert_INTERRUPT( GPIOx, E_ttc_assert_origin_auto );
    Assert_INTERRUPT( Pin < TTC_GPIO_MAX_PINS, E_ttc_assert_origin_auto );

    // ensure that AFIO power is enabled before writing to EXTI registers (RM0008 p.201)
    register_stm32f1xx_RCC.APB2ENR.AFIO_EN = 1;

    // mask will allow fast bit masking in different registers
    Config->LowLevel.Mask = 1 << Pin;

    // ensure interrupt is disabled before reconfiguring it
    interrupt_stm32f1xx_disable_gpio( Config );

    // initialize GPIO pin as input
    ttc_gpio_init( Config->PhysicalIndex, E_ttc_gpio_mode_input_floating, E_ttc_gpio_speed_max );

    // calculate bit banding address of EXTI-line (==Pin) in first register of
    t_u32* BitBand_EXTI = cm3_calc_peripheral_BitBandAddress( &register_stm32f1xx_EXTI, Pin );

    // calculate bit banding adresses of corresponding bits in EXTI sub registers by adding constant register offsets
    t_u32* BitBand_RTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                      + 32 * ( ( ( t_u32 ) &register_stm32f1xx_EXTI.RTSR ) - ( ( t_u32 ) &register_stm32f1xx_EXTI ) )
                                    );
    t_u32* BitBand_FTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                      + 32 * ( ( ( t_u32 ) &register_stm32f1xx_EXTI.FTSR ) - ( ( t_u32 ) &register_stm32f1xx_EXTI ) )
                                    );

    // EXTI_RTSR + EXTI_FTSR
    switch ( Config->Common.Type ) {

        case  E_ttc_interrupt_type_GPIO_Active_Low:         // voltage on input pin is low
        case  E_ttc_interrupt_type_GPIO_Falling:            // voltage on input pin falls from logical 1 to 0

            *BitBand_RTSR = 0; // disable rising trigger
            *BitBand_FTSR = 1; // enable falling trigger
            break;

        case  E_ttc_interrupt_type_GPIO_Active_High:        // voltage on input pin is high
        case  E_ttc_interrupt_type_GPIO_Rising:             // voltage on input pin rises from logical 0 to 1

            *BitBand_FTSR = 0; // disable falling trigger
            *BitBand_RTSR = 1; // enable rising trigger
            break;

        case  E_ttc_interrupt_type_GPIO_Rising_Falling:     // voltage on input pin falls or rises
            *BitBand_FTSR = 1; // enable falling trigger
            *BitBand_RTSR = 1; // enable rising trigger
            break;

        default: ttc_assert_halt_origin( ec_basic_InvalidImplementation ); break; // unknown/ invalid interrupt type!
    }

    Assert_INTERRUPT( Pin < 16, E_ttc_assert_origin_auto );
    t_u8 IRQ_Channel = interrupt_stm32f1xx_IRQChannel_GPIO[Pin];
    Assert_INTERRUPT( ( IRQ_Channel  > 5 ) && ( IRQ_Channel < 41 ), E_ttc_assert_origin_auto ); // only EXT interrupt lines 6..40 available in STM32f1xx (->RM0008 p,???)

    // set interrupt priority of corresponding IRQ channel in NVIC
    interrupt_cortexm3_priority( IRQ_Channel, 0, 3 );

    // enable selected NVIC IRQ channel
    interrupt_cortexm3_enable( IRQ_Channel, 1 );

    TTC_RETURN ec_interrupt_OK;

    /* DEPRECATED (previous implementation)

        t_ttc_gpio_bank Bank;
        t_u8 Pin;
        ttc_gpio_from_index( Config->PhysicalIndex, &Bank, &Pin );
        Assert_INTERRUPT( Bank != 0, E_ttc_assert_origin_auto );
        Assert_INTERRUPT( Pin < TTC_GPIO_MAX_PINS, E_ttc_assert_origin_auto );

        // ensure interrupt is disabled before reconfiguring it
        interrupt_stm32f1xx_disable_gpio( Config );

        // initialize GPIO pin as input
        ttc_gpio_init( Config->PhysicalIndex, E_ttc_gpio_mode_input_floating, E_ttc_gpio_speed_max );

        if ( 1 ) { // configure interrupt source multiplexer for EXTI line
            // enable AFIO clockc
            register_stm32f1xx_RCC.APB2ENR.AFIO_EN = 1;

            t_u8 Value = 0;
            switch ( Bank ) {
                case E_ttc_gpio_bank_a: Value = gis_pa; break;
                case E_ttc_gpio_bank_b: Value = gis_pb; break;
                case E_ttc_gpio_bank_c: Value = gis_pc; break;
                case E_ttc_gpio_bank_d: Value = gis_pd; break;
                case E_ttc_gpio_bank_e: Value = gis_pe; break;
                case E_ttc_gpio_bank_f: Value = gis_pf; break;
                case E_ttc_gpio_bank_g: Value = gis_pg; break;
                default: Assert_INTERRUPT_EXTRA( 0, E_ttc_assert_origin_auto );
            }

            switch ( Pin ) { // configure EXTI line multiplexer that corresponds to given pin
                case  0: register_stm32f1xx_AFIO.EXTICR1.EXTI0  = Value; break;
                case  1: register_stm32f1xx_AFIO.EXTICR1.EXTI1  = Value; break;
                case  2: register_stm32f1xx_AFIO.EXTICR1.EXTI2  = Value; break;
                case  3: register_stm32f1xx_AFIO.EXTICR1.EXTI3  = Value; break;
                case  4: register_stm32f1xx_AFIO.EXTICR2.EXTI4  = Value; break;
                case  5: register_stm32f1xx_AFIO.EXTICR2.EXTI5  = Value; break;
                case  6: register_stm32f1xx_AFIO.EXTICR2.EXTI6  = Value; break;
                case  7: register_stm32f1xx_AFIO.EXTICR2.EXTI7  = Value; break;
                case  8: register_stm32f1xx_AFIO.EXTICR3.EXTI8  = Value; break;
                case  9: register_stm32f1xx_AFIO.EXTICR3.EXTI9  = Value; break;
                case 10: register_stm32f1xx_AFIO.EXTICR3.EXTI10 = Value; break;
                case 11: register_stm32f1xx_AFIO.EXTICR3.EXTI11 = Value; break;
                case 12: register_stm32f1xx_AFIO.EXTICR4.EXTI12 = Value; break;
                case 13: register_stm32f1xx_AFIO.EXTICR4.EXTI13 = Value; break;
                case 14: register_stm32f1xx_AFIO.EXTICR4.EXTI14 = Value; break;
                case 15: register_stm32f1xx_AFIO.EXTICR4.EXTI15 = Value; break;
            }
        }

        // calculate bit banding address of EXTI-line (==Pin) in first register of
        t_u32* BitBand_EXTI = cm3_calc_peripheral_BitBandAddress( &register_stm32f1xx_EXTI, Pin );

        // calculate bit banding adresses of corresponding bits in EXTI sub registers by adding constant register offsets
        t_u32* BitBand_RTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                          + 32 * ( ( ( t_u32 ) &register_stm32f1xx_EXTI.RTSR ) - ( ( t_u32 ) &register_stm32f1xx_EXTI ) )
                                        );
        t_u32* BitBand_FTSR = ( t_u32* )( ( ( t_u32 ) BitBand_EXTI )
                                          + 32 * ( ( ( t_u32 ) &register_stm32f1xx_EXTI.FTSR ) - ( ( t_u32 ) &register_stm32f1xx_EXTI ) )
                                        );

        switch ( Config->Common.Type ) { // configure trigger (EXTI_RTSR + EXTI_FTSR)

            case  E_ttc_interrupt_type_GPIO_Active_Low:         // voltage on input pin is low
            case  E_ttc_interrupt_type_GPIO_Falling:            // voltage on input pin falls from logical 1 to 0

                *BitBand_RTSR = 0; // disable rising trigger
                *BitBand_FTSR = 1; // enable falling trigger
                break;

            case  E_ttc_interrupt_type_GPIO_Active_High:        // voltage on input pin is high
            case  E_ttc_interrupt_type_GPIO_Rising:             // voltage on input pin rises from logical 0 to 1

                *BitBand_FTSR = 0; // disable falling trigger
                *BitBand_RTSR = 1; // enable rising trigger
                break;

            case  E_ttc_interrupt_type_GPIO_Rising_Falling:     // voltage on input pin falls or rises
                *BitBand_FTSR = 1; // enable falling trigger
                *BitBand_RTSR = 1; // enable rising trigger
                break;

            default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // unknown/ invalid interrupt type!
        }

        t_u8 IRQ_Channel = interrupt_stm32f1xx_IRQChannel_GPIO[Pin];
        Assert_INTERRUPT( IRQ_Channel < 57, E_ttc_assert_origin_auto );  // only interrupt lines 0..56 available in STM32f1xx (->RM0038 p,181,182)

        // set interrupt priority of corresponding IRQ channel in NVIC
        interrupt_cortexm3_priority( IRQ_Channel, 0, 3 );

        // enable selected NVIC IRQ channel
        interrupt_cortexm3_enable( IRQ_Channel, 1 );
    */
    /* DEPRECATED (initialize using StdPeripheral Library)
        GPIO_TypeDef* GPIOx;
        t_u8 Pin;

        ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );
        Assert_INTERRUPT_Writable( GPIOx, E_ttc_assert_origin_auto );
        Assert_INTERRUPT( Pin < TTC_GPIO_MAX_PINS, E_ttc_assert_origin_auto );


        EXTI_InitTypeDef EXTI_Cfg;

        t_u8 GPIO_PortSource = 0;
        t_u8 GPIO_PinType    = Pin;

        EXTI_StructInit( &EXTI_Cfg );

        // Enable AFIO clock
        RCC_APB2PeriphClockCmd( RCC_APB2Periph_AFIO, ENABLE );

        switch ( ( t_u32 ) GPIOx ) {
    #ifdef GPIOA
            case ( t_u32 ) GPIOA:
                GPIO_PortSource = GPIO_PortSourceGPIOA;
                break;
    #endif
    #ifdef GPIOB
            case ( t_u32 ) GPIOB:
                GPIO_PortSource = GPIO_PortSourceGPIOB;
                break;
    #endif
    #ifdef GPIOC
            case ( t_u32 ) GPIOC:
                GPIO_PortSource = GPIO_PortSourceGPIOC;
                break;
    #endif
    #ifdef GPIOD
            case ( t_u32 ) GPIOD:
                GPIO_PortSource = GPIO_PortSourceGPIOD;
                break;
    #endif
    #ifdef GPIOE
            case ( t_u32 ) GPIOE:
                GPIO_PortSource = GPIO_PortSourceGPIOE;
                break;
    #endif
    #ifdef GPIOF
            case ( t_u32 ) GPIOF:
                GPIO_PortSource = GPIO_PortSourceGPIOF;
                break;
    #endif
    #ifdef GPIOG
            case ( t_u32 ) GPIOG:
                GPIO_PortSource = GPIO_PortSourceGPIOG;
                break;
    #endif
            default:
                ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break;
        }

        EXTI_Cfg.EXTI_Mode = EXTI_Mode_Interrupt;
        EXTI_Cfg.EXTI_Line = 1 << Pin;

        switch ( Type ) { // find memory location where to store function pointer + argument

            case E_ttc_interrupt_type_GPIO_Falling:
                EXTI_Cfg.EXTI_Trigger = EXTI_Trigger_Falling;
                break;

            case E_ttc_interrupt_type_GPIO_Rising:
                EXTI_Cfg.EXTI_Trigger = EXTI_Trigger_Rising;
                break;

            case E_ttc_interrupt_type_GPIO_Rising_Falling:
                EXTI_Cfg.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
                break;

            default:
                ttc_assert_halt_origin( E_ttc_assert_origin_auto );
                break;
        }

        //configure the External interrupt/event line mapping
        GPIO_EXTILineConfig( GPIO_PortSource, GPIO_PinType );

        EXTI_Cfg.EXTI_LineCmd = DISABLE;
        EXTI_Init( &EXTI_Cfg );

        // 32-bit architecture allows fast constant arrays with direct pointer access
        static const t_u8 stm32_Interrupt_IRQChannel[TTC_INTERRUPT_GPIO_AMOUNT] = {
            EXTI0_IRQn,
            EXTI1_IRQn,
            EXTI2_IRQn,
            EXTI3_IRQn,
            EXTI4_IRQn,
            EXTI9_5_IRQn,
            EXTI9_5_IRQn,
            EXTI9_5_IRQn,
            EXTI9_5_IRQn,
            EXTI9_5_IRQn,
            EXTI15_10_IRQn,
            EXTI15_10_IRQn,
            EXTI15_10_IRQn,
            EXTI15_10_IRQn,
            EXTI15_10_IRQn,
            EXTI15_10_IRQn
        };

        interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel                   = stm32_Interrupt_IRQChannel[Pin];
        interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
        interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority        = 0x0F;
        interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd                = ENABLE;
        NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
    */

    return ec_interrupt_OK;
}
void interrupt_stm32f1xx_deinit_all_gpio() {
    ttc_memory_set( ( t_u8* ) & register_stm32f1xx_EXTI, 0, sizeof( register_stm32f1xx_EXTI ) );
    t_ttc_interrupt_config_gpio Config_ISR;
    Config_ISR.PhysicalIndex = E_ttc_gpio_pin_a0;

    for ( t_u8 IndexEXTI = 0; IndexEXTI < 32; IndexEXTI++ ) { // deinit all 32 EXTI lines
        interrupt_stm32f1xx_deinit_gpio( &Config_ISR );
        Config_ISR.PhysicalIndex += 4;
    }
}
void interrupt_stm32f1xx_deinit_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    interrupt_stm32f1xx_disable_gpio( Config );

    if ( 1 ) { // disable interrupt channel in NVIC ICER register
        t_ttc_gpio_bank* GPIOx;
        t_u8 Pin;
        ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );

        t_u8 IRQ_Channel = interrupt_stm32f1xx_IRQChannel_GPIO[Pin];
        t_u8 RegisterIndex = IRQ_Channel / 32;
        Assert_INTERRUPT( RegisterIndex < 3, E_ttc_assert_origin_auto );  // each NVIC register has three 32 bit fields!

        // disable selected IRQ channel
        volatile t_u32* ICER = &register_cortexm3_NVIC.ICER0.All;
        t_u32* BitBand_ICER = cm3_calc_peripheral_BitBandAddress( ICER + RegisterIndex, IRQ_Channel );
        *BitBand_ICER = 1; // set corresponding bit in ICER register

        // disble selected NVIC IRQ channel
        interrupt_cortexm3_enable( IRQ_Channel, 0 );
    }
}
void interrupt_stm32f1xx_disable_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // clear corresponding bit in Interrupt Mask Register
    volatile t_u32 IMR = * ( ( t_u32* ) &register_stm32f1xx_EXTI.IMR.All );
    IMR &= ~Config->LowLevel.Mask;
    register_stm32f1xx_EXTI.IMR.All = IMR;

    /* DEPRECATED (previous implementation
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    GPIO_TypeDef* GPIOx;
    t_u8 Pin;
    ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );

    // clear corresponding bit in Interrupt Mask Register
    volatile t_u32 IMR = * ( ( t_u32* ) &register_stm32f1xx_EXTI.IMR.All );
    IMR &= ~( 1 << Pin );
    register_stm32f1xx_EXTI.IMR.All = IMR;
    */
    /* DEPRECATED (using STM Standard Peripheral Library)
    EXTI_InitTypeDef* EXTI_Cfg = _interrupt_stm32f1xx_enable_gpio( Config );
    EXTI_Cfg->EXTI_LineCmd = 0;

    EXTI_Init( EXTI_Cfg );
    */
}
void interrupt_stm32f1xx_enable_gpio( t_ttc_interrupt_config_gpio* Config ) {
#if (TTC_Assert_INTERRUPTS==1)
    Assert_INTERRUPT( !A_is_null( ttc_interrupt_config_gpio ), E_ttc_assert_origin_auto );  // init interrupt first!
    t_ttc_interrupt_config_gpio* GPIO_ISR_Entry = A( ttc_interrupt_config_gpio, Pin );
    Assert_INTERRUPT( GPIO_ISR_Entry, E_ttc_assert_origin_auto );  // init interrupt first!
#endif

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // set corresponding bit in Interrupt Mask Register
    register_stm32f1xx_EXTI.IMR.All |= Config->LowLevel.Mask;

    // re-set pending bit (required to re-activate if interrupt already occured)
    register_stm32f1xx_EXTI.PR.All = Config->LowLevel.Mask;

    /* DEPRECATED (previous implementation
    #if 1
    #if (TTC_Assert_INTERRUPTS==1)
    Assert_INTERRUPT( !A_is_null( ttc_interrupt_config_gpio ), E_ttc_assert_origin_auto );  // init interrupt first!
    t_ttc_interrupt_config_gpio* GPIO_ISR_Entry = A( ttc_interrupt_config_gpio, Pin );
    Assert_INTERRUPT( GPIO_ISR_Entry, E_ttc_assert_origin_auto );  // init interrupt first!
    #endif

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    t_ttc_gpio_bank* GPIOx;
    t_u8 Pin;
    ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );
    t_u32 Mask = 1 << Pin;

    // set corresponding bit in Interrupt Mask Register
    register_stm32f1xx_EXTI.IMR.All |= Mask;

    // re-set pending bit (required to re-activate if interrupt already occured)
    register_stm32f1xx_EXTI.PR.All  |= Mask;
    */
    /* DEPRECATED (using STM Standard Peripheral Library)
    EXTI_InitTypeDef* EXTI_Cfg = _interrupt_stm32f1xx_enable_gpio( Config );
    EXTI_Cfg->EXTI_LineCmd = 1;

    EXTI_Init( EXTI_Cfg );
    */
}
void interrupt_stm32f1xx_generate_gpio( t_ttc_interrupt_config_gpio* Config ) {
#if (TTC_Assert_INTERRUPTS==1)
    Assert_INTERRUPT( !A_is_null( ttc_interrupt_config_gpio ), E_ttc_assert_origin_auto );  // init interrupt first!
    t_ttc_interrupt_config_gpio* GPIO_ISR_Entry = A( ttc_interrupt_config_gpio, Pin );
    Assert_INTERRUPT( GPIO_ISR_Entry, E_ttc_assert_origin_auto );  // init interrupt first!
#endif
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

    // re-set pending bit (required to re-activate if interrupt already occured)
    register_stm32f1xx_EXTI.PR.All = Config->LowLevel.Mask;

    // set corresponding bit in Interrupt Mask Register to trigger interrupt
    register_stm32f1xx_EXTI.SWIER.All = Config->LowLevel.Mask;

    /* DEPRECATED (previous implementation

    t_ttc_gpio_bank* GPIOx;
    t_u8 Pin;
    ttc_gpio_from_index( Config->PhysicalIndex, ( t_ttc_gpio_bank* ) &GPIOx, &Pin );
    t_u32 Mask = 1 << Pin;

    // set corresponding bit in Interrupt Mask Register to trigger interrupt
    register_stm32f1xx_EXTI.SWIER.All |= Mask;
    */
}
/* DEPRECATED
t_u8 interrupt_stm32f1xx_gpio_calc_interruptline( t_base PhysicalIndex ) {
    t_u8 InterruptLine;
    t_ttc_gpio_bank GPIOx;

    // STM32F1xx provides one interrupt line for each pin index. All GPIO banks share the same interrupt lines.
    ttc_gpio_from_index( PhysicalIndex, &GPIOx, &InterruptLine );
    Assert_INTERRUPT( GPIOx != 0, E_ttc_assert_origin_auto );
    Assert_INTERRUPT( InterruptLine < TTC_INTERRUPT_GPIO_AMOUNT, E_ttc_assert_origin_auto );

    return InterruptLine;
}
*/
/* moved to ttc_gpio_from_index
void interrupt_stm32f1xx_gpio_from_index(t_base PhysicalIndex, t_ttc_gpio_bank* GPIOx, t_u8* Pin) {
    Assert_GPIO(GPIOx); // pointers must not be NULL
    Assert_GPIO(Pin); // pointers must not be NULL

    *GPIOx = (t_ttc_gpio_bank) cm3_calc_peripheral_Word(PhysicalIndex);
    *Pin   = (t_u8)            cm3_calc_peripheral_BitNumber(PhysicalIndex);
}
*/
t_base interrupt_stm32f1xx_gpio_create_index( t_ttc_gpio_bank GPIOx, t_u8 Pin ) {
    return ( t_base ) cm3_calc_peripheral_BitBandAddress( GPIOx, Pin );
}
t_u8 interrupt_stm32f1xx_gpio_calc_config_index( t_base PhysicalIndex ) {

    e_ttc_gpio_pin GPIO = ( e_ttc_gpio_pin ) PhysicalIndex; // we know that physical index is bit banding address of gpio pin
    t_u8 PinNo = cm3_calc_peripheral_BitNumber( GPIO );

    return PinNo; // stm32f1xx has one interrupt line for each pin over all banks (E_ttc_gpio_pin_a1 == tgpb1 == ...) so only pin # is important
}
t_bool interrupt_stm32f1xx_is_enablight_gpio( t_ttc_interrupt_config_gpio* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

    t_bool IsEnabled = ( register_stm32f1xx_EXTI.IMR.All & Config->LowLevel.Mask ) ? TRUE : FALSE;

    TTC_RETURN( IsEnabled );   // will perform stack overflow check and return given value
}

/* Interrupt handlers for GPIO on stm32f1xx
 *
 * The interrupt handling implemented here is a compromise between
 * handling speed and code size.
 *
 * The interrupt starts in one of the EXTI*_IRQHandler() functions below:
 * 1) Reset interrupt line as fast as possible.
 *    It writes a constant 32 bit value to register_stm32f1xx_EXTI.PR.
 *    The C code is a one liner that compiles to a very fast 32-bit constant write to PR register.
 * 2) GPIO_General_IRQHandler() is called with the current interrupt line.
 *    The use of a central interrupt handler avoids to duplicate its code seven times to reduce code size.
 * 3) GPIO_General_IRQHandler()
 *    a) Looks up registration entry for given interrupt line.
 *    b) Calls registered function pointer Entry->isr() with two arguments:
 *       - Entry->PhysicalIndex = GPIO pin identifyer
 *       - Entry->Argument      = Registered argument (32 bit value given to interrupt_stm32f1xx_init_gpio() before)
 *
 */
void GPIO_General_IRQHandler( t_physical_index InterruptLine ) {

    // declaring static variable is a little bit faster as we do not have to allocate stack space
    static t_ttc_interrupt_config_gpio* Entry = NULL;

    Entry = A( ttc_interrupt_config_gpio, InterruptLine );
    if ( Entry->isr )
    { Entry->isr( Entry->PhysicalIndex, Entry->Argument ); }
}
void EXTI0_IRQHandler() {
    // reset interrupt source (required to be retriggered)
    *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 0;

    // call the general GPIO Interrupt handler (will call registered interrupt service routine)
    GPIO_General_IRQHandler( 0 );
}
void EXTI1_IRQHandler() {
    // reset interrupt source (required to be retriggered)
    *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 1;
    GPIO_General_IRQHandler( 1 );
}
void EXTI2_IRQHandler() {
    // reset interrupt source (required to be retriggered)
    *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 2;

    GPIO_General_IRQHandler( 2 );
}
void EXTI3_IRQHandler() {
    // reset interrupt source (required to be retriggered)
    *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 3;

    GPIO_General_IRQHandler( 3 );
}
void EXTI4_IRQHandler() {
    // reset interrupt source (required to be retriggered)
    *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 4;

    GPIO_General_IRQHandler( 4 );
}
void EXTI9_5_IRQHandler() {

    t_u32 PR = *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR );

    if ( PR & 1 << 5 ) { // EXTI5 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 5;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 5 );
    }
    if ( PR & 1 << 6 ) { // EXTI6 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 6;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 6 );
    }
    if ( PR & 1 << 7 ) { // EXTI7 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 7;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 7 );
    }
    if ( PR & 1 << 8 ) { // EXTI8 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 8;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 8 );
    }
    if ( PR & 1 << 9 ) { // EXTI9 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 9;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 9 );
    }
}
void EXTI15_10_IRQHandler() {

    t_u32 PR = *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR );

    if ( PR & 1 << 10 ) { // EXTI10 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 10;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 10 );
    }
    if ( PR & 1 << 11 ) { // EXTI11 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 11;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 11 );
    }
    if ( PR & 1 << 12 ) { // EXTI12 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 12;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 12 );
    }
    if ( PR & 1 << 13 ) { // EXTI13 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 13;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 13 );
    }
    if ( PR & 1 << 14 ) { // EXTI14 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 14;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 14 );
    }
    if ( PR & 1 << 15 ) { // EXTI15 interrupt active
        // reset interrupt source (required to be retriggered)
        *( ( volatile t_u32* ) & register_stm32f1xx_EXTI.PR ) = 1 << 15;

        // call the general GPIO Interrupt handler
        GPIO_General_IRQHandler( 15 );
    }
}

#endif //}EXTENSION_ttc_gpio
#ifdef EXTENSION_ttc_usart //{

e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    Assert_INTERRUPT( Config->PhysicalIndex < 3, E_ttc_assert_origin_auto );  // stm32f1xx has only three USARTs

    Config->LowLevel.BaseRegister = interrupt_stm32f1xx_BaseRegister_USART[Config->PhysicalIndex];
    Assert_INTERRUPT( Config->LowLevel.BaseRegister->CR1.Bits.UE == 1, E_ttc_assert_origin_auto ); // device unit is switched off. Switch it on or initialize it first!

    if ( 1 ) { // calculate bit banding address of interrupt enable bits for fast and atomic enable/disable
        volatile t_u32* ControlRegister1 = NULL;
        t_u8 BitNumber1 = 0;
        volatile t_u32* ControlRegister2 = NULL;
        t_u8 BitNumber2 = 0;

        switch ( Type ) { // enable individual USART interrupt source

            case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR3 );
                BitNumber1 = 10;
                break;
            }
            case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 4;
                break;
            }
            case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 6;
                break;
            }
            case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 7;
                break;
            }
            case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 5;
                break;
            }
            case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR2 );
                BitNumber1 = 6;
                break;
            }
            case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
                ControlRegister1 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR1 );
                BitNumber1 = 8;
                ControlRegister2 = ( volatile t_u32* ) & ( Config->LowLevel.BaseRegister->CR3 );
                BitNumber2 = 0;
                break;
            }
            default: ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
        }

        Assert_INTERRUPT_EXTRA_Writable( ( void* ) ControlRegister1, E_ttc_assert_origin_auto );
        volatile t_u32* EnableBit1 = cm3_calc_peripheral_BitBandAddress( ControlRegister1, BitNumber1 );
        volatile t_u32* EnableBit2 = NULL;
        if ( ControlRegister2 ) {
            EnableBit2 = cm3_calc_peripheral_BitBandAddress( ControlRegister2, BitNumber2 );
        }

        switch ( Type ) { // enable individual USART interrupt source

            case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
                Config->LowLevel.EnableBits.CTSIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
                Config->LowLevel.EnableBits.IDLEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
                Config->LowLevel.EnableBits.TCIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
                Config->LowLevel.EnableBits.TXEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
                Config->LowLevel.EnableBits.RXNEIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
                Config->LowLevel.EnableBits.LBDIE = EnableBit1;
                break;
            }
            case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
                Config->LowLevel.EnableBits.PEIE = EnableBit1;
                Config->LowLevel.EnableBits.EIE  = EnableBit2;
                break;
            }
            default: ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
        }
    }
    if ( 1 ) { // enable selected NVIC IRQ channel
        t_u8 IRQ_Channel = interrupt_stm32f1xx_IRQChannel_USART[Config->PhysicalIndex];
        interrupt_cortexm3_enable( IRQ_Channel, 1 );
    }
    if ( 1 ) { // reading from SR and then from DR will reset flags in SR
        volatile t_u32 Dummy = Config->LowLevel.BaseRegister->SR.All;
        Dummy = Config->LowLevel.BaseRegister->DR.All;
        ( void ) Dummy;
    }
    if ( 1 ) { // configure NVIC
        t_u8 IRQ_Channel = interrupt_stm32f1xx_IRQChannel_USART[Config->PhysicalIndex];

        // set interrupt priority of corresponding IRQ channel in NVIC
        interrupt_cortexm3_priority( IRQ_Channel, 0, 3 );
    }
    /* DEPRECATED (Kept as comparison)
    //    //Enable NVIC interrupt channel for USART
    //    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
    //    NVIC_InitTypeDef NVIC_InitStructure;

    //    switch (Config->PhysicalIndex)
    //    {
    //        case 0:
    //            NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
    //        break;
    //        case 1:
    //            NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
    //        break;
    //        case 2:
    //            NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
    //        break;
    //    }
    //    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    //    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    //    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    //    NVIC_Init(&NVIC_InitStructure);
    */

    TTC_RETURN( ec_interrupt_OK );
}
t_bool                    interrupt_stm32f1xx_is_enabled_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)
    t_bool IsEnabled = FALSE;
    switch ( Type ) {
        case E_ttc_interrupt_type_USART_Cts: {
            if ( Config->LowLevel.EnableBits.CTSIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_Error: {
            if ( *Config->LowLevel.EnableBits.PEIE != 0 ) { IsEnabled = TRUE; }
            if ( *Config->LowLevel.EnableBits.EIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_Idle: {
            if ( *Config->LowLevel.EnableBits.IDLEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_TxComplete: {
            if ( *Config->LowLevel.EnableBits.TCIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_TransmitDataEmpty: {
            if ( *Config->LowLevel.EnableBits.TXEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_RxNE: {
            if ( *Config->LowLevel.EnableBits.RXNEIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        case E_ttc_interrupt_type_USART_LinBreak: {
            if ( *Config->LowLevel.EnableBits.LBDIE != 0 ) { IsEnabled = TRUE; }

            break;
        }
        default: break;
    }

    TTC_RETURN( IsEnabled );   // will perform stack overflow check and return given value
}
void                     _interrupt_stm32f1xx_enable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type, t_bool Enable ) {
    /** enables or disables interrupt source of given type by using atomic bit band access to configuration register.
     *
     */

    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL
    t_physical_index interrupt_stm32f1xx_PhysicalIndex = Config->PhysicalIndex;
    Assert_INTERRUPT( interrupt_stm32f1xx_PhysicalIndex <= 5, E_ttc_assert_origin_auto );

    volatile t_register_stm32f1xx_usart* BaseRegister = Config->LowLevel.BaseRegister;
    Assert_INTERRUPT_Writable( ( void* ) BaseRegister, E_ttc_assert_origin_auto );

    if ( 1 ) { // enable interrupt source using atomic, precalculated bit banding access
        volatile t_u32* EnableFlag;
        switch ( Type ) { // select interrupt enable flag for given type

            case E_ttc_interrupt_type_USART_Cts: {               // CR3.Bits.CTSIE
                EnableFlag = Config->LowLevel.EnableBits.CTSIE;
                break;
            }
            case E_ttc_interrupt_type_USART_Idle: {              // CR1.Bits.IDLEIE
                EnableFlag = Config->LowLevel.EnableBits.IDLEIE;
                break;
            }
            case E_ttc_interrupt_type_USART_TxComplete: {        // CR1.Bits.TCIE
                EnableFlag = Config->LowLevel.EnableBits.TCIE;
                break;
            }
            case E_ttc_interrupt_type_USART_TransmitDataEmpty: { // CR1.Bits.TXEIE
                EnableFlag = Config->LowLevel.EnableBits.TXEIE;
                break;
            }
            case E_ttc_interrupt_type_USART_RxNE: {              // CR1.Bits.RXNEIE
                EnableFlag = Config->LowLevel.EnableBits.RXNEIE;
                break;
            }
            case E_ttc_interrupt_type_USART_LinBreak: {          // CR2.Bits.LBDIE
                EnableFlag = Config->LowLevel.EnableBits.LBDIE;
                break;
            }
            case E_ttc_interrupt_type_USART_Error: {             // CR1.Bits.PEIE, CR3.Bits.EIE
                EnableFlag = Config->LowLevel.EnableBits.PEIE;

                // second flag is handled right here to reduce overall code size
                Assert_INTERRUPT_EXTRA_Writable( ( void* ) Config->LowLevel.EnableBits.EIE, E_ttc_assert_origin_auto );
                *( Config->LowLevel.EnableBits.EIE )  = 0;
                break;
            }
            default: EnableFlag = NULL; ttc_assert_halt_origin( ec_interrupt_InvalidImplementation ); break; // invalid type given!
        }

        // atomically enable interrupts for given type using bitband address
        Assert_INTERRUPT_Writable( ( void* ) EnableFlag, E_ttc_assert_origin_auto );
        *EnableFlag = ( Enable ) ? 1 : 0;
    }
}
void                      interrupt_stm32f1xx_enable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    _interrupt_stm32f1xx_enable_usart( Config, Type, TRUE );
    /* DEPRECATED (previous implementation)

    // Enable and set USARTn Interrupt to the lowest priority
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;

    USART_TypeDef* USART_Base = NULL;

    switch ( Config->PhysicalIndex ) { // physical index starts at 0

        case 0: USART_Base = USART1; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART1_IRQn; break;
        case 1: USART_Base = USART2; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART2_IRQn; break;
        case 2: USART_Base = USART3; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART3_IRQn; break;
    #if defined(STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined(STM32F10X_XL) ||defined(STM32F10X_CL)
        case 3: USART_Base = UART4;  interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  UART4_IRQn; break;
    #endif
    #if defined(STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined(STM32F10X_XL) ||defined(STM32F10X_CL)
        case 4: USART_Base = UART5;  interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  UART5_IRQn; break;
    #endif
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    #if 1==0
    // produces lots of warning in GCC 4.7 (ToDo: fix warnings)
    t_register_stm32f1xx_usart_cr1 CR1; *( ( t_u16* ) &CR1 ) = ( t_u16* ) & ( USART_Base->CR1 );
    t_register_stm32f1xx_usart_cr2 CR2; *( ( t_u16* ) &CR2 ) = ( t_u16* ) & ( USART_Base->CR2 );
    t_register_stm32f1xx_usart_cr3 CR3; *( ( t_u16* ) &CR3 ) = ( t_u16* ) & ( USART_Base->CR3 );
    #else
    t_register_stm32f1xx_usart_cr1 CR1; CR1.All = USART_Base->CR1;
    t_register_stm32f1xx_usart_cr2 CR2; CR2.All = USART_Base->CR2;
    t_register_stm32f1xx_usart_cr3 CR3; CR3.All = USART_Base->CR3;
    #endif

    switch ( Type ) { // set/ clear corresponding bits

        case E_ttc_interrupt_type_USART_Cts:
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_ClearToSend != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CR3.Bits.CTSIE  = 1;
            break;
        case E_ttc_interrupt_type_USART_Idle:
            CR1.Bits.IDLEIE = 1;

            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_IdleLine != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_TxComplete:
            CR1.Bits.TCIE   = 1;
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_TransmissionComplete != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_TransmitDataEmpty:
            CR1.Bits.TXEIE  = 1;
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_TransmitDataEmpty != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_RxNE:
            CR1.Bits.RXNEIE = 1;
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_ReceiveDataNotEmpty != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_LinBreak:
            CR2.Bits.LBDIE  = 1;
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_LinBreakDetected != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_Error:
            Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_Error != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CR1.Bits.PEIE   = 1;  // irq on parity error
            CR3.Bits.EIE    = 1;  // irq on general error
            break;
        default: break;
    }

    if ( ! 1 ) { // disable interrupt channel only if no other interrupt event for this USART is active
        if ( ( CR1.Bits.PEIE   == 0 ) &&
                ( CR1.Bits.IDLEIE == 0 ) &&
                ( CR1.Bits.TCIE   == 0 ) &&
                ( CR1.Bits.TXEIE  == 0 ) &&
                ( CR1.Bits.RXNEIE == 0 ) &&
                ( CR2.Bits.LBDIE  == 0 ) &&
                ( CR3.Bits.CTSIE  == 0 ) &&
                ( CR3.Bits.EIE    == 0 )
           )
        { interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = DISABLE; }
    }
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );

    // Note: We have to copy struct-data in this odd way because USART_Base->CR1.TXEIE = 1; gives unpredictable results!
    ( ( USART_TypeDef* ) USART_Base )->CR1 = *( ( t_u16* ) &CR1 );
    ( ( USART_TypeDef* ) USART_Base )->CR2 = *( ( t_u16* ) &CR2 );
    ( ( USART_TypeDef* ) USART_Base )->CR3 = *( ( t_u16* ) &CR3 );
    */
}
void                      interrupt_stm32f1xx_disable_usart( t_ttc_interrupt_config_usart* Config, e_ttc_interrupt_type Type ) {
    _interrupt_stm32f1xx_enable_usart( Config, Type, FALSE );
    /* DEPRECATED (previous implementation)
    // Enable and set USARTn Interrupt to the lowest priority
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;

    USART_TypeDef* USART_Base = NULL;

    switch ( Config->PhysicalIndex ) { // physical index starts at 0

        case 0: USART_Base = USART1; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART1_IRQn; break;
        case 1: USART_Base = USART2; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART2_IRQn; break;
        case 2: USART_Base = USART3; interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = USART3_IRQn; break;
    #if defined(STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined(STM32F10X_XL) ||defined(STM32F10X_CL)
        case 3: USART_Base = UART4;  interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  UART4_IRQn; break;
    #endif
    #if defined(STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined(STM32F10X_XL) ||defined(STM32F10X_CL)
        case 4: USART_Base = UART5;  interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  UART5_IRQn; break;
    #endif
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    #if 1==0
    // produces lots of warning in GCC 4.7 (ToDo: fix warnings)
    t_register_stm32f1xx_usart_cr1 CR1; *( ( t_u16* ) &CR1 ) = ( t_u16* ) & ( USART_Base->CR1 );
    t_register_stm32f1xx_usart_cr2 CR2; *( ( t_u16* ) &CR2 ) = ( t_u16* ) & ( USART_Base->CR2 );
    t_register_stm32f1xx_usart_cr3 CR3; *( ( t_u16* ) &CR3 ) = ( t_u16* ) & ( USART_Base->CR3 );
    #else
    t_register_stm32f1xx_usart_cr1 CR1; CR1.All = USART_Base->CR1;
    t_register_stm32f1xx_usart_cr2 CR2; CR2.All = USART_Base->CR2;
    t_register_stm32f1xx_usart_cr3 CR3; CR3.All = USART_Base->CR3;
    #endif

    switch ( Type ) { // set/ clear corresponding bits

        case E_ttc_interrupt_type_USART_Cts:
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_ClearToSend != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            CR3.Bits.CTSIE  = 0;
            break;
        case E_ttc_interrupt_type_USART_Idle:
            CR1.Bits.IDLEIE = 0;
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_IdleLine != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_TxComplete:
            CR1.Bits.TCIE   = 0;
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_TransmissionComplete != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_TransmitDataEmpty:
            CR1.Bits.TXEIE  = 0;
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_TransmitDataEmpty != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_RxNE:
            CR1.Bits.RXNEIE = 0;
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_ReceiveDataNotEmpty != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_LinBreak:
            CR2.Bits.LBDIE  = 0;
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_LinBreakDetected != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            break;
        case E_ttc_interrupt_type_USART_Error:
            if ( 0 )
            { Assert_INTERRUPT( A( ttc_interrupt_config_usart, Config->PhysicalIndex )->isr_Error != NULL, E_ttc_assert_origin_auto ); }  // no isr configured for this interrupt!
            CR1.Bits.PEIE   = 0;  // irq on parity error
            CR3.Bits.EIE    = 0;  // irq on general error
            break;
        default: break;
    }

    // disable interrupt channel only if no other interrupt event for this USART is active
    if ( ( CR1.Bits.PEIE   == 0 ) &&
            ( CR1.Bits.IDLEIE == 0 ) &&
            ( CR1.Bits.TCIE   == 0 ) &&
            ( CR1.Bits.TXEIE  == 0 ) &&
            ( CR1.Bits.RXNEIE == 0 ) &&
            ( CR2.Bits.LBDIE  == 0 ) &&
            ( CR3.Bits.CTSIE  == 0 ) &&
            ( CR3.Bits.EIE    == 0 )
       )
    { interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = DISABLE; }
    else
    { interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE; }

    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );

    // Note: We have to copy struct-data in this odd way because USART_Base->CR1.TXEIE = 1; gives unpredictable results!
    ( ( USART_TypeDef* ) USART_Base )->CR1 = *( ( t_u16* ) &CR1 );
    ( ( USART_TypeDef* ) USART_Base )->CR2 = *( ( t_u16* ) &CR2 );
    ( ( USART_TypeDef* ) USART_Base )->CR3 = *( ( t_u16* ) &CR3 );
    */
}
void                      interrupt_stm32f1xx_deinit_usart( t_ttc_interrupt_config_usart* Config ) {
    ( void ) Config;
    // all done in ttc_interrupt
}

// USART/UART IRQ Handlers
IF_Assert_INTERRUPT( t_u32 interrupt_stm32f1xx_AmountEmptyIsrUSART = 0 ); //DEBUG
void USART_General_IRQHandler( t_physical_index PhysicalIndex, volatile t_register_stm32f1xx_usart* USART ) {

    // using volatile variables to ensure correct read order from USART registers (reading from SR and then from DR will reset many flags)
    volatile t_register_stm32f1xx_usart_cr1 CR1            = USART->CR1;
    volatile t_register_stm32f1xx_usart_sr  StatusRegister = USART->SR;  // reading from SR and then from DR will reset flags in SR

    t_ttc_interrupt_config_usart* ISRs = A( ttc_interrupt_config_usart, PhysicalIndex );
    Assert_INTERRUPT( ISRs != NULL, E_ttc_assert_origin_auto );  // Usart indexed by PhysicalIndex has not been initialized. Memory corruption or broken ttc_interrupt implementation?
    t_u8 Handled = 0;
    if ( StatusRegister.Bits.RXNE && CR1.Bits.RXNEIE ) { // handling byte receive first to minimize risk of loosing bytes
        Assert_INTERRUPT_Executable( ISRs->isr_ReceiveDataNotEmpty, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_ReceiveDataNotEmpty( PhysicalIndex, ISRs->Argument_ReceiveDataNotEmpty );
        goto EndOfFunction; // short cut: leave now to process additional received bytes faster (other pending interrupts are served in next call)

        Handled = 1;
    }
    if ( StatusRegister.Bits.TXE  && CR1.Bits.TXEIE ) { // handlind byte transmit second to maximum speed
        Assert_INTERRUPT_Executable( ISRs->isr_TransmitDataEmpty, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_TransmitDataEmpty( PhysicalIndex, ISRs->Argument_TransmitDataEmpty );
        StatusRegister.Bits.TXE = 0;
        if ( StatusRegister.All == 0 )
        { goto EndOfFunction; } // short cut: no other interrupts pending: leave now

        Handled = 1;
    }
    if ( StatusRegister.Bits.TC   && CR1.Bits.TCIE ) {
        Assert_INTERRUPT_Executable( ISRs->isr_TransmissionComplete, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_TransmissionComplete( PhysicalIndex, ISRs->Argument_TransmissionComplete );
        Handled = 1;
    }
    if ( StatusRegister.Bits.IDLE && CR1.Bits.IDLEIE ) {
        Assert_INTERRUPT_Executable( ISRs->isr_IdleLine, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_IdleLine( PhysicalIndex, ISRs->Argument_IdleLine );
        Handled = 1;
    }
    if ( StatusRegister.Bits.LBD  && USART->CR2.Bits.LBDIE ) {
        Assert_INTERRUPT_Executable( ISRs->isr_LinBreakDetected, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_LinBreakDetected( PhysicalIndex, ISRs->Argument_LinBreakDetected );
        Handled = 1;
    }
    if ( StatusRegister.Bits.CTS  && USART->CR3.Bits.CTSIE ) {
        Assert_INTERRUPT_Executable( ISRs->isr_ClearToSend, E_ttc_assert_origin_auto ); // someone has enabled this interrupt but did not register a valid function pointer.
        ISRs->isr_ClearToSend( PhysicalIndex, ISRs->Argument_ClearToSend );
        Handled = 1;
    }

    if ( ( *( ( t_u8* ) &StatusRegister ) ) & 0xf ) {  // error occured: gather data + call error handler
        if ( ISRs->isr_Error != NULL ) {  // error-handler has been defined: call it

            t_ttc_interrupt_usart_errors Error; *( ( t_u8* ) &Error ) = 0;
            if ( StatusRegister.Bits.FE )   { Error.Framing = 1; }
            if ( StatusRegister.Bits.NE )   { Error.Noise   = 1; }
            if ( StatusRegister.Bits.ORE )  { Error.Overrun = 1; }
            if ( StatusRegister.Bits.PE )   { Error.Parity  = 1; }

            ISRs->isr_Error( PhysicalIndex, Error, ISRs->Argument_Error );
            Handled = 1;
        }
    }
    if ( Handled == 0 ) { // interrupt was not handled at all: reset StatusRegister
        // reading from SR and then from DR will reset flags in SR
        volatile t_u32 Dummy = USART->SR.All;
        Dummy = USART->DR.All;
        ( void ) Dummy;

        // reset all other status bits
        USART->SR.All = 0;
        IF_Assert_INTERRUPT( interrupt_stm32f1xx_AmountEmptyIsrUSART++ ); //DEBUG
    }

EndOfFunction:
    return;
}
void USART1_IRQHandler() {
    USART_General_IRQHandler( 0, ( t_register_stm32f1xx_usart* ) USART1 );
    USART1->SR = 0; // clear all interrupt flags
}
void USART2_IRQHandler() {
    USART_General_IRQHandler( 1, ( t_register_stm32f1xx_usart* ) USART2 );
    USART2->SR = 0; // clear all interrupt flags
}
void USART3_IRQHandler() {
    USART_General_IRQHandler( 2, ( t_register_stm32f1xx_usart* ) USART3 );
    USART3->SR = 0; // clear all interrupt flags
}
void UART4_IRQHandler() {
    USART_General_IRQHandler( 3, ( t_register_stm32f1xx_usart* ) UART4 );
    UART4->SR = 0; // clear all interrupt flags
}
void UART5_IRQHandler() {
    USART_General_IRQHandler( 4, ( t_register_stm32f1xx_usart* ) UART5 );
    UART5->SR = 0; // clear all interrupt flags
}

#endif //}EXTENSION_ttc_usart
#ifdef EXTENSION_ttc_i2c //{
TODO( "Implement I2C interrupt handling for stm32f1xx!" )

t_bool interrupt_stm32f1xx_is_enabled_i2c( t_ttc_interrupt_config_i2c* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_is_enabled_i2c() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_i2c( t_ttc_interrupt_config_i2c* Config ) {

    ttc_assert_halt_origin( E_ttc_assert_origin_auto );

    return ( e_ttc_interrupt_errorcode ) 0;
}
void interrupt_stm32f1xx_enable_i2c( t_ttc_interrupt_config_i2c* Config ) {

    ttc_assert_halt_origin( E_ttc_assert_origin_auto );
}
void interrupt_stm32f1xx_disable_i2c( t_ttc_interrupt_config_i2c* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL



}
void interrupt_stm32f1xx_deinit_all_i2c() {




}
void interrupt_stm32f1xx_deinit_i2c( t_ttc_interrupt_config_i2c* Config ) {

}

#endif //}
#ifdef EXTENSION_ttc_spi //{
TODO( "Implement SPI interrupt handling for stm32f1xx!" )

e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_spi( t_ttc_interrupt_config_spi* Config ) {

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!
#warning Function interrupt_stm32f1xx_init_spi() is empty.


    return ( e_ttc_interrupt_errorcode ) 0;
}
t_bool interrupt_stm32f1xx_is_enabled_spi( t_ttc_interrupt_config_spi* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_is_enabled_spi() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
void interrupt_stm32f1xx_enable_spi( t_ttc_interrupt_config_spi* Config ) {

#warning Function interrupt_stm32f1xx_enable_spi() is empty.

    ttc_assert_halt_origin( E_ttc_assert_origin_auto ); // ToDo: implement!
}
void interrupt_stm32f1xx_deinit_all_spi() {


#warning Function interrupt_stm32f1xx_deinit_all_spi() is empty.


}
void interrupt_stm32f1xx_disable_spi( t_ttc_interrupt_config_spi* Config ) {
    Assert_INTERRUPT_Writable( Config, E_ttc_assert_origin_auto );  // pointers must not be NULL

#warning Function interrupt_stm32f1xx_disable_spi() is empty.


}
void interrupt_stm32f1xx_deinit_spi( t_ttc_interrupt_config_spi* Config ) {

}
#endif //}
#ifdef EXTENSION_ttc_timer //{

t_bool interrupt_stm32f1xx_is_enabled_timer( t_ttc_interrupt_config_timer* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_is_enabled_timer() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_timer( t_ttc_interrupt_config_timer* Config ) {

    t_register_stm32f1xx_timer* TIMER_Base;
    ( void )TIMER_Base;     //Avoid the warning

    switch ( Config->PhysicalIndex ) { // physical index starts is the logical index
        case 1: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER1;
            switch ( Config->Common.Type ) {
                case E_ttc_interrupt_type_TIMER_Updating:
#  if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL) || defined (STM32F10X_HD_VL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_UP_TIM16_IRQn;
#  elif defined (STM32F10X_LD) || defined (STM32F10X_MD) || defined (STM32F10X_HD) || defined (STM32F10X_CL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_UP_IRQn;
#  else
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_UP_TIM10_IRQn;
#  endif
                    break;
                case E_ttc_interrupt_type_TIMER_Break:
#  if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL) || defined (STM32F10X_HD_VL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_BRK_TIM15_IRQn;
#  elif defined (STM32F10X_LD) || defined (STM32F10X_MD) || defined (STM32F10X_HD) || defined (STM32F10X_CL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_BRK_IRQn;
#  else
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_BRK_TIM9_IRQn;
#  endif
                    break;
                case E_ttc_interrupt_type_TIMER_CC1:
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_CC_IRQn;
                    break;
                case E_ttc_interrupt_type_TIMER_CC2:
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_CC_IRQn;
                    break;
                case E_ttc_interrupt_type_TIMER_CC3:
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_CC_IRQn;
                    break;
                case E_ttc_interrupt_type_TIMER_CC4:
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_CC_IRQn;
                    break;
                case E_ttc_interrupt_type_TIMER_Triggering:
#  if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL) || defined (STM32F10X_HD_VL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_TRG_COM_TIM17_IRQn;
#  elif defined (STM32F10X_LD) || defined (STM32F10X_MD) || defined (STM32F10X_HD) || defined (STM32F10X_CL)
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_TRG_COM_IRQn;
#  else
                    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM1_TRG_COM_TIM11_IRQn;
#  endif
                    break;
                default: ttc_assert_halt_origin( E_ttc_assert_origin_auto );
                    break;
            }
            break;
        case 2: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER2;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM2_IRQn;
            break;
        case 3: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER3;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM3_IRQn;
            break;
        case 4: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER4;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM4_IRQn;
            break;
#if defined (STM32F10X_HD) || defined(STM32F10X_HD_VL) || defined(STM32F10X_XL) || defined (STM32F10X_CL)
        case 5: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER5;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM5_IRQn;
            break;
#endif
#if defined(STM32F10X_LD_VL) || defined(STM32F10X_MD_VL) || defined(STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
        case 6: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER6;
#  if defined(STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM6_IRQn;
#  else
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  54;//TIM6_DAC_IRQn;
#  endif
            break;
        case 7: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER7;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM7_IRQn;
            break;
#endif
#if defined(STM32F10X_HD) || defined(STM32F10X_XL)
        case 8: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER8;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM8_IRQn;
            break;
#endif
#if defined (STM32F10X_XL)
        case 9: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER9;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_BRK_TIM9_IRQn;
            break;
        case 10: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER10;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_UP_TIM10_IRQn;
            break;
        case 11: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER11;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_TRG_COM_TIM11_IRQn;
            break;
#endif
#if defined (STM32F10X_HD_VL) || defined (STM32F10X_XL)
        case 12: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER12;
#  if defined (STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM8_BRK_TIM12_IRQn;
#  else
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM12_IRQn;
#  endif
            break;
        case 13: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER13;
#  if defined (STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM8_UP_TIM13_IRQn;
#  else
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM13_IRQn;
#  endif
            break;
        case 14: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER14;
#  if defined (STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel = TIM8_TRG_COM_TIM14_IRQn;
#  else
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM14_IRQn;
#  endif
            break;
#endif
#if defined (STM32F10X_HD_VL) || defined (STM32F10X_HD_VL) || defined (STM32F10X_LD_VL)
        case 15: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER15;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_BRK_TIM15_IRQn;
            break;
        case 16: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER16;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_UP_TIM16_IRQn;
            break;
        case 17: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER17;
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel =  TIM1_TRG_COM_TIM17_IRQn;
            break;
#endif
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    // Enable and set TIMx Interrupt to the lowest priority
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x0F;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
    return ec_interrupt_OK;

}
void _interrupt_stm32f1xx_enable_timer( t_ttc_interrupt_config_timer* Config ) {
    t_register_stm32f1xx_timer* TIMER_Base = NULL;

    switch ( Config->PhysicalIndex ) {
        case 1: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER1;
            break;
        case 2: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER2;
            break;
        case 3: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER3;
            break;
        case 4: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER4;
            break;
        case 5: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER5;
            break;
#if defined (STM32F10X_XL)
        case 9: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER9;
            break;
        case 10: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER10;
            break;
        case 11: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER11;
            break;
        case 12: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER12;
            break;
        case 13: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER13;
            break;
        case 14: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER14;
            break;
#endif
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto );
            break;
    }

#ifndef STM32F10X_XL
    switch ( Config->Common.Type ) { // select trigger according to Type
        case E_ttc_interrupt_type_TIMER_Updating:
            TIMER_Base->DIER.All |= ( ( ut_int16 )0x0001 );
            break;
        case E_ttc_interrupt_type_TIMER_CC1:
            TIMER_Base->DIER.All |= ( ( ut_int16 )0x0002 );
            break;
        case E_ttc_interrupt_type_TIMER_CC2:
            TIMER_Base->DIER.All |= ( ( ut_int16 )0x0004 );
            break;
        case E_ttc_interrupt_type_TIMER_Triggering:
            TIMER_Base->DIER.All |= ( ( ut_int16 )0x0040 );
            break;
        case E_ttc_interrupt_type_TIMER_Break:
            if ( TIMER_Base == &register_stm32f1xx_TIMER1 ) //|| (TIMER_Base == TIM8))
            { TIMER_Base->DIER.All |= ( ( ut_int16 )0x0080 ); }
            break;
        default:
            ttc_assert_halt_origin( E_ttc_assert_origin_auto );
            break;
    }
#endif
# ifdef STM32F10X_XL
    if ( ( TIMER_Base == &register_stm32f1xx_TIMER10 ) || ( TIMER_Base == &register_stm32f1xx_TIMER11 ) || ( TIMER_Base == &register_stm32f1xx_TIMER13 ) || ( TIMER_Base == &register_stm32f1xx_TIMER14 ) ) {
        switch ( Type ) {
            case E_ttc_interrupt_type_TIMER_Updating:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0001 );
                break;
            case E_ttc_interrupt_type_TIMER_CC1:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0002 );
                break;
            default:
                ttc_assert_halt_origin( E_ttc_assert_origin_auto );
                break;
        }
    }
    else {
        switch ( Type ) { // select trigger according to Type
            case E_ttc_interrupt_type_TIMER_Updating:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0001 );
                break;
            case E_ttc_interrupt_type_TIMER_CC1:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0002 );
                break;
            case E_ttc_interrupt_type_TIMER_CC2:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0004 );
                break;
            case E_ttc_interrupt_type_TIMER_Triggering:
                TIMER_Base->DIER.All |= ( ( ut_int16 )0x0040 );
                break;
            case E_ttc_interrupt_type_TIMER_Break:
                if ( TIMER_Base == &register_stm32f1xx_TIMER1 ) //|| (TIMER_Base == TIM8))
                { TIMER_Base->DIER.All |= ( ( ut_int16 )0x0080 ); }
                break;
            default:
                ttc_assert_halt_origin( E_ttc_assert_origin_auto );
                break;
        }
    }
#endif
}
void interrupt_stm32f1xx_enable_timer( t_ttc_interrupt_config_timer* Config ) {
    _interrupt_stm32f1xx_enable_timer( Config ); // initializes interrupt_stm32f1xx_NVIC_Cfg
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
}
void interrupt_stm32f1xx_disable_timer( t_ttc_interrupt_config_timer* Config ) {

    _interrupt_stm32f1xx_enable_timer( Config );

    // disable interrupt channel only if no other interrupt event for this TIMER is active
    t_register_stm32f1xx_timer* TIMER_Base = NULL;
    u_register_stm32f1xx_timx_dier EnableRegister;

    switch ( Config->PhysicalIndex ) {
        case 1: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER1;
            break;
        case 2: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER2;
            break;
        case 3: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER3;
            break;
        case 4: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER4;
            break;
        case 5: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER5;
            break;
#if defined (STM32F10X_XL)
        case 9: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER9;
            break;
        case 10: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER10;
            break;
        case 11: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER11;
            break;
        case 12: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER12;
            break;
        case 13: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER13;
            break;
        case 14: TIMER_Base = ( t_register_stm32f1xx_timer* )&register_stm32f1xx_TIMER14;
            break;
#endif
        default: ttc_assert_halt_origin( E_ttc_assert_origin_auto );
            break;
    }

    EnableRegister.All = ( t_u16 ) TIMER_Base->DIER.All;

    if ( ( EnableRegister.Bits.UIE == 0 )   &&
            ( EnableRegister.Bits.CC1IE == 0 ) &&
            ( EnableRegister.Bits.CC2IE == 0 ) &&
            ( EnableRegister.Bits.CC3IE == 0 ) &&
            ( EnableRegister.Bits.CC4IE == 0 ) &&
            ( EnableRegister.Bits.COMIE == 0 ) &&
            ( EnableRegister.Bits.TIE == 0 )   &&
            ( EnableRegister.Bits.BIE == 0 )
       )
    { interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = DISABLE; }
    else
    { interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE; }
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
}

#endif //}EXTENSION_ttc_timer
#ifdef EXTENSION_ttc_can //{
TODO( "Implement CAN interrupt handling for stm32f1xx!" )

e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_can( t_ttc_interrupt_config_can* Config ) {
    t_register_stm32f1xx_can* CAN_Base = NULL;
    ( void )CAN_Base;

    // Enable and set CANn Interrupt to the lowest priority
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelPreemptionPriority = 0x00;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelSubPriority = 0x00;
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;

    switch ( Config->PhysicalIndex ) { // physical index starts at 0
        case 0: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN1;
            break;
        case 1: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN2;        break;
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    switch ( Type ) { // set/clear corresponding bits
        case E_ttc_interrupt_type_CAN_Transmit_Empty:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= USB_HP_CAN1_TX_IRQn;
#elif  defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_TX_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Pending:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= USB_LP_CAN1_RX0_IRQn;
#elif  defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX0_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Full:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= USB_LP_CAN1_RX0_IRQn;
#elif  defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX0_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Overrun:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= USB_LP_CAN1_RX0_IRQn;
#elif  defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX0_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Pending:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX1_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Full:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX1_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Overrun:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_RX1_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_Error_Warning:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_SCE_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_Error_Passive:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_SCE_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_Bus_Off:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_SCE_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_WakeUp_State:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_SCE_IRQn;
#endif
            break;
        case E_ttc_interrupt_type_CAN_Sleep_State:
#if defined(STM32F10X_LD) || defined(STM32F10X_MD) || defined(STM32F10X_HD) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
            interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannel |= CAN1_SCE_IRQn;
#endif
            break;
        default: break;
    }

    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );

    return ( e_ttc_interrupt_errorcode ) 0;
}
void interrupt_stm32f1xx_enable_can( t_ttc_interrupt_config_can* Config ) {
    t_register_stm32f1xx_can* CAN_Base = NULL;

    switch ( Config->PhysicalIndex ) { // physical index starts at 0
        case 0: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN1;
            break;
        case 1: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN2;        break;
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    switch ( Type ) { // set/clear corresponding bits
        case E_ttc_interrupt_type_CAN_Transmit_Empty:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_Transmit != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000001 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Pending:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO0_Pending != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000002 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Full:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO0_Full != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000004 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Overrun:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO0_Overrun != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000008 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Pending:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO1_Pending != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000010 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Full:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO1_Full != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000020 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Overrun:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_FIFO1_Overrun != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000040 );
            break;
        case E_ttc_interrupt_type_CAN_Error_Warning:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_Error_Warning != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000100 );
            break;
        case E_ttc_interrupt_type_CAN_Error_Passive:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_Error_Passive != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000200 );
            break;
        case E_ttc_interrupt_type_CAN_Bus_Off:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_Bus_Off != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00000400 );
            break;
        case E_ttc_interrupt_type_CAN_WakeUp_State:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_WakeUp_State != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00010000 );
            break;
        case E_ttc_interrupt_type_CAN_Sleep_State:
            Assert_INTERRUPT( A( ttc_interrupt_config_can, Config->PhysicalIndex )->isr_Sleep_State != NULL, E_ttc_assert_origin_auto );  // no isr configured for this interrupt!
            CAN_Base->IER.All |= ( ( t_u32 )0x00020000 );
            break;
        default: break;
    }

    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
}
void interrupt_stm32f1xx_disable_can( t_ttc_interrupt_config_can* Config ) {
    t_register_stm32f1xx_can* CAN_Base = NULL;

    switch ( Config->PhysicalIndex ) { // physical index starts at 0
        case 0: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN1;
            break;
        case 1: CAN_Base = ( t_register_stm32f1xx_can* )&register_stm32f1xx_CAN2;        break;
        default: { ttc_assert_halt_origin( E_ttc_assert_origin_auto ); }
    }

    switch ( Type ) { // set/clear corresponding bits
        case E_ttc_interrupt_type_CAN_Transmit_Empty:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000001 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Pending:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000002 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Full:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000004 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO0_Overrun:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000008 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Pending:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000010 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Full:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000020 );
            break;
        case E_ttc_interrupt_type_CAN_FIFO1_Overrun:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000040 );
            break;
        case E_ttc_interrupt_type_CAN_Error_Warning:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000100 );
            break;
        case E_ttc_interrupt_type_CAN_Error_Passive:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000200 );
            break;
        case E_ttc_interrupt_type_CAN_Bus_Off:
            CAN_Base->IER.All |= ( ( t_u32 )0x00000400 );
            break;
        case E_ttc_interrupt_type_CAN_WakeUp_State:
            CAN_Base->IER.All |= ( ( t_u32 )0x00010000 );
            break;
        case E_ttc_interrupt_type_CAN_Sleep_State:
            CAN_Base->IER.All |= ( ( t_u32 )0x00020000 );
            break;
        default: break;
    }

    // disable interrupt channel only if no other interrupt event for this CAN is active
    interrupt_stm32f1xx_NVIC_Cfg.NVIC_IRQChannelCmd = DISABLE;
    NVIC_Init( &interrupt_stm32f1xx_NVIC_Cfg );
}
#endif //}

void interrupt_stm32f1xx_deinit_all_usart() {




}
#ifdef EXTENSION_ttc_rtc //{
TODO( "Implement RTC interrupt handling for stm32f1xx!" )

void interrupt_stm32f1xx_deinit_all_rtc() {


#warning Function interrupt_stm32f1xx_deinit_all_rtc() is empty.


}

void interrupt_stm32f1xx_deinit_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto );  // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_deinit_rtc() is empty.


}
void interrupt_stm32f1xx_disable_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto );  // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_disable_rtc() is empty.


}
void interrupt_stm32f1xx_enable_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto );  // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_enable_rtc() is empty.


}
e_ttc_interrupt_errorcode interrupt_stm32f1xx_init_rtc( t_ttc_interrupt_config_rtc* Config ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto );  // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_init_rtc() is empty.

    return ( e_ttc_interrupt_errorcode ) 0;
}
t_bool interrupt_stm32f1xx_is_enabled_rtc( t_ttc_interrupt_config_rtc* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_is_enabled_rtc() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}
#endif //}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//{ irq-handlers (supported) ***************************************************

#ifdef EXTENSION_ttc_timer
void TIMER_General_IRQHandler( t_physical_index PhysicalIndex, volatile t_register_stm32f1xx_timer* TIMER ) {
    Assert_INTERRUPT( TIMER != NULL, E_ttc_assert_origin_auto );
    u_register_stm32f1xx_timx_sr StatusRegister;
    StatusRegister.All = ( t_u16 )TIMER->SR.All; // reading from SR and then from DR will reset flags in SR
    t_ttc_interrupt_config_timer* ISRs = A( ttc_interrupt_config_timer, PhysicalIndex );
    Assert_INTERRUPT( ISRs != NULL, E_ttc_assert_origin_auto );  // should never happen: the usart indexed by PhysicalIndex has not been initialized!

    if ( StatusRegister.Bits.UIF )
    { ISRs->isr_Update( PhysicalIndex, ISRs->Argument_Update ); }
    /*
    if(StatusRegister.Bits.CC1IF)
       ISRs->isr_CC1IF(PhysicalIndex, ISRs->Argument_CC1IF);
    if(StatusRegister.Bits.CC2IF)
       ISRs->isr_CC2IF(PhysicalIndex, ISRs->Argument_CC2IF);
    if(StatusRegister.Bits.CC3IF)
       ISRs->isr_CC3IF(PhysicalIndex, ISRs->Argument_CC3IF);
    if(StatusRegister.Bits.CC4IF)
       ISRs->isr_CC4IF(PhysicalIndex, ISRs->Argument_CC4IF);
    if(StatusRegister.Bits.TIF)
       ISRs->isr_Trigger(PhysicalIndex, ISRs->Argument_Trigger);
    if(StatusRegister.Bits.BIF)
       ISRs->isr_Break(PhysicalIndex, ISRs->Argument_Break);
    if(StatusRegister.Bits.CC1OF)
       ISRs->isr_CC10F(PhysicalIndex, ISRs->Argument_CC10F);
    if(StatusRegister.Bits.CC2OF)
       ISRs->isr_CC20F(PhysicalIndex, ISRs->Argument_CC20F);
    if(StatusRegister.Bits.CC3OF)
       ISRs->isr_CC30F(PhysicalIndex, ISRs->Argument_CC30F);
    if(StatusRegister.Bits.CC4OF)
       ISRs->isr_CC40F(PhysicalIndex, ISRs->Argument_CC40F);
    */
}
void TIM1_UP_IRQHandler() {
    TIMER_General_IRQHandler( 1, ( t_register_stm32f1xx_timer* ) &register_stm32f1xx_TIMER1 );
    register_stm32f1xx_TIMER1.SR.All = 0; // clear all interrupt flags
}
void TIM2_IRQHandler() {
    TIMER_General_IRQHandler( 2, ( t_register_stm32f1xx_timer* ) &register_stm32f1xx_TIMER2 );
    register_stm32f1xx_TIMER2.SR.All = 0; // clear all interrupt flags
}
void TIM3_IRQHandler() {
    TIMER_General_IRQHandler( 3, ( t_register_stm32f1xx_timer* ) &register_stm32f1xx_TIMER3 );
    register_stm32f1xx_TIMER3.SR.All = 0; // clear all interrupt flags
}
void TIM4_IRQHandler() {
    TIMER_General_IRQHandler( 4, ( t_register_stm32f1xx_timer* ) &register_stm32f1xx_TIMER4 );
    register_stm32f1xx_TIMER4.SR.All = 0; // clear all interrupt flags
}
void TIM5_IRQHandler() {
    TIMER_General_IRQHandler( 5, ( t_register_stm32f1xx_timer* ) &register_stm32f1xx_TIMER5 );
    register_stm32f1xx_TIMER5.SR.All = 0; // clear all interrupt flags
}
# ifdef STM32F10X_XL
void TIM9_IRQHandler() {
    TIMER_General_IRQHandler( 9, ( t_register_stm32f1xx_timer* ) TIM9 );
}
void TIM10_IRQHandler() {
    TIMER_General_IRQHandler( 10, ( t_register_stm32f1xx_timer* ) TIM10 );
}
void TIM11_IRQHandler() {
    TIMER_General_IRQHandler( 11, ( t_register_stm32f1xx_timer* ) TIM11 );
}
void TIM12_IRQHandler() {
    TIMER_General_IRQHandler( 12, ( t_register_stm32f1xx_timer* ) TIM12 );
}
void TIM13_IRQHandler() {
    TIMER_General_IRQHandler( 13, ( t_register_stm32f1xx_timer* ) TIM13 );
}
void TIM14_IRQHandler() {
    TIMER_General_IRQHandler( 14, ( t_register_stm32f1xx_timer* ) TIM14 );
}
# endif
void interrupt_stm32f1xx_deinit_all_timer() {




}
void interrupt_stm32f1xx_deinit_timer( t_ttc_interrupt_config_timer* Config ) {

}
#else
void TIM1_UP_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM1_TRG_COM_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM1_CC_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM2_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM3_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM4_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}

#endif
#ifdef EXTENSION_ttc_can
TODO( "Implement CAN interrupt handling for stm32f1xx!" )

void interrupt_stm32f1xx_deinit_all_can() {




}
t_bool interrupt_stm32f1xx_is_enabled_can( t_ttc_interrupt_config_can* Config, e_ttc_interrupt_type Type ) {
    Assert_INTERRUPT_EXTRA_Writable( Config, E_ttc_assert_origin_auto ); // check if it points to RAM (change for read only memory!)

#warning Function interrupt_stm32f1xx_is_enabled_can() is empty.

    TTC_RETURN( 0 );   // will perform stack overflow check and return given value
}

void CAN_General_IRQHandler( t_physical_index PhysicalIndex, volatile t_register_stm32f1xx_can* ISR_CAN ) {
#warning Replace stm32f1xx dependent code by hardware independent structures and functions!
    register_stm32f1xx_can_msr_t MSR = ISR_CAN->MSR;
    register_stm32f1xx_can_tsr_t TSR = ISR_CAN->TSR;  // reading from SR and then from DR will reset flags in SR
    register_stm32f1xx_can_rf0r_t RF0R = ISR_CAN->RF0R;
    register_stm32f1xx_can_rf1r_t RF1R = ISR_CAN->RF1R;
    register_stm32f1xx_can_esr_t ESR = ISR_CAN->ESR;

    t_ttc_interrupt_config_can* ISRs = A( ttc_interrupt_config_can, PhysicalIndex );
    Assert_INTERRUPT( ISRs != NULL, E_ttc_assert_origin_auto );  // should never happen: the can indexed by PhysicalIndex has not been initialized!

    if ( TSR.Bits.RQCP0 && ISRs->isr_Transmit ) {
        TSR.Bits.RQCP0 = 0;
        ISRs->isr_Transmit( PhysicalIndex, ISRs->Argument_Transmit );
    }
    if ( TSR.Bits.RQCP1 && ISRs->isr_Transmit ) {
        TSR.Bits.RQCP1 = 0;
        ISRs->isr_Transmit( PhysicalIndex, ISRs->Argument_Transmit );
    }
    if ( TSR.Bits.RQCP2 && ISRs->isr_Transmit ) {
        TSR.Bits.RQCP2 = 0;
        ISRs->isr_Transmit( PhysicalIndex, ISRs->Argument_Transmit );
    }
    if ( RF0R.Bits.FMP0 && ISRs->isr_FIFO0_Pending ) {
        RF0R.Bits.FMP0 = 0;
        ISRs->isr_FIFO0_Pending( PhysicalIndex, ISRs->Argument_FIFO0_Pending );
    }
    if ( RF0R.Bits.FULL0 && ISRs->isr_FIFO0_Full ) {
        RF0R.Bits.FULL0 = 0;
        ISRs->isr_FIFO0_Full( PhysicalIndex, ISRs->Argument_FIFO0_Full );
    }
    if ( RF0R.Bits.FOVR0 && ISRs->isr_FIFO0_Overrun ) {
        RF0R.Bits.FOVR0 = 0;
        ISRs->isr_FIFO0_Overrun( PhysicalIndex, ISRs->Argument_FIFO0_Overrun );
    }
    if ( RF1R.Bits.FMP1 && ISRs->isr_FIFO1_Pending ) {
        RF1R.Bits.FMP1 = 0;
        ISRs->isr_FIFO1_Pending( PhysicalIndex, ISRs->Argument_FIFO1_Pending );
    }
    if ( RF1R.Bits.FULL1 && ISRs->isr_FIFO1_Full ) {
        RF1R.Bits.FULL1 = 0;
        ISRs->isr_FIFO1_Full( PhysicalIndex, ISRs->Argument_FIFO1_Full );
    }
    if ( RF1R.Bits.FOVR1 && ISRs->isr_FIFO1_Overrun ) {
        RF1R.Bits.FOVR1 = 0;
        ISRs->isr_FIFO1_Overrun( PhysicalIndex, ISRs->Argument_FIFO1_Overrun );
    }
    if ( ESR.Bits.BOFF && ISRs->isr_Bus_Off ) {
        ESR.Bits.BOFF = 0;
        ISRs->isr_Bus_Off( PhysicalIndex, ISRs->Argument_Bus_Off );
    }
    if ( ESR.Bits.EWGF && ISRs->isr_Error_Warning ) {
        ESR.Bits.EWGF = 0;
        ISRs->isr_Error_Warning( PhysicalIndex, ISRs->Argument_Error_Warning );
    }
    if ( ESR.Bits.EPVF && ISRs->isr_Error_Passive ) {
        ESR.Bits.EPVF = 0;
        ISRs->isr_Error_Passive( PhysicalIndex, ISRs->Argument_Error_Passive );
    }
    if ( MSR.Bits.WKUI && ISRs->isr_WakeUp_State ) {
        MSR.Bits.WKUI = 0;
        ISRs->isr_WakeUp_State( PhysicalIndex, ISRs->Argument_WakeUp_State );
    }
    if ( MSR.Bits.SLAKI && ISRs->isr_Sleep_State ) {
        MSR.Bits.SLAKI = 0;
        ISRs->isr_Sleep_State( PhysicalIndex, ISRs->Argument_Sleep_State );
    }
}

void USB_HP_CAN1_TX_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void CAN1_TX_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void USB_LP_CAN1_RX0_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void CAN1_RX0_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void CAN1_RX1_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void CAN1_SCE_IRQHandler() {
    CAN_General_IRQHandler( 0, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN1 );
}
void CAN2_TX_IRQHandler() {
    CAN_General_IRQHandler( 1, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN2 );
}
void CAN2_RX0_IRQHandler() {
    CAN_General_IRQHandler( 1, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN2 );
}
void CAN2_RX1_IRQHandler() {
    CAN_General_IRQHandler( 1, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN2 );
}
void CAN2_SCE_IRQHandler() {
    CAN_General_IRQHandler( 1, ( t_register_stm32f1xx_can* ) &register_stm32f1xx_CAN2 );
}
void interrupt_stm32f1xx_deinit_can( t_ttc_interrupt_config_can* Config ) {

}
#else
void CAN1_RX1_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void CAN1_SCE_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
#endif

//}irq-handlers (supported)
//{ irq-handlers (yet unsupported) *********************************************

/* void Reset_Handler() {
  // write your own reset-handler here...


  Assert_INTERRUPT(0, E_ttc_assert_origin_auto);
} */
void DebugMon_Handler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void WWDG_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void PVD_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TAMPER_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void FLASH_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void RCC_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel1_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel2_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel3_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel4_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel5_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel6_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA1_Channel7_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void ADC1_2_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void TIM1_BRK_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void I2C1_EV_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void I2C1_ER_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void I2C2_EV_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void I2C2_ER_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void SPI1_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void SPI2_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void RTCAlarm_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void OTG_FS_WKUP_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void SPI3_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA2_Channel1_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA2_Channel2_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA2_Channel3_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA2_Channel4_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void DMA2_Channel5_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void ETH_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void ETH_WKUP_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void SDIO_IRQHandler( void ) {
    // Process All SDIO Interrupt Sources
    //X  SD_ProcessIRQSrc();
}
void USB_HP_CAN1_TX_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void USB_LP_CAN1_RX0_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}
void USBWakeUp_IRQHandler() {
    // call your interrupt-handler right here, or else ...


    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}

#ifdef USE_USB_OTG_FS
    void OTG_FS_IRQHandler( void )
#else
    void OTG_HS_IRQHandler( void )
#endif
{
#ifdef EXTENSION_400_stm32_usb_fs_device_lib
    STM32_PCD_OTG_ISR_Handler();
#elif EXTENSION_400_stm32_usb_fs_host_lib
    USBH_OTG_ISR_Handler( &USB_OTG_Core_dev );
#else
    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
#endif
}

//#ifdef STM32F10X_CL
//void OTG_FS_IRQHandler() {
//    // call your interrupt-handler right here, or else ...
//#ifdef EXTENSION_400_stm32_usb_fs_device_lib
//    STM32_PCD_OTG_ISR_Handler();
//#elif EXTENSION_400_stm32_usb_fs_host_lib
//    USBH_OTG_ISR_Handler(&USB_OTG_Core_dev);
//#else
//    Assert_INTERRUPT(0, E_ttc_assert_origin_auto);
//#endif
//}
//#endif
void BootRAM() {

    // call your interrupt-handler right here, or else ...

    Assert_INTERRUPT( 0, E_ttc_assert_origin_auto );
}

//} irq-handlers (yet unsupported)

//}FunctionDefinitions
//{ private functions (ideally) ************************************************

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//} private functions
