#ifndef INTERRUPT_STM32F1XX_TYPES_H
#define INTERRUPT_STM32F1XX_TYPES_H

/** { interrupt_stm32f1xx.h ****************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level datatypes for INTERRUPT devices on stm32f1xx architectures.
 *  Structures, Enums and Defines being required by ttc_interrupt_types.h
 *
 *  Created from template device_architecture_types.h revision 20 at 20140224 21:37:20 UTC
 *
 *  Note: See ttc_interrupt.h for description of architecture independent INTERRUPT implementation.
 *
 *  Authors: Gregor Rebel, Greg Knoll
 *
}*/
//{ Defines/ TypeDefs **********************************************************

#ifndef TTC_INTERRUPT1   // device not defined in makefile
    #define TTC_INTERRUPT1    E_ttc_interrupt_architecture_stm32f1xx   // example device definition for current architecture (Disable line if not desired!)
#endif
//InsertDefines above (DO NOT DELETE THIS LINE!)

//InsertEnums above (DO NOT DELETE THIS LINE!)

#define TTC_INTERRUPT_GPIO_AMOUNT  16 // CortexM3 provides 16 lines for external gpio-interrupts
#define TTC_INTERRUPT_USART_AMOUNT 5
#ifdef STM32F10X_XL
    #define TTC_INTERRUPT_TIMER_AMOUNT 11
#else
    #define TTC_INTERRUPT_TIMER_AMOUNT 5
#endif
#define TTC_INTERRUPT_SPI_AMOUNT 2
#define TTC_INTERRUPT_CAN_AMOUNT 2
#define TTC_INTERRUPT_I2C_AMOUNT 2
//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Includes *******************************************************************

#include "../ttc_basic.h"
#include "../register/register_stm32f1xx_types.h"
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Structures/ Enums required by ttc_interrupt_types.h ************************

typedef struct { // register description (adapt according to stm32f1xx registers)
    unsigned Reserved1 : 16;
    unsigned Reserved2 : 16;
} t_interrupt_gpio_register;

typedef struct {  // stm32f1xx specific configuration data of single INTERRUPT device
    t_register_stm32f1xx_exti* regEXTI;
} __attribute__( ( __packed__ ) ) t_interrupt_stm32f1xx_config;

typedef struct {  // stm32l1xx specific interrupt configuration data of single device
    volatile t_register_stm32f1xx_usart* BaseRegister;       // base address of device registers
    struct { // Bit Banding Addresses of all Interrupt Enable Bits in USARTx_BASE->CR1
        volatile t_u32* CTSIE;
        volatile t_u32* IDLEIE;
        volatile t_u32* TCIE;
        volatile t_u32* TXEIE;
        volatile t_u32* RXNEIE;
        volatile t_u32* LBDIE;
        volatile t_u32* PEIE;
        volatile t_u32* EIE;
    } EnableBits;
} __attribute__( ( __packed__ ) ) t_interrupt_stm32l1xx_config_usart;

// t_ttc_interrupt_architecture is required by ttc_interrupt_types.h
#define t_ttc_interrupt_architecture t_interrupt_stm32f1xx_config
#define t_interrupt_driver_config_usart t_interrupt_stm32l1xx_config_usart

//} Structures/ Enums


#endif //INTERRUPT_STM32F1XX_TYPES_H
