#ifndef interrupt_types_h
#define interrupt_types_h

/** { interrupt_types.h *********************************************************
 *
 *                          The ToolChain
 *
 *  Created from template ttc-lib/templates/new_file.h revision 11 at 20191210 13:20:12 UTC
 *
 *  Authors: gregor
 *
 *  Basic datatypes for all interrupt_* low-level drivers
}*/
/** { Includes *********************************************************
 *
 * Put all includes here that are required to include this header file.
 * Do NOT add includes required to compile interrupt_types.c here!
 */

#include "../ttc_basic_types.h" // basic datatypes
//InsertIncludes above (DO NOT DELETE THIS LINE!)
//}Includes
/** { Constant Defines / Macros ****************************************
 *
 * As a convention, constant definitions are all UPPERCASED to distinguish
 * them from variables and functions.
 *
 * Examples:

 #define ABC 1
 #define calculate(A,B) (A+B)
 */

//}ConstantDefines
//{ Enums ****************************************************************
typedef enum {   // e_ttc_interrupt_errorcode
    ec_interrupt_OK,                    // =0: no error
    ec_interrupt_NotEgnoughResources,   // out of interrupt lines for requested source
    ec_interrupt_TimeOut,               // timeout occured in called function
    ec_interrupt_IrqSourceNotFound,     // adressed interrupt source not available in current uC
    ec_interrupt_NotInitialized,        // an interrupt has occured that was not initialized before
    ec_interrupt_NotImplemented,        // function has no implementation for current architecture
    ec_interrupt_InvalidArgument,       // general argument error
    ec_interrupt_InvalidImplementation, // Your code does not behave as expected

    ec_interrupt_unknownError
} e_ttc_interrupt_errorcode;
typedef enum { // e_ttc_interrupt_type (not all sources listed here may be supported by current architecture driver)
    E_ttc_interrupt_type_None,

    // Note: Do not change order of following lines!
    // Note: Keep in sync with ttc_interrupt_init()!

    E_ttc_interrupt_type_First_GPIO,   //{ interrupt type for general purpose IO

    // interrupt types supported for GPIO pins
    // Note: PhysicalIndex = ttc_interrupt_gpio_create_index8(GPIOx, Pin)
    // Note: Don't forget to configure your port pin as input!
    E_ttc_interrupt_type_GPIO_Falling,            // voltage on input pin falls from logical 1 to 0
    E_ttc_interrupt_type_GPIO_Rising,             // voltage on input pin rises from logical 0 to 1
    E_ttc_interrupt_type_GPIO_Rising_Falling,     // voltage on input pin falls or rises
    E_ttc_interrupt_type_GPIO_Active_High,        // voltage on input pin is high
    E_ttc_interrupt_type_GPIO_Active_Low,         // voltage on input pin is low

    E_ttc_interrupt_type_Last_GPIO, //}
    E_ttc_interrupt_type_First_I2C,    //{ interrupt types for I2C-bus

    E_ttc_interrupt_type_I2C_StartBit,
    E_ttc_interrupt_type_I2C_AddrSent,
    E_ttc_interrupt_type_I2C_10_HDR,
    E_ttc_interrupt_type_I2C_Stop,
    E_ttc_interrupt_type_I2C_DB_TF,
    E_ttc_interrupt_type_I2C_RxNE,
    E_ttc_interrupt_type_I2C_TxE,
    E_ttc_interrupt_type_I2C_BusError,
    E_ttc_interrupt_type_I2C_ArbitrationLoss,
    E_ttc_interrupt_type_I2C_AckFail,
    E_ttc_interrupt_type_I2C_Overrun,
    E_ttc_interrupt_type_I2C_PECError,
    E_ttc_interrupt_type_I2C_Timeout,
    E_ttc_interrupt_type_I2C_SMBUSAlert,

    E_ttc_interrupt_type_Last_I2C, //}
    E_ttc_interrupt_type_First_TIMER,  //{ interrupt types for timers

    E_ttc_interrupt_type_TIMER_Updating,         // Hardware Timer expired, update needed
    E_ttc_interrupt_type_TIMER_CC1,              // Channel 1 Capture/Compare Flag
    E_ttc_interrupt_type_TIMER_CC2,              // Channel 2 Capture/Compare Flag
    E_ttc_interrupt_type_TIMER_CC3,              // Channel 3 Capture/Compare Flag
    E_ttc_interrupt_type_TIMER_CC4,              // Channel 4 Capture/Compare Flag
    E_ttc_interrupt_type_TIMER_Triggering,       // Trigger Interrupt Flag
    E_ttc_interrupt_type_TIMER_Break,            // Hardware break required

    E_ttc_interrupt_type_Last_TIMER, //}
    E_ttc_interrupt_type_First_RADIO,  //{ interrupt types for external or internal radio transceivers

    E_ttc_interrupt_type_Last_RADIO, //}
    E_ttc_interrupt_type_First_SPI,   //{ interrupt types for SPI bus

    E_ttc_interrupt_type_Last_SPI, //}
    E_ttc_interrupt_type_First_USART, //{ interrupt types for universal serial asynchronous receiver transmitter

    E_ttc_interrupt_type_USART_All,               // apply to all USART interrupts (only for disabling!)
    E_ttc_interrupt_type_USART_Cts,               // Cts signal has been received
    E_ttc_interrupt_type_USART_Error,             // error condition has arised (detailed data is handed to error handler)
    E_ttc_interrupt_type_USART_Idle,              // deivce has gone idle
    E_ttc_interrupt_type_USART_TxComplete,        // transfer completed
    E_ttc_interrupt_type_USART_TransmitDataEmpty, // transmit buffer ist empty ( bytes sent)
    E_ttc_interrupt_type_USART_RxNE,              // receive buffer not empty  ( >= 1 byte received)
    E_ttc_interrupt_type_USART_LinBreak,          // break condition detected on LIN bus

    E_ttc_interrupt_type_Last_USART, //}

    E_ttc_interrupt_type_First_CAN,    //{ interrupt types for CAN-bus
    E_ttc_interrupt_type_CAN_Transmit_Empty,
    E_ttc_interrupt_type_CAN_FIFO0_Pending,
    E_ttc_interrupt_type_CAN_FIFO0_Full,
    E_ttc_interrupt_type_CAN_FIFO0_Overrun,
    E_ttc_interrupt_type_CAN_FIFO1_Pending,
    E_ttc_interrupt_type_CAN_FIFO1_Full,
    E_ttc_interrupt_type_CAN_FIFO1_Overrun,
    E_ttc_interrupt_type_CAN_Error_Warning,
    E_ttc_interrupt_type_CAN_Error_Passive,
    E_ttc_interrupt_type_CAN_Bus_Off,
    E_ttc_interrupt_type_CAN_WakeUp_State,
    E_ttc_interrupt_type_CAN_Sleep_State,
    E_ttc_interrupt_type_Last_CAN, //}


    E_ttc_interrupt_type_First_USB, //[interrupt types from USB-bus
    E_ttc_interrupt_type_USB_OTG__FS_global,
    E_ttc_interrupt_type_Last_USB,//}

    E_ttc_interrupt_type_First_RTC, //{ interrupt types for universal serial asynchronous receiver transmitter

    E_ttc_interrupt_type_RTC_All,           // apply to all USART interrupts (only for disabling!)
    E_ttc_interrupt_type_RTC_TimeStamp,     // Cts signal has been received
    E_ttc_interrupt_type_RTC_WakeUP,        // error condition has arised (detailed data is handed to error handler)
    E_ttc_interrupt_type_RTC_AlarmA,        // deivce has gone idle
    E_ttc_interrupt_type_RTC_AlarmB,        // transfer completed
    E_ttc_interrupt_type_RTC_Tamper,        // transmit buffer ist empty ( bytes sent)
    E_ttc_interrupt_type_RTC_Pin_line_wakeup,
    E_ttc_interrupt_type_Last_RTC, //}

    // additional types of sources go here...

    E_ttc_interrupt_type_Error
} e_ttc_interrupt_type;
typedef enum {    // e_ttc_interrupt_architecture  types of architectures supported by INTERRUPT driver
    E_ttc_interrupt_architecture_none,      // no architecture selected
    E_ttc_interrupt_architecture_cortexm3,  // automatically added by ./create_DeviceDriver.pl
    E_ttc_interrupt_architecture_stm32f1xx, // automatically added by ./create_DeviceDriver.pl
    E_ttc_interrupt_architecture_stm32w1xx, // automatically added by ./create_DeviceDriver.pl
    E_ttc_interrupt_architecture_stm32l1xx, // automatically added by ./create_DeviceDriver.pl
    //InsertArchitectureEnum  above (DO NOT DELETE THIS LINE!)

    E_ttc_interrupt_architecture_unknown        // architecture not supported
} e_ttc_interrupt_architecture;

//InsertEnums above (DO NOT DELETE THIS LINE!)

//}Enums
/** { Structure Declarations *******************************************
 *
 * This can be typedef struct or typedef enum declarations.
 * Always prefix structures and enums with filename to avoid double declarations!
 */

// datatype used to enumerate physical interrupt devices (E.g. USART1, USART2, ...)
#define t_physical_index t_base

typedef struct { // t_ttc_interrupt_config
    e_ttc_interrupt_type Type;             // interrupt source type as given to _init()
    t_u8                 ConfigIndex;      // index of this entry in ttc_interrupt_config_DEVICE[]
} t_ttc_interrupt_config;

//}StructureDeclarations


#endif //interrupt_types_h
