#ifndef interrupt_usart_types_h
#define interrupt_usart_types_h

/** { interrupt_usart_types.h *********************************************************
 *
 *                          The ToolChain
 *
 *  Created from template ttc-lib/templates/new_file.h revision 11 at 20191210 12:42:46 UTC
 *
 *  Authors: gregor rebel
 *
 *  USART specific types and structures for ttc_interrupt.
}*/
/** { Includes *********************************************************
 *
 * Put all includes here that are required to include this header file.
 * Do NOT add includes required to compile interrupt_usart_types.c here!
 */

#include "../ttc_basic_types.h" // basic datatypes
#include "interrupt_types.h"
//InsertIncludes above (DO NOT DELETE THIS LINE!)
//}Includes
/** { Constant Defines / Macros ****************************************
 *
 * As a convention, constant definitions are all UPPERCASED to distinguish
 * them from variables and functions.
 *
 * Examples:

 #define ABC 1
 #define calculate(A,B) (A+B)
 */

//}ConstantDefines
/** { Structure Declarations *******************************************
 *
 * This can be typedef struct or typedef enum declarations.
 * Always prefix structures and enums with filename to avoid double declarations!
 *
 * Examples:

   typedef struct interrupt_usart_types_list_s {
      struct interrupt_usart_types_list_s* Next;
      t_base Value;
   } interrupt_usart_types_list_t;

   typedef enum {
     interrupt_usart_types_None,
     interrupt_usart_types_Type1
   } e_interrupt_usart_types_types;

 */

typedef struct { // t_ttc_interrupt_usart_errors - support for ttc_interrupt_types.h
    unsigned Overrun  : 1;
    unsigned Noise    : 1;
    unsigned Framing  : 1;
    unsigned Parity   : 1;
    unsigned reserved : 4;

} t_ttc_interrupt_usart_errors;

typedef struct s_ttc_interrupt_source_usart { // description of a single interrupt source list item
    struct s_ttc_interrupt_source_usart* Next;    // interrupt sources can be linked together in a list
    t_ttc_interrupt_config               Config;  // interrupt source configuration
} t_ttc_interrupt_source_usart;

typedef struct s_ttc_interrupt_config_usart { // t_ttc_interrupt_config_usart - pointers of interrupt service routines (-> STM32_RM0008 p. 788)
    t_physical_index PhysicalIndex;    // physical index of USART unit

    // references to interrupt service routines being called
    // Note: last argument shall be of type t_ttc_usart_config* which cannot be used here without including ttc_usart_types.h
    //       -> void ( *isr_XXX )( t_physical_index, t_ttc_usart_config* );
    void ( *isr_ClearToSend )( t_physical_index,          void* );
    void ( *isr_IdleLine )( t_physical_index,             void* );
    void ( *isr_TransmitDataEmpty )( t_physical_index,    void* );
    void ( *isr_ReceiveDataNotEmpty )( t_physical_index,  void* );
    void ( *isr_TransmissionComplete )( t_physical_index, void* );
    void ( *isr_LinBreakDetected )( t_physical_index,     void* );
    void ( *isr_Error )( t_physical_index, t_ttc_interrupt_usart_errors, void* );

    // arguments passed to isr_XXX()
    void* Argument_ClearToSend;
    void* Argument_IdleLine;
    void* Argument_TransmitDataEmpty;
    void* Argument_ReceiveDataNotEmpty;
    void* Argument_TransmissionComplete;
    void* Argument_LinBreakDetected;
    void* Argument_Error;

    /* head of single linked list of interrupt sources
     * Each _ttc_interrupt_init_usart() call with a new e_ttc_interrupt_type will add a new entry to this list.
     * _ttc_interrupt_init_usart() will return a pointer to one item of this list. This allows to enable/disable
     * individual USART interrupts.
     */
    t_ttc_interrupt_source_usart* Sources;

    // Low-Level Architecture dependent Interrupt Configuration
    // (Must be defined by low-level driver to something usefull or simply void*)
    t_interrupt_driver_config_usart LowLevel;

} t_ttc_interrupt_config_usart;
//}StructureDeclarations
/** { Function Declarations ********************************************
 *
 * Declare all functions here that may be called from outside.
 * Rules of Good Code:
 * - document every function declaration
 * - prefix public function names with their filename to avoid double definitions
 */
// { Example declaration
/** calculates the sum of A and B
 *
 * @param A  unsigned integer
 * @param B  unsigned integer
 * @return   A + B
 */
// t_u8 interrupt_usart_types_calculate(t_u16 A, t_u16 B);
//}


//InsertFunctionDeclarations above (DO NOT DELETE THIS LINE!)
//}Function Declarations

#endif //interrupt_usart_types_h
