/** light_common.c ***********************************************{
 *
 *                          The ToolChain
 *
 *  Common source code available to light low-level drivers of all architectures.
 *
 *  Created from template ttc-lib/templates/device_common.c revision 13 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
}*/

/** Includes ********************************************************{
 *
 * Put all includes here that are required to compile this source code.
 */

#include "light_common.h"
#include "../interfaces/ttc_light_interface.h" // allows to call low-level driver functions via  _driver_light_*()
#include "../ttc_heap.h"      // dynamic memory and safe arrays  
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//}Includes
/** Global Variables ************************************************{
 *
 * Always prefix your global variables to reflect the filename to which they belong to!
 * Example:
 * file:     my_cool_file.c
 * variable: t_u8 mcf_Index = 1;
 */

//InsertGlobalVariables above (DO NOT DELETE THIS LINE!)

//}GlobalVariables
/** Private Function Declarations ***********************************{
 *
 * Declare all functions here that should not be called from outside.
 * You may prefix private functions by an underscore to help identify them as private:
 * E.g.: void _privateCalc();
 */

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)
//}PrivateFunctions
/** Function Definitions ********************************************{
 */

t_ttc_light_channel_config* light_common_get_channel_configuration( t_ttc_light_config* Config, t_u8 ChannelIndex ) {
    Assert_LIGHT_Writable( Config, E_ttc_assert_origin_auto ); // always check incoming pointer arguments
    Assert_LIGHT( ( ChannelIndex > 0 ) && ( ChannelIndex <= Config->AmountChannels ), E_ttc_assert_origin_auto ); // must be 1..Config->AmountChannels. Check implementation of caller!
    t_ttc_light_channel_config* Result = NULL;

    if ( Config->Light == NULL ) { // first call: allocate structure
        Assert_LIGHT( Config->AmountChannels > 0, E_ttc_assert_origin_auto ); // low-level function being called from ttc_light_interface_load_defaults() should have set this!

        t_u32 Size = 0;
        e_ttc_light_type Type = 0;

        switch ( Config->AmountChannels ) {
            case 1: Size = sizeof( t_ttc_light_channels_4_config ); Type = E_ttc_light_type_Channels_1; break;
            case 2: Size = sizeof( t_ttc_light_channels_4_config ); Type = E_ttc_light_type_Channels_2; break;
            case 3: Size = sizeof( t_ttc_light_channels_4_config ); Type = E_ttc_light_type_Channels_3; break;
            case 4: Size = sizeof( t_ttc_light_channels_4_config ); Type = E_ttc_light_type_Channels_4; break;
            default: ttc_assert_halt_origin( E_ttc_assert_origin_auto ); break; // invalid/unsupported amount of channels. Check implementation of low-level light_*_load_defaults() function!
        }
        Config->Light = ttc_heap_alloc_zeroed( Size );
        Config->Light->Type = Type;
    }
    if ( ChannelIndex <= Config->AmountChannels ) {
        Assert_LIGHT_Writable( Config->Light, E_ttc_assert_origin_auto );
        Result = &( Config->Light->channels4.Channel[ChannelIndex - 1] );
    }

    return Result; // t_ttc_light_channel_config*
}
//InsertFunctionDefinitions above (DO NOT DELETE THIS LINE!)
//}FunctionDefinitions
/** Private Functions ***********************************************{
 *
 * Private functions are declared in this c-file. They can be called from outside
 * via an extern declaration (or by ignoring the corresponding compiler warning).
 * Though it is possible to call them from outside, it is not intended to do so!
 * Private functions may be changed, renamed or deleted without notification by
 * the maintainer of this file.
 */

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Private Functions
