#ifndef LIGHT_GPIO_H
#define LIGHT_GPIO_H

/** { light_gpio.h **********************************************
 *
 *                               The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-level driver for ttc_light.h providing
 *  function prototypes being required by ttc_light.h
 *
 *  control multiple light emitting diodes (led)
 *
 *  Created from template device_architecture.h revision 32 at 20191218 12:21:38 UTC
 *
 *  Note: See ttc_light.h for description of gpio independent LIGHT implementation.
 *
 *  Authors: Gregor Rebel
 *
 */
/** Description of light_gpio (Do not delete this line!)
 *
 * Each LIGHT is configured by constant defines in a makefile
 * (Replace <LogicalIndex> by 1..ttc_light_get_max_index()!)
 * Multi channel lights like full color rgb leds require three gpio pins TTC_LIGHT<LogicalIndex>_GPIO1..TTC_LIGHT<LogicalIndex>_GPIO3.
 *
 * H2: Mandatory Constant Defines
 * *) TTC_LIGHT<LogicalIndex>
 *    Architecture driver to use.
 *    ==E_ttc_light_architecture_gpio for gpio controlled lights (mostly LEDs).
 * *) TTC_LIGHT<LogicalIndex>_GPIO1
 *    GPIO pin being connected to first (or only) channel of light device.
 *    == one from e_ttc_gpio_pin.
 *
 * H2: Optional Constant Defines
 * *) TTC_LIGHT<LogicalIndex>_GPIO2
 * *) TTC_LIGHT<LogicalIndex>_GPIO3
 * *) TTC_LIGHT<LogicalIndex>_GPIO4
 *    GPIO pin being connected to channels 2..4 of light.
 *    Each channel typically represent a color (red,green,blue,white).
 *    == one from e_ttc_gpio_pin.
 * *) TTC_LIGHT<LogicalIndex>_LOWACTIVE
 *    Default logic of all channels of this light.
 *    ==1: gpio pins have to be pulled low to activate LED
 *    ==0: ...pushed high...
 *    Default: 0
 * *) TTC_LIGHT<LogicalIndex>_MODE_GPIO
 *    Special pin mode to use for gpio pins of all channels of this light (One from e_ttc_gpio_mode)
 *    Default: E_ttc_gpio_mode_output_push_pull
 *
 *
 * H2: Example Configuration of two LEDs
 *
 * COMPILE_OPTS += -DTTC_LIGHT1=E_ttc_light_architecture_gpio
 * COMPILE_OPTS += -DTTC_LIGHT1_GPIO1=E_ttc_gpio_pin_c6
 * COMPILE_OPTS += -DTTC_LIGHT1_LOWACTIVE=0
 * COMPILE_OPTS += -DTTC_LIGHT1_MODE_GPIO=E_ttc_gpio_mode_output_push_pull
 * COMPILE_OPTS += -DTTC_LIGHT2=E_ttc_light_architecture_gpio
 * COMPILE_OPTS += -DTTC_LIGHT2_GPIO1=E_ttc_gpio_pin_c7
 * COMPILE_OPTS += -DTTC_LIGHT2_LOWACTIVE=1
 * COMPILE_OPTS += -DTTC_LIGHT2_MODE_GPIO=E_ttc_gpio_mode_output_open_drain
}*/
//{ Defines/ TypeDefs ****************************************************

#undef EXTENSION_LED_DRIVER_AVAILABLE
#define EXTENSION_LED_DRIVER_AVAILABLE // signal existence of at least one low-level driver (checked by ttc_light_interface.c)

//InsertDefines above (DO NOT DELETE THIS LINE!)

//InsertEnums above (DO NOT DELETE THIS LINE!)

//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Includes *************************************************************
//
// Header files include all header files that are required to include this
// header file. Includes that provide function declarations should be placed
// in "light_gpio.c"
//
#include "../ttc_light_types.h" // will include light_gpio_types.h (do not include it directly!)
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Macro definitions ****************************************************

// define all supported low-level functions for ttc_light_interface.h
//
// Some driver functions may be optional to implement.
// Check ttc_light_interface.c for default implementations.
// ttc_driver_*() functions not being implemented must be undefined via #undef
// Example: #undef ttc_driver_light_foo
//
#define ttc_driver_light_configuration_check light_gpio_configuration_check
#define ttc_driver_light_deinit light_gpio_deinit
#define ttc_driver_light_init light_gpio_init
#define ttc_driver_light_prepare light_gpio_prepare
#define ttc_driver_light_reset light_gpio_reset
#define ttc_driver_light_off light_gpio_off
#define ttc_driver_light_on light_gpio_on
#undef ttc_driver_light_load_defaults
#define ttc_driver_light_brightness1 light_gpio_brightness1
#define ttc_driver_light_brightness2 light_gpio_brightness2
#define ttc_driver_light_brightness3 light_gpio_brightness3
#define ttc_driver_light_brightness4 light_gpio_brightness4
#define ttc_driver_light_brightness_update light_gpio_brightness_update
//InsertLowLevelMacros above (DO NOT DELETE THIS LINE!)

//} Macro definitions
/** Function prototypes *************************************************{
 *
 * The prototypes listed below have been created automatically by create_Driver.pl based
 * on section "Prototypes of low-level driver functions" in file ttc_light.h.
 * You may remove all prototypes here and have them reconstructed automatically.
 * Check file ttc_light.h for details.
 *
 * You normally should not need to add functions here.
 * Add your private functions in the private section below.
 */


/** Checks + corrects entries in given Config to meet limits of current architecture
 *
 * Note: High-level function ttc_light_configuration_check() has already checked Config against
 *       all fields being found in t_ttc_light_features.
 *
 * @param Config        Configuration of light device
 * @return              Fields of *Config have been adjusted if necessary
 */
void light_gpio_configuration_check( t_ttc_light_config* Config );


/** shutdown single LIGHT unit device
 * @param Config        Configuration of light device
 * @return              == 0: LIGHT has been shutdown successfully; != 0: error-code
 */
e_ttc_light_errorcode light_gpio_deinit( t_ttc_light_config* Config );


/** initializes single LIGHT unit for operation
 * @param Config        Configuration of light device
 * @return              == 0: LIGHT has been initialized successfully; != 0: error-code
 */
e_ttc_light_errorcode light_gpio_init( t_ttc_light_config* Config );


/** loads configuration of indexed LIGHT unit with default values
 *
 * Note: Must allocate Config->LowLevelConfig according to connected light device.
 *
 *
 * @param Config        Configuration of light device
 * @return  == 0:       configuration was loaded successfully
 */
e_ttc_light_errorcode light_gpio_load_defaults( t_ttc_light_config* Config );


/** Prepares light Driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void light_gpio_prepare();


/** reset configuration of indexed device and connected hardware
 *
 * @param Config        Configuration of light device
 */
void light_gpio_reset( t_ttc_light_config* Config );


/** switch off single led
 *
 * Config->Brightness will not be changed
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)               Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void light_gpio_off( t_ttc_light_config* Config );


/** switch on single light to preconfigured brightness level
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)               Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void light_gpio_on( t_ttc_light_config* Config );


/** set new brightness levels for first channel only
 *
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Level        (t_u8)                 new brightness level for channel 1
 */
void light_gpio_brightness1( t_ttc_light_config* Config, t_u8 Level );


/** set new brightness levels for first two channels at once
 *
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Level1       (t_u8)                 new brightness level for channel 1
 * @param Level2       (t_u8)                 new brightness level for channel 2
 */
void light_gpio_brightness2( t_ttc_light_config* Config, t_u8 Level1, t_u8 Level2 );


/** set new brightness levels for first three channels at once
 *
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Red          (t_u8)                 new brightness level for channel 1
 * @param Blue         (t_u8)                 new brightness level for channel 2
 * @param Green        (t_u8)                 new brightness level for channel 3
 */
void light_gpio_brightness3( t_ttc_light_config* Config, t_u8 Red, t_u8 Green, t_u8 Blue );


/** set new brightness levels for first four channels at once
 *
 * Note: High level driver will copy new brightness levels into Config->Light->channels2.Channel[].Init.Brightness after calling low-level driver.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 * @param Red          (t_u8)                 new brightness level for channel 1
 * @param Blue         (t_u8)                 new brightness level for channel 2
 * @param Green        (t_u8)                 new brightness level for channel 3
 * @param White        (t_u8)                 new brightness level for channel 4
 */
void light_gpio_brightness4( t_ttc_light_config* Config, t_u8 Red, t_u8 Green, t_u8 Blue, t_u8 White );


/** Sends latest Brightness levels in all channels to light controller
 *
 * Call this function to update brightness after changing individual channel brightness values in Config->Light->channels4!
 * Low-level driver will update light brightness in all channels.
 *
 * @param Config       (t_ttc_light_config*)  Configuration of light device as returned by ttc_light_get_configuration()
 * @param LogicalIndex (t_u8)                 Logical index of light instance. Each logical device <n> is defined via COMPILE_OPTS += -DTTC_LIGHT<n> lines in extensions.active/makefile
 */
void light_gpio_brightness_update( t_ttc_light_config* Config );

//InsertFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//} Function prototypes

#endif //LIGHT_GPIO_H
