/** { ttc_interrupt_types.h ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level datatype definitions for INTERRUPT device.
 *  Structures, Enums and Defines being required by both, high- and low-level interrupt.
 *
 *  Created from template ttc_device_types.h revision 22 at 20140224 21:37:20 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

// inform create_DeviceDriver.pl where to find its insertion mark
//InsertArchitectureEnum  moved_to 'interrupt/interrupt_types.h'

#ifndef TTC_INTERRUPT_TYPES_H
#define TTC_INTERRUPT_TYPES_H

// handle used to identify an initialized interrupt
typedef void* t_ttc_interrupt_handle;

//{ Includes *************************************************************

#include "ttc_basic_types.h"
#ifndef TTC_BASIC_H
    #include "compile_options.h"
#endif
#include "interrupt/interrupt_types.h"

// Low-Level Architecture Drivers for Interrupt Datatypes
#ifdef EXTENSION_interrupt_stm32w1xx
    #include "interrupt/interrupt_stm32w1xx_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
#ifdef EXTENSION_interrupt_stm32f1xx
    #include "interrupt/interrupt_stm32f1xx_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
#ifdef EXTENSION_interrupt_cortexm3
    #include "interrupt/interrupt_cortexm3_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
#ifdef EXTENSION_interrupt_stm32f30x
    #include "interrupt/interrupt_stm32f30x_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
#ifdef EXTENSION_interrupt_stm32l1xx
    #include "interrupt/interrupt_stm32l1xx_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
//InsertTypesIncludes above (DO NOT DELETE THIS LINE!)

// High-Level Datatypes for individual interrupt enabled devices
#ifdef EXTENSION_ttc_usart
    #include "interrupt/interrupt_usart_types.h"
    #ifndef TTC_INTERRUPT_USART_AMOUNT // amount of interrupt lines from USART devices
        #warning Missing value for TTC_INTERRUPT_USART_AMOUNT
        #define TTC_INTERRUPT_USART_AMOUNT 0 // TC_USART_MAX_AMOUNT
    #endif
#endif
#ifdef EXTENSION_ttc_gpio
    #include "ttc_gpio_types.h"
    #ifndef TTC_INTERRUPT_GPIO_AMOUNT // amount of interrupt lines from GPIO devices
        #warning Missing value for TTC_INTERRUPT_GPIO_AMOUNT
        #define TTC_INTERRUPT_GPIO_AMOUNT 0
    #endif
#endif
#ifdef EXTENSION_ttc_timer //{
    #include "ttc_timer_types.h"
    #ifndef TTC_INTERRUPT_TIMER_AMOUNT // amount of interrupt lines from TIMER devices
        #warning Missing value for TTC_INTERRUPT_TIMER_AMOUNT
        #define TTC_INTERRUPT_TIMER_AMOUNT 0
    #endif
#endif //}
#ifdef EXTENSION_ttc_can //{
    #include "ttc_can_types.h"
    #ifndef TTC_INTERRUPT_CAN_AMOUNT // amount of interrupt lines from CAN devices
        #warning Missing value for TTC_INTERRUPT_CAN_AMOUNT
        #define TTC_INTERRUPT_CAN_AMOUNT 0
    #endif
#endif //}
#ifdef EXTENSION_ttc_i2c
    #include "ttc_i2c_types.h"
    #ifndef TTC_INTERRUPT_I2C_AMOUNT // amount of interrupt lines from I2C devices
        #warning Missing value for TTC_INTERRUPT_I2C_AMOUNT
        #define TTC_INTERRUPT_I2C_AMOUNT 0 // TC_I2C_MAX_AMOUNT
    #endif
#endif
#ifdef EXTENSION_ttc_spi
    #include "ttc_spi_types.h"
    #ifndef TTC_INTERRUPT_SPI_AMOUNT // amount of interrupt lines from SPI devices
        #warning Missing value for TTC_INTERRUPT_SPI_AMOUNT
        #define TTC_INTERRUPT_SPI_AMOUNT 0 // TC_SPI_MAX_AMOUNT
    #endif
#endif
#ifdef EXTENSION_ttc_rtc
    #include "ttc_rtc_types.h"
    #ifndef TTC_INTERRUPT_RTC_AMOUNT // amount of interrupt lines from RADIO devices
        #warning Missing value for TTC_INTERRUPT_RTC_AMOUNT
        #define TTC_INTERRUPT_RTC_AMOUNT 0 // TC_RTC_MAX_AMOUNT
    #endif
#endif

#include "ttc_list_types.h"

//} Includes
//{ Defines **************************************************************


//InsertDefines above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Static Configuration *************************************************

/** Check definition that have to be set by low-level driver.. */

/* Device specific Assert function
     *
     * The use of Assert() is the basic key for stable code in embedded software.
     * Every function should check all given arguments for plausibility.
     * Calling Assert() with a zero (false) argument will hold program execution in an endless loop.
     * This prevents any possibly dangerous operations.
     *
     * Every device driver provides its own Assert_{DEVICE}() function.
     * This allows to switch off the use of assert code for each device driver from your makefile.
     * Simply add the following line to your makefile.700_extra_settings to see a slight performance improvement:
     *   COMPILE_OPTS += -DTTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT 0  # disable assert handling for interrupt devices
     *
     */
#ifndef TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT    /* ==1: enable assert macros for this driver; ==0: disable (faster but no self checks) */
    #define TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT 1  // default asserts are enabled by default
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT == 1)  // use Assert()s in INTERRUPT code (somewhat slower but alot easier to debug)
    #define Assert_INTERRUPT(Condition, Origin)          Assert( ((Condition) != 0), Origin)
    #define Assert_INTERRUPT_Writable(Address, Origin)   Assert_Writable(Address, Origin)
    #define Assert_INTERRUPT_Readable(Address, Origin)   Assert_Readable(Address, Origin)
    #define Assert_INTERRUPT_Executable(Address, Origin) Assert_Executable(Address, Origin)
    #define IF_Assert_INTERRUPT(Code) Code
#else  // use no default Assert()s in INTERRUPT code (somewhat smaller + faster, but crashes are hard to debug)
    #define Assert_INTERRUPT(Condition, Origin)
    #define Assert_INTERRUPT_Writable(Address, Origin)
    #define Assert_INTERRUPT_Readable(Address, Origin)
    #define Assert_INTERRUPT_Executable(Address, Origin)
    #define IF_Assert_INTERRUPT(Code)
#endif


#ifndef TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT_EXTRA    /* Extra Assert macros provide self tests that can be usefull while developing 
                                                         * high- or low-level driver code for ttc_interrupt. After successfull completion,
                                                         * these extra checks can be disabled.
                                                         * ==1: enable extra assert macros for ttc_interrupt driver; 
                                                         * ==0: disable (faster but no self checks)
                                                         */
#define TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT_EXTRA 1  // define as zero after completing your driver!
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_INTERRUPT_EXTRA == 1)  // use Assert()s in INTERRUPT code (somewhat slower but alot easier to debug)
    #define Assert_INTERRUPT_EXTRA(Condition, Origin)        Assert( ((Condition) != 0), Origin)
    #define Assert_INTERRUPT_EXTRA_Writable(Address, Origin) Assert_Writable(Address, Origin)
    #define Assert_INTERRUPT_EXTRA_Readable(Address, Origin) Assert_Readable(Address, Origin)
#else  // use no extra Assert()s in INTERRUPT code (somewhat smaller + faster, but crashes are hard to debug)
    #define Assert_INTERRUPT_EXTRA(Condition, Origin)
    #define Assert_INTERRUPT_EXTRA_Writable(Address, Origin)
    #define Assert_INTERRUPT_EXTRA_Readable(Address, Origin)
#endif

/** Architecture dependent device configuration
     *
     * Each low-level driver may define this structure to increase visibility of t_ttc_interrupt_config during debugging.
     * Simply copy the define line below into your _types.h and change void* to any structure you like.
     */
#ifndef t_ttc_interrupt_architecture
    #warning Missing low-level definition for t_ttc_interrupt_architecture (using default)
    #define t_ttc_interrupt_architecture void
#endif

// list of function pointers to initialization functions of this type:
// e_ttc_interrupt_errorcode (*interrupt_init)(e_ttc_interrupt_type Source, void* ISR);
// each architecture driver (GPIO, I2C, ...) may define one init() and multiple enums (-> e_ttc_interrupt_type).
#ifndef TTC_INTERRUPT_INIT_GPIO
    #define TTC_INTERRUPT_INIT_GPIO NULL
#endif
#ifndef TTC_INTERRUPT_INIT_I2C
    #define TTC_INTERRUPT_INIT_I2C NULL
#endif
#ifndef TTC_INTERRUPT_INIT_TIMER
    #define TTC_INTERRUPT_INIT_TIMER NULL
#endif
#ifndef TTC_INTERRUPT_INIT_RADIO
    #define TTC_INTERRUPT_INIT_RADIO NULL
#endif
#ifndef TTC_INTERRUPT_INIT_SPI
    #define TTC_INTERRUPT_INIT_SPI NULL
#endif
#ifndef TTC_INTERRUPT_INIT_interrupt
    #define TTC_INTERRUPT_INIT_interrupt NULL
#endif
#ifndef TTC_INTERRUPT_INIT_CAN
    #define TTC_INTERRUPT_INIT_CAN NULL
#endif

//}Static Configuration
//{ Structures ***********************************************************

typedef union { // t_ttc_interrupt_gpio_index
    t_u8 U8;
    struct {
        unsigned Bank : 4; // index of GPIO-bank to use (0 = GPIOA, ...)
        unsigned Pin  : 4; // index of GPIO-pin to use (0 = Pin1, ...)
    } B;
} __attribute__( ( __packed__ ) ) t_ttc_interrupt_gpio_index;


#ifdef EXTENSION_ttc_i2c
typedef struct { // t_ttc_interrupt_config_i2c - pointers of interrupt service routines (-> STM32_RM0008 p. 788)

    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    t_physical_index     PhysicalIndex;    // physical index of I2C unit

    // interrupt service routines being called
    void ( *isr_StartBitSent )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_AddressSent )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_10BitHeader )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_StopReceived )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_DataByteTransferFinished )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_ReceiveBufferNotEmpty )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_TransmitBufferEmpty )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_BusError )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_ArbitrationLoss )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_AckFail )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_Overrun )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_PECError )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_Timeout )( t_physical_index,              t_ttc_i2c_config* );
    void ( *isr_SMBusAlert )( t_physical_index,              t_ttc_i2c_config* );

    // arguments passed to isr_XXX()
    t_ttc_i2c_config* Argument_StartBitSent;
    t_ttc_i2c_config* Argument_AddressSent;
    t_ttc_i2c_config* Argument_10BitHeader;
    t_ttc_i2c_config* Argument_StopReceived;
    t_ttc_i2c_config* Argument_DataByteTransferFinished;
    t_ttc_i2c_config* Argument_ReceiveBufferNotEmpty;
    t_ttc_i2c_config* Argument_TransmitBufferEmpty;
    t_ttc_i2c_config* Argument_BusError;
    t_ttc_i2c_config* Argument_ArbitrationLoss;
    t_ttc_i2c_config* Argument_AckFail;
    t_ttc_i2c_config* Argument_Overrun;
    t_ttc_i2c_config* Argument_PECError;
    t_ttc_i2c_config* Argument_Timeout;
    t_ttc_i2c_config* Argument_SMBusAlert;

#ifdef t_interrupt_driver_config_i2c
    // low-level architecture dependent interrupt configuration
    t_interrupt_driver_config_i2c LowLevel;
#endif
} t_ttc_interrupt_config_i2c;
#endif

#ifdef EXTENSION_ttc_spi
typedef struct { // t_ttc_interrupt_config_spi - pointers of interrupt service routines (-> STM32_RM0008 p. 788)

    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    t_physical_index     PhysicalIndex;    // physical index of SPI unit

    // interrupt service routines being called
    void ( *isr_ChangeMe )( t_physical_index,              t_ttc_spi_config* );

    // arguments passed to isr_XXX()
    t_ttc_spi_config* Argument_ChangeMe;

#ifdef interrupt_driver_config_spi_t
    // low-level architecture dependent interrupt configuration
    interrupt_driver_config_spi_t LowLevel;
#endif
} t_ttc_interrupt_config_spi;
#endif
#ifdef EXTENSION_ttc_gpio
typedef struct { // t_ttc_interrupt_config_gpio - configuration data of single external interrupt line (-> STM32_RM0008 p. 201)
    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    e_ttc_gpio_pin PhysicalIndex;          // gpio pin index
    void ( *isr )( t_physical_index, void* ); // interrupt service routine being called
    void* Argument;                        // argument passed to isr()

#ifdef t_interrupt_driver_config_gpio  // defined by low-level driver to store additional data for each gpio pin
    t_interrupt_driver_config_gpio LowLevel;      // low-level architecture dependent interrupt configuration
#endif
} t_ttc_interrupt_config_gpio;
#endif
#ifdef EXTENSION_ttc_timer
typedef struct { // t_ttc_interrupt_config_timer - configuration data of single external interrupt line (-> STM32_RM0008 p. 201)

    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    t_physical_index     PhysicalIndex;    // physical index of USART unit

    // interrupt service routines being called
    void ( *isr_Update )( t_physical_index, t_ttc_timer_config* );

    // arguments passed to isr_XXX()
    t_ttc_timer_config* Argument_Update;

#ifdef t_interrupt_driver_config_timer
    t_interrupt_driver_config_timer LowLevel;    // low-level architecture dependent interrupt configuration
#endif

} t_ttc_interrupt_config_timer;
#endif
#ifdef EXTENSION_ttc_can
typedef struct { // t_ttc_interrupt_config_can - configuration data of single external interrupt line (-> STM32_RM0008 p. 201)

    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    t_physical_index     PhysicalIndex;    // physical index of CAN unit

    // interrupt service routine being called
    void ( *isr_Transmit )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO0_Pending )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO0_Full )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO0_Overrun )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO1_Pending )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO1_Full )( t_physical_index, t_ttc_can_config* );
    void ( *isr_FIFO1_Overrun )( t_physical_index, t_ttc_can_config* );
    void ( *isr_Error_Warning )( t_physical_index, t_ttc_can_config* );
    void ( *isr_Error_Passive )( t_physical_index, t_ttc_can_config* );
    void ( *isr_Bus_Off )( t_physical_index, t_ttc_can_config* );
    void ( *isr_WakeUp_State )( t_physical_index, t_ttc_can_config* );
    void ( *isr_Sleep_State )( t_physical_index, t_ttc_can_config* );

    // argument passed to isr()
    t_ttc_can_config* Argument_Transmit;
    t_ttc_can_config* Argument_FIFO0_Pending;
    t_ttc_can_config* Argument_FIFO0_Full;
    t_ttc_can_config* Argument_FIFO0_Overrun;
    t_ttc_can_config* Argument_FIFO1_Pending;
    t_ttc_can_config* Argument_FIFO1_Full;
    t_ttc_can_config* Argument_FIFO1_Overrun;
    t_ttc_can_config* Argument_Error_Warning;
    t_ttc_can_config* Argument_Error_Passive;
    t_ttc_can_config* Argument_Bus_Off;
    t_ttc_can_config* Argument_WakeUp_State;
    t_ttc_can_config* Argument_Sleep_State;


#ifdef interrupt_driver_config_can_t
    interrupt_driver_config_can_t LowLevel; // low-level architecture dependent interrupt configuration
#endif
} t_ttc_interrupt_config_can;
#endif


#ifdef EXTENSION_ttc_rtc
typedef struct { // t_ttc_interrupt_config_rtc - configuration data of single external interrupt line (-> STM32_RM0008 p. 201)

    t_ttc_interrupt_config Common;         // this part is common to all types of interrupt configs
    t_physical_index     PhysicalIndex;    // physical index of rtc unit

    // interrupt service routine being called
    void ( *isr_TimeStamp )( t_physical_index, t_ttc_rtc_config* );
    void ( *isr_Tamper )( t_physical_index, t_ttc_rtc_config* );
    void ( *isr_Wakeup )( t_physical_index, t_ttc_rtc_config* );
    void ( *isr_AlarmA )( t_physical_index, t_ttc_rtc_config* );
    void ( *isr_AlarmB )( t_physical_index, t_ttc_rtc_config* );


    // argument passed to isr()
    t_ttc_rtc_config* Argument_TimeStamp;
    t_ttc_rtc_config* Argument_Tamper;
    t_ttc_rtc_config* Argument_Wakeup;
    t_ttc_rtc_config* Argument_AlarmA;
    t_ttc_rtc_config* Argument_AlarmB;



#ifdef interrupt_driver_config_rtc_t
    interrupt_driver_config_rtc_t LowLevel; // low-level architecture dependent interrupt configuration
#endif
} t_ttc_interrupt_config_rtc;
#endif
//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//}Structures

#endif // TTC_INTERRUPT_TYPES_H
