/** { ttc_light_types.h ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  High-Level datatype definitions for LIGHT device.
 *  Structures, Enums and Defines being required by both, high- and low-level light.
 *  This file does not provide function declarations.
 *
 *  Note: See ttc_light.h for description of high-level light implementation!
 *
 *  Created from template ttc_device_types.h revision 46 at 20191218 12:21:38 UTC
 *
 *  Authors: Gregor Rebel
 *
}*/

#ifndef TTC_LIGHT_TYPES_H
#define TTC_LIGHT_TYPES_H

//{ Includes1 ************************************************************
//
// Includes being required for static configuration.
//
// _types.h Header files should include only other *_types.h header files to avoid circular includes.
// Includes that provide function declarations should be placed
// in "ttc_light.h" or "ttc_light.c"
//
#include "ttc_basic_types.h"    // basic datatypes
#include "ttc_gpio_types.h"     // gpio type definitions
#include "ttc_list_types.h"     // list header and list item header
#include "ttc_systick_types.h"  // universal delays for single- and multitasking mode
#include "compile_options.h"    // compile options collected from command line arguments

//} Includes

/** { Static Configuration ***********************************************
 *
 * Most ttc drivers are statically configured by constant definitions in makefiles.
 * A static configuration allows to hide certain code fragments and variables for the
 * compilation process. If used wisely, your code will be lean and fast and do only
 * what is required for the current configuration.
 */

#define LIGHT_GPIO_MAX_AMOUNT_CHANNELS 4  // maximum supported amount of individual channels for gpio controlled ttc_light device

/** {  TTC_LIGHTn has to be defined as constant in one of your makefiles
 *
 * Here we count how much devices have been defined and if all lower index devices have been defined too.
 */
#ifndef TTC_LIGHT_AMOUNT //{ 10
    #ifdef TTC_LIGHT10
        #ifndef TTC_LIGHT9
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT9 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT8
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT8 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT7
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT7 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT6
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT6 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT5
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT5 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT10 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 10
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 9
    #ifdef TTC_LIGHT9
        #ifndef TTC_LIGHT8
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT8 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT7
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT7 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT6
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT6 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT5
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT5 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT9 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 9
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 8
    #ifdef TTC_LIGHT8
        #ifndef TTC_LIGHT7
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT7 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT6
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT6 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT5
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT5 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT8 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 8
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 7
    #ifdef TTC_LIGHT7
        #ifndef TTC_LIGHT6
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT6 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT5
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT5 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT7 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 7
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 6
    #ifdef TTC_LIGHT6
        #ifndef TTC_LIGHT5
            #error TTC_LIGHT6 is defined, but not TTC_LIGHT5 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT6 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT6 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT6 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT6 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 6
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 5
    #ifdef TTC_LIGHT5
        #ifndef TTC_LIGHT4
            #error TTC_LIGHT5 is defined, but not TTC_LIGHT4 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT5 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT5 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT5 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 5
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 4
    #ifdef TTC_LIGHT4
        #ifndef TTC_LIGHT3
            #error TTC_LIGHT4 is defined, but not TTC_LIGHT3 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT2
            #error TTC_LIGHT4 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT4 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 4
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 3
    #ifdef TTC_LIGHT3

        #ifndef TTC_LIGHT2
            #error TTC_LIGHT3 is defined, but not TTC_LIGHT2 - all lower TTC_LIGHTn must be defined!
        #endif
        #ifndef TTC_LIGHT1
            #error TTC_LIGHT3 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 3
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 2
    #ifdef TTC_LIGHT2

        #ifndef TTC_LIGHT1
            #error TTC_LIGHT2 is defined, but not TTC_LIGHT1 - all lower TTC_LIGHTn must be defined!
        #endif

        #define TTC_LIGHT_AMOUNT 2
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ 1
    #ifdef TTC_LIGHT1
        #define TTC_LIGHT_AMOUNT 1
    #endif
#endif //}
#ifndef TTC_LIGHT_AMOUNT //{ no devices defined at all
    #warning Missing definition for TTC_LIGHT1. Using default type. Add valid definition to your makefile to get rid if this message!
    // option 1: no devices available
    #define TTC_LIGHT_AMOUNT 0
    // option 2: define a default device (enable lines below)
    // #define TTC_LIGHT_AMOUNT 1
    // #define TTC_LIGHT1 ??? // <- place entry from e_ttc_light_architecture here!!!
#endif

//}
//}
/** { Device specific Assert function
 *
 * The use of Assert() is the basic key for stable code in embedded software.
 * Every function should check all given arguments for plausibility.
 * Calling Assert() with a zero (false) argument will hold program execution in an endless loop.
 * This prevents any possibly dangerous operations.
 *
 * Every device driver provides two own assert functions:
 *
 * 1) The default assert is enabled by default and must be disabled explicitly.
 *    This assert is to be used for all arguments provided from outside of this driver.
 *
 * 2) The extra assert is disabled by default and must be enabled explicitly.
 *    The idea of an extra assert is to provide extra internal checks while the driver code is under development.
 *    Once the driver is assumed to be stable, extra checks can be disabled.
 *
 * Asserts for each driver can be enabled or disabled from your makefile.
 * Simply add the following line to your makefile.700_extra_settings to change assert settings:
 *
 * #1 Disable default asserts for slight performance improvement
 *    COMPILE_OPTS += -DTTC_TUNABLE_SELFCHECK_ASSERT_LIGHT 0        # disable default asserts for light driver
 *
 * #2 Enable extra checks for more intense checks
 *    COMPILE_OPTS += -DTTC_TUNABLE_SELFCHECK_ASSERT_LIGHT_EXTRA 1  # enable extra asserts for light driver
 *
 * When debugging code that has been compiled with optimization (gcc option -O) then
 * certain variables may be optimized away. Declaring variables as VOLATILE_LIGHT_ makes them
 * visible in debug session when asserts are enabled. You have no overhead at all if asserts are disabled.
 + foot_t* VOLATILE_LIGHT_ VisiblePointer;
 *
 */
#ifndef TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT    /* ==1: enable assert macros for this driver; ==0: disable (faster but no self checks) */
    #define TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT 1  // default asserts are enabled by default
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT == 1)  // use Assert()s in LIGHT code (somewhat slower but alot easier to debug)
    #define Assert_LIGHT(Condition, Origin)          Assert(Condition, Origin)
    #define Assert_LIGHT_Writable(Address, Origin)   Assert_Writable(Address, Origin)
    #define Assert_LIGHT_Readable(Address, Origin)   Assert_Readable(Address, Origin)
    #define Assert_LIGHT_Executable(Address, Origin) Assert_Executable(Address, Origin)
    #define VOLATILE_LIGHT                           volatile
#else  // use no default Assert()s in LIGHT code (somewhat smaller + faster, but crashes are hard to debug)
    #define Assert_LIGHT(Condition, Origin)
    #define Assert_LIGHT_Writable(Address, Origin)
    #define Assert_LIGHT_Readable(Address, Origin)
    #define Assert_LIGHT_Executable(Address, Origin)
    #define VOLATILE_LIGHT
#endif

#ifndef TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT_EXTRA    /* Extra Assert macros provide self tests that can be usefull while developing 
                                                     * high- or low-level driver code for ttc_light. After successfull completion,
                                                     * these extra checks can be disabled.
                                                     * ==1: enable extra assert macros for ttc_light driver; 
                                                     * ==0: disable (faster but no self checks)
                                                     */
    #define TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT_EXTRA 1  // define as zero after completing your driver!
#endif
#if (TTC_TUNABLE_SELFCHECK_ASSERT_LIGHT_EXTRA == 1)  // use Assert()s in LIGHT code (somewhat slower but alot easier to debug)
    #define Assert_LIGHT_EXTRA(Condition, Origin) Assert(Condition, Origin)
    #define Assert_LIGHT_EXTRA_Writable(Address, Origin) Assert_Writable(Address, Origin)
    #define Assert_LIGHT_EXTRA_Readable(Address, Origin) Assert_Readable(Address, Origin)
#else  // use no extra Assert()s in LIGHT code (somewhat smaller + faster, but crashes are hard to debug)
    #define Assert_LIGHT_EXTRA(Condition, Origin)
    #define Assert_LIGHT_EXTRA_Writable(Address, Origin)
    #define Assert_LIGHT_EXTRA_Readable(Address, Origin)
#endif
//}
/** { Check static configuration of light devices
 *
 * Each TTC_LIGHTn must be defined as one from e_ttc_light_architecture.
 * Additional defines of type TTC_LIGHTn_* may be added.
 *
 * The constants being checked here must be defined in your makefile.
 * Most constants are defined by EXTENSION_100_board_* or EXTENSION_150_board_extension_* files.
 */

#ifdef TTC_LIGHT1
    #ifndef TTC_LIGHT1_LOWACTIVE
        #warning Missing definition for TTC_LIGHT1_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT1_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT1_GPIO1
        #warning Missing definition for TTC_LIGHT1_GPIO1. Add a line to your makefile that defines it as one from e_ttc_gpio_pin ! Check ttc_light.h for more details!
        #define TTC_LIGHT1_GPIO1 E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT1_MAX_DUTY
        #define TTC_LIGHT1_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT2
    #ifndef TTC_LIGHT2_LOWACTIVE
        #warning Missing definition for TTC_LIGHT2_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT2_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT2_GPIO1
        #warning Missing definition for TTC_LIGHT2_GPIO1. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT2_GPIO1 E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT2_MAX_DUTY
        #define TTC_LIGHT2_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT3
    #ifndef TTC_LIGHT3_LOWACTIVE
        #warning Missing definition for TTC_LIGHT3_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT3_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT3_GPIO1
        #warning Missing definition for TTC_LIGHT3_GPIO1. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT3_GPIO1 E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT3_MAX_DUTY
        #define TTC_LIGHT3_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT4
    #ifndef TTC_LIGHT4_LOWACTIVE
        #warning Missing definition for TTC_LIGHT4_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT4_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT4_GPIO1
        #warning Missing definition for TTC_LIGHT4_GPIO1. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT4_GPIO1 E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT4_MAX_DUTY
        #define TTC_LIGHT4_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT5
    #ifndef TTC_LIGHT5_LOWACTIVE
        #warning Missing definition for TTC_LIGHT5_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT5_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT5_GPIO
        #warning Missing definition for TTC_LIGHT5_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT5_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT5_MAX_DUTY
        #define TTC_LIGHT5_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT6
    #ifndef TTC_LIGHT6_LOWACTIVE
        #warning Missing definition for TTC_LIGHT6_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT6_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT6_GPIO
        #warning Missing definition for TTC_LIGHT6_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT6_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT6_MAX_DUTY
        #define TTC_LIGHT6_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT7
    #ifndef TTC_LIGHT7_LOWACTIVE
        #warning Missing definition for TTC_LIGHT7_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT7_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT7_GPIO
        #warning Missing definition for TTC_LIGHT7_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT7_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT7_MAX_DUTY
        #define TTC_LIGHT7_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT8
    #ifndef TTC_LIGHT8_LOWACTIVE
        #warning Missing definition for TTC_LIGHT8_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT8_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT8_GPIO
        #warning Missing definition for TTC_LIGHT8_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT8_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT8_MAX_DUTY
        #define TTC_LIGHT8_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT9
    #ifndef TTC_LIGHT9_LOWACTIVE
        #warning Missing definition for TTC_LIGHT9_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHT9_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHT9_GPIO
        #warning Missing definition for TTC_LIGHT9_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHT9_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHT9_MAX_DUTY
        #define TTC_LIGHT9_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
#ifdef TTC_LIGHT10
    #ifndef TTC_LIGHTTTC_LIGHT10_LOWACTIVE
        #warning Missing definition for TTC_LIGHTTTC_LIGHT10_LOWACTIVE. Will use default value. Define it as 1 if LIGHT is switched on by setting gpio pin to high or 0 otherwise!!
        #define TTC_LIGHTTTC_LIGHT10_LOWACTIVE 0
    #endif
    #ifndef TTC_LIGHTTTC_LIGHT10_GPIO
        #warning Missing definition for TTC_LIGHTTTC_LIGHT10_GPIO. Add a line to your makefile that defines it as one from e_ttc_gpio_GPIO ! Check ttc_light.h for more details!
        #define TTC_LIGHTTTC_LIGHT10_GPIO E_ttc_gpio_pin_none
    #endif
    #ifndef TTC_LIGHTTTC_LIGHT10_MAX_DUTY
        #define TTC_LIGHTTTC_LIGHT10_MAX_DUTY 255  // Only valid if LIGHT is controlled using pulse width modulation. 0=always off; 255=always on
    #endif
#endif
//}

//}Static Configuration

//{ Includes2 ************************************************************
//
// _types.h Header files should include only other *_types.h header files to avoid circular includes.
// Includes that provide function declarations should be placed
// in "ttc_light.h" or "ttc_light.c"
//

//InsertIncludes above (DO NOT DELETE THIS LINE!)

#ifdef EXTENSION_light_gpio
    #include "light/light_gpio_types.h" // automatically added by ./create_DeviceDriver.pl
#endif
//InsertTypesIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Defines **************************************************************

//}

//InsertDefines above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Enums/ Structures ****************************************************

typedef enum {   // e_ttc_light_errorcode       return codes of LIGHT devices
    E_ttc_light_errorcode_OK = 0,

    // other warnings go here..

    E_ttc_light_errorcode_ERROR,                  // general failure
    E_ttc_light_errorcode_NULL,                   // NULL pointer not accepted
    E_ttc_light_errorcode_DeviceNotFound,         // corresponding device could not be found
    E_ttc_light_errorcode_InvalidConfiguration,   // sanity check of device configuration failed
    E_ttc_light_errorcode_InvalidImplementation,  // your code does not behave as expected

    // other failures go here..

    E_ttc_light_errorcode_unknown                // no valid errorcodes past this entry
} e_ttc_light_errorcode;
typedef enum {   // e_ttc_light_architecture    types of architectures supported by LIGHT driver
    E_ttc_light_architecture_None = 0,       // no architecture selected


    E_ttc_light_architecture_gpio, // automatically added by ./create_DeviceDriver.pl
    //InsertArchitectureEnum above (DO NOT DELETE THIS LINE!)

    E_ttc_light_architecture_unknown        // architecture not supported
} e_ttc_light_architecture;
//InsertEnums above (DO NOT DELETE THIS LINE!)

typedef union {  // architecture dependent configuration
    t_u8 unused;
#ifdef EXTENSION_light_gpio
    t_light_gpio_config gpio;  // Dynamically allocated as gpio[Config->AmountChannels]. Only access gpio[0..Config->AmountChannels-1]!
#endif
    //InsertArchitectureStructs above (DO NOT DELETE THIS LINE!)
} u_ttc_light_architecture;
typedef struct s_ttc_light_features { // static minimum, maximum and default values of features of single light channel

    // Add any amount of architecture independent values describing light devices here.
    // You may also want to add a plausibility check for each value to _ttc_light_configuration_check().

    t_u8 BrightnessMin;  // >BrightnessMax: minimum selectable brightness
    t_u8 BrightnessMax;  // <BrightnessMin: maximum selectable brightness
    t_u8 BrightnessStep; // >0:             minimum increasement step of brightness
    t_u8 reserved;       // fill up to 32 bits
} __attribute__( ( __packed__ ) ) t_ttc_light_features;
typedef union { // definition of a wide range periodic blinking for one channel (1/20000 Hz .. 500kHz)
    /* Timing of Blinking
     *
     *  |Intensity
     *  |
     *  |     .---          .---
     *  |    /!  !\        /!  !     *  |   / !  ! \      / !  !      *  |__/  !  !  \____/  !  !  \__
     * -+-+---+--+--+---+---+--+------> Time
     *  | 0   1  2  3   4   1  2 ...
     *
     *  T0 = Offset to central timer
     *  T1 = T(1) - T(0) = T(1) - T(4)
     *  T2 = T(2) - T(1)
     *  T3 = T(3) - T(2)
     *  T4 = T(4) - T(3)
     *
     * H3: Frequency range
     *
     * H4: Lowest Frequency:
     *     Multiplier = 2^16-1 = 65535
     *     T0 = T1 = T2 = T3 = T4 = 2^8-1 = 255
     *     Period    = 255 * 5 * 255 ms = 325 s = 5 minutes 25 seconds
     *     Frequency = (1 / 83557) Hz
     *
     * H4: Highest Frequency:
     *     Multiplier = 1
     *     T0 = T1 = T3 = 0
     *     T2 = T4 = 1
     *     Period    = 2 ms
     *     Frequency = 1 / (2 ms) = 500 Hz
     */
    t_u8 Multiplier; // milliseconds being multiplied to each T0..T4 times below
    union {
        struct { // individual, named times
            t_u8 T0_Off;      // time to shift phase start relative to other LIGHTs using same frequency
            t_u8 T1_RampUp;   // time to ramp up from BrightnessMin to BrightnessMax
            t_u8 T2_Full;     // time to hold BrightnessMax
            t_u8 T3_RampDown; // time to ramp down from BrightnessMax to BrightnessMin
            t_u8 T4_Off;      // time to hold BrightnessMin
        } named;
        t_u8 array[5]; // all time periods in one array for automated access
    } times;

} t_ttc_light_blinking;
typedef enum { // e_ttc_light_type - types of light
    E_ttc_light_type_None = 0,
    E_ttc_light_type_Channels_1, // single channel light source (monochromatic)           ->
    E_ttc_light_type_Channels_2, // dual channel light source (unusual)                   ->
    E_ttc_light_type_Channels_3, // triple channel light source (red+gren+blue)           ->
    E_ttc_light_type_Channels_4, // quadruple channel light source (red+gren+blue+white)  ->
    // Note: Update switch inside ttc_light_load_defaults() when adding types!

    E_ttc_light_type_Unknown   // all values below are unknown and illegal
} e_ttc_light_type;
typedef struct { //  init configuration of a single light channel
    const t_ttc_light_features* Features;   // constant features of this light channel (pointer will be cleared on ttc_led_reset())
    const t_ttc_light_blinking* Blinking;   // !=NULL: LIGHT will blink according to this data if switched on; ==NULL: no blinking (just on or off)
    t_u8                        Brightness; /* brightness level if switched on (Features->BrightnessMin .. Features->BrightnessMax)
                                             * After calling ttc_light_init(), this value can be adjusted by calling ttc_light_brightness())
                                             */
    TODO( "Add CIE correction!" );
} t_ttc_light_channel_init;
typedef enum {           // e_ttc_light_blinking_state - states of blinking statemachine for each channel
    E_ttc_light_blinking_state_T0_Off      = 0,
    E_ttc_light_blinking_state_T1_RampUp   = 1,
    E_ttc_light_blinking_state_T2_Full     = 2,
    E_ttc_light_blinking_state_T3_RampDown = 3,
    E_ttc_light_blinking_state_T4_Off      = 4,
    E_ttc_light_blinking_state_Overflow    = 0xff
} e_ttc_light_blinking_state;
typedef struct {         // blinking data of a single light channel (required during runtime to control automatic blinking)
    t_ttc_systick_delay        Delay;      // delay for current Time[TimeIndex]
    e_ttc_light_blinking_state State;      // 0xff,0..4 current time delay (==Overflow,Time0 ... Time4)
    t_s8                       Delta;      // Brightness Increase or decrease during E_ttc_light_blinking_state_T1_RampUp / E_ttc_light_blinking_state_T3_RampDown
} t_ttc_light_channel_blinking;
typedef union  {
    t_u8 Unused;
#ifdef EXTENSION_light_gpio
    t_light_gpio_channel_config  gpio;  // channel configuration for gpio controlled lights
#endif
    // Note: additional low-level configurations must be added manually!
} u_ttc_light_channel_low_level;

typedef struct { // configuration of a light with single channel light source
    t_ttc_light_channel_init       Init;             // Channel application configurable part. Call ttc_light_init() after changing any parameter inside!
    t_ttc_light_channel_blinking   PrivateBlinking;  // Private channel blinking data required during runtime to control automatic blinking
    u_ttc_light_channel_low_level* LowLevel;         // low-level driver configuration data for one channel
} t_ttc_light_channel_config;
typedef struct { // configuration of a light with single channel light source
    e_ttc_light_type Type; // E_ttc_light_type_Channels_1

    t_ttc_light_channel_config Channel[1];
} t_ttc_light_channels_1_config;
typedef struct { // configuration of a light with single channel light source
    e_ttc_light_type Type; // E_ttc_light_type_Channels_2

    t_ttc_light_channel_config Channel[2];
} t_ttc_light_channels_2_config;
typedef struct { // configuration of a light with single channel light source
    e_ttc_light_type Type; // E_ttc_light_type_Channels_3

    t_ttc_light_channel_config Channel[3]; // red=Channel[0], green=Channel[1], blue=Channel[2]
} t_ttc_light_channels_3_config;
typedef struct { // configuration of a light with single channel light source
    e_ttc_light_type Type; // E_ttc_light_type_Channels_4

    t_ttc_light_channel_config Channel[4]; // red=Channel[0], green=Channel[1], blue=Channel[2], white=Channel[3]
} t_ttc_light_channels_4_config;
typedef union { // union of different light type configurations
    // Type of light configuration structure being stored in this structue.
    e_ttc_light_type Type;

    t_ttc_light_channels_1_config channels1; // Type == E_ttc_light_type_Channels_1
    t_ttc_light_channels_2_config channels2; // Type == E_ttc_light_type_Channels_2
    t_ttc_light_channels_3_config channels3; // Type == E_ttc_light_type_Channels_3
    t_ttc_light_channels_4_config channels4; // Type == E_ttc_light_type_Channels_4
} u_ttc_light_type_config;
typedef struct s_ttc_light_config { //          architecture independent configuration data
    TTC_LIST_ITEM_HEADER( struct s_ttc_light_config ); // t_ttc_light_config can be added to a ttc_list (must be first entry!)

    // Hint: Order structure elements for optimal alignment.
    //       You should place smaller datatypes like t_u8 and t_u16 after all t_u32 and pointer types
    //       to prevent insertion of pad-bytes. Most architectures don't support placing 32-bit datatypes at
    //       unaligned address.

    // Light type dependend configuration data.
    // Check Light.Type value before accessing any channels* sub data!
    // Use ttc_light_channel_config() to read or write this area in a safe way!
    u_ttc_light_type_config* Light;

    // Fields below are read-only for application! ----------------------------------------------------------

    u_ttc_light_architecture* LowLevelConfig; // low-level configuration array for 1..4 channels (structure defined and allocated by low-level driver)


    struct { // status flags common for all architectures
        unsigned Initialized        : 1;  // ==1: device has been initialized successfully
        unsigned On                 : 1;  // ==1: device has been switched on; ==0: ...off
        unsigned IsQueued4Blinking  : 1;  // ==1: device is queued to be controlled by _ttc_light_task_blinking()
        unsigned IsBlinking         : 1;  // ==1: device is currently controlled by _ttc_light_task_blinking()
    } Flags;

    t_u8                      LogicalIndex;       // automatically set: logical index of device to use (1 = TTC_LIGHT1_GPIO1, ...)
    t_u8                      AmountChannels;     // amount of channels supported by this light (common values: 1 for monochromatic and 3 for rgb lights)
    e_ttc_light_architecture  Architecture;       // type of architecture used for current light device

    // Additional high-level attributes go here..

} __attribute__( ( __packed__ ) ) t_ttc_light_config;
//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//}Structures

#endif // TTC_LIGHT_TYPES_H
