#ifndef ARCHITECTURE_USART_TYPES_H
#define ARCHITECTURE_USART_TYPES_H


/** { usart_stm32f1xx_types.h ************************************************
 *
 *                          The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Low-Level datatypes for USART devices on stm32f1xx architectures.
 *  Structures, Enums and Defines being required by ttc_usart_types.h
 *
 *  Created from template device_architecture_types.h revision 20 at 20140217 09:36:59 UTC
 *
 *  Note: See ttc_usart.h for description of architecture independent USART implementation.
 *
 *  Authors: Gregor Rebel and Adrián Romero
 *
}*/
//{ Defines/ TypeDefs ******************************************************

//InsertDefines above (DO NOT DELETE THIS LINE!)

// maximum amount of available USART/ UART units in current architecture
#define TTC_USART_MAX_AMOUNT 5

//InsertEnums above (DO NOT DELETE THIS LINE!)

#define _driver_usart_isr_rx    usart_stm32f1xx_rx_isr // define function pointer for receive interrupt service routine
#define _driver_usart_isr_tx    usart_stm32f1xx_tx_isr // define function pointer for transmit interrupt service routine
#define t_ttc_usart_register    t_register_stm32f1xx_usart
#define t_ttc_usart_register_tx t_register_stm32f1xx_usart
#define t_ttc_usart_register_rx t_register_stm32f1xx_usart
//InsertTypeDefs above (DO NOT DELETE THIS LINE!)

//} Defines
//{ Includes ***************************************************************

#include "../ttc_basic.h"
#include "../gpio/gpio_stm32f1xx.h"
#include "../register/register_stm32f1xx.h"
//InsertIncludes above (DO NOT DELETE THIS LINE!)

//} Includes
//{ Structures/ Enums required by ttc_usart_types.h ***********************

typedef struct { // register description (adapt according to stm32f1xx registers)
    unsigned Reserved1 : 16;
    unsigned Reserved2 : 16;
} t_usart_register;

typedef struct {  // stm32f1xx specific configuration data of single USART device
    t_u32* BitBand_SR_RxNE; // bit banding address: SR register RxNE Bit
} __attribute__( ( __packed__ ) ) t_usart_stm32f1xx_config;
// ttc_usart_arch_t is required by ttc_usart_types.h
#define t_ttc_usart_architecture t_usart_stm32f1xx_config
//X #undef t_ttc_usart_architecture // not storing low-level configuration data

//} Structures/ Enums


#endif //ARCHITECTURE_USART_TYPES_H
