#ifndef EXAMPLE_LEDS
#define EXAMPLE_LEDS

/*{ ExampleLeds::.h ************************************************

 Empty template for new header files.
 Copy and adapt to your needs.

}*/
//{ Includes *************************************************************
// Header files should typically only include *_types.h files
// to avoid nasty include loops.

#include "../ttc-lib/ttc_basic_types.h"    // basic datatypes for current microcontroller architecture
#include "../ttc-lib/ttc_gpio_types.h"     // allows to store gpio pins in structures
#include "../ttc-lib/ttc_task_types.h"     // allows to store task delay data in structures
#include "../ttc-lib/ttc_list_types.h"     // customizable single linked lists

//}Includes
//{ Defines/ TypeDefs ****************************************************

//}Defines
//{ Structures/ Enums ****************************************************

// define a list item that will store LED configuration data and is compatible with ttc_list
typedef struct s_configled { // t_config_led
    TTC_LIST_ITEM_HEADER( struct s_configled );  // this makes our structure compatible with ttc_list

    e_ttc_gpio_pin      PortPin;          // LED to use
    t_u8                Brightness;       // ==0: off, ==99: maximum duty level
    t_u8                MaxDuty;          // Maximum duty level * 255 (255 = always on, 0 = always off)
    t_s16               OnTime;           // ==0: off, ==255: 100% on, ==-1: must be recalculated via calcOnTime()
    BOOL                LowActive;        // == TRUE: Port pin must be set to 0V to activate LED; == FALSE: Port pin must be set..
    t_s8                Velocity;         // required for cool multidimensional lightning effect
    t_s8                Acceleration;     // required for cool multidimensional lightning effect
} t_config_led;

// define a list datatype especially for storing t_config_led type items
typedef struct s_my_list { // t_config_led
    TTC_LIST_HEADER( t_config_led );
} t_my_list;

//}Structures/ Enums
//{ Function declarations **************************************************

// main entry: initializes hardware + spawns all threads
// returns immediately
void example_leds_prepare();

// task: changes brightness continuosly
void taskLightShow( void* TaskArgument );

// task: applies brightness to single LED
void taskLedPWM( void* TaskArgument );

//InsertFunctionDeclarations above (DO NOT DELETE THIS LINE!)
//}Function Declarations

#endif //EXAMPLE_LEDS
