#ifndef EXAMPLE_BOARD_H
#define EXAMPLE_BOARD_H

/** example_ttc_BOARD.h **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Example for low-cost discovery board for stm32f100 from st microelectronics providing only one button an led, pin headers and a swd programming interface
 *
 *  Basic flow of examples with enabled multitasking:
 *  (1) example_ttc_board_stm32l100_discovery_prepare() gets called in single tasking mode from 
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates all required tasks
 *      - it returns immediately (if multitasking should be started) 
 *  (2) Scheduler is started.
 *  (3) Tasks being created by example_ttc_board_stm32l100_discovery_prepare() get executed.
 *
 *  Basic flow of examples with disabled multitasking:
 *  (1) example_ttc_board_stm32l100_discovery_prepare() gets called in single tasking mode from 
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates first task
 *  (2) Every registered task function is called one after another in an endless loop
 *      from ttc_task_start_scheduler().
 *
 *  Created from template example_ttc_device.h revision 19 at 20180705 13:29:54 UTC
 *
 *  Authors: <AUTHOR>
 *
 *  
 *  Description of example_ttc_BOARD
 *
 *  ToDo: Document this example!
}*/
//{ Includes *************************************************************

// Include all header files being required to include this file.
// Includes being required to compile example_ttc_BOARD.c only do not belong here! 
// This is why header files typically include *_types.h files only (avoids nasty include loops).

#include "../ttc-lib/ttc_basic_types.h"    // basic datatypes for current microcontroller architecture
#include "../ttc-lib/ttc_gpio_types.h"     // allows to store gpio pins in structures 
#include "../ttc-lib/ttc_systick_types.h"  // allows to store task delay data in structures
#include "../ttc-lib/ttc_board_types.h" // datatypes of board devices

//}Includes
//{ Defines/ TypeDefs ****************************************************

//}Defines
//{ Structures/ Enums ****************************************************

typedef struct { // example_board_t
    t_ttc_board_config* Config_board; // pointer to configuration of board device to use
    t_ttc_systick_delay    Delay;           // this type of delay is usable in single- and multitasking setup

    // more arguments...
} t_example_board_data;

//}Structures/ Enums
//{ Function prototypes **************************************************

/** start point of this example
 *
 * - initializes data 
 * - spawns all threads
 * - returns immediately
 */
void example_ttc_board_prepare();


//}Function prototypes

#endif //EXAMPLE_BOARD_H
