#ifndef EXAMPLE_GPIO_H
#define EXAMPLE_GPIO_H

/** example_ttc_GPIO.h **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Example for gpio driver for stm32f1xx architecture
 *
 *  Basic flow of examples with enabled multitasking:
 *  (1) example_ttc_gpio_stm32f1xx_prepare() gets called in single tasking mode from 
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates all required tasks
 *      - it returns immediately (if multitasking should be started) 
 *  (2) Scheduler is started.
 *  (3) Tasks being created by example_ttc_gpio_stm32f1xx_prepare() get executed.
 *
 *  Basic flow of examples with disabled multitasking:
 *  (1) example_ttc_gpio_stm32f1xx_prepare() gets called in single tasking mode from 
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates first task
 *  (2) Every registered task function is called one after another in an endless loop
 *      from ttc_task_start_scheduler().
 *
 *  Created from template example_ttc_device.h revision 18 at 20180202 13:26:39 UTC
 *
 *  Authors: <AUTHOR>
 *
 *  
 *  Description of example_ttc_GPIO
 *
 *  ToDo: Document this example!
}*/
//{ Includes *************************************************************

// Include all header files being required to include this file.
// Includes being required to compile example_ttc_GPIO.c only do not belong here! 
// This is why header files typically include *_types.h files only (avoids nasty include loops).

#include "../ttc-lib/ttc_basic_types.h"    // basic datatypes for current microcontroller architecture
#include "../ttc-lib/ttc_gpio_types.h"     // allows to store gpio pins in structures 
#include "../ttc-lib/ttc_task_types.h"     // allows to store task delay data in structures
#include "../ttc-lib/ttc_gpio_types.h" // datatypes of gpio devices

//}Includes
//{ Defines/ TypeDefs ****************************************************

//}Defines
//{ Structures/ Enums ****************************************************

typedef struct { // example_gpio_t
    t_ttc_gpio_config* Config_GPIO; // pointer to configuration of gpio device to use
    t_ttc_systick_delay    Delay;           // this type of delay is usable in single- and multitasking setup

    // more arguments...
} example_gpio_data_t;

//}Structures/ Enums
//{ Function prototypes **************************************************

/** start point of this example
 *
 * - initializes data 
 * - spawns all threads
 * - returns immediately
 */
void example_ttc_gpio_prepare();


//}Function prototypes

#endif //EXAMPLE_GPIO_H
