#ifndef EXAMPLE_GPIO_STM32F1XX_H
#define EXAMPLE_GPIO_STM32F1XX_H

/** example_ttc_GPIO_STM32F1XX.h **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  This source should provide a small and simple to understand code example
 *  how to use ttc_gpio_stm32f1xx.
 *  The basic flow is as follows:
 *  (1) example_ttc_gpio_stm32f1xx_prepare() gets called in single tasking mode from ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global variables
 *      - it creates all required tasks
 *      - it returns immediately (only if multitasking should be started) 
 *  (2) Scheduler is started
 *  (3) Tasks being created by example_ttc_gpio_stm32f1xx_prepare() gets executed
 *
 *  Created from template example_ttc_device_architecture.h revision 16 at 20180202 13:26:39 UTC
 *
 *  Authors: <AUTHOR>
 *  
}*/
//{ Includes *************************************************************

// Include all header files being required to include this file.
// Includes being required to compile example_ttc_GPIO.c only do not belong here! 
// This is why header files typically include *_types.h files only (avoids nasty include loops).

#include "../ttc-lib/ttc_basic_types.h"    // basic datatypes for current microcontroller architecture
#include "../ttc-lib/ttc_gpio_types.h"     // allows to store gpio pins in structures 
#include "../ttc-lib/ttc_task_types.h"     // allows to store task delay data in structures
#include "../ttc-lib/ttc_gpio_types.h" // datatypes of gpio devices
#include "../ttc-lib/gpio/gpio_stm32f1xx_types.h" // datatypes of stm32f1xx gpio devices

//}Includes
//{ Defines/ TypeDefs ****************************************************

//}Defines
//{ Structures/ Enums ****************************************************

typedef struct { // example_gpio_stm32f1xx_t
    t_ttc_gpio_config* Config_GPIO; // pointer to configuration of gpio device to use
    t_ttc_systick_delay   Delay;            // this type of delay is usable in single- and multitasking setup

    // more arguments...
} example_gpio_stm32f1xx_data_t;

//}Structures/ Enums
//{ Function prototypes **************************************************

/** start point of this example
 *
 * - initializes data 
 * - spawns all threads
 * - returns immediately
 */
void example_ttc_gpio_stm32f1xx_prepare();


//}Function prototypes

#endif //EXAMPLE_GPIO_STM32F1XX_H
