#ifndef EXAMPLE_LIGHT_H
#define EXAMPLE_LIGHT_H

/** example_ttc_light.h **********************************************{
 *
 *                              The ToolChain
 *
 *  The ToolChain has been originally developed by Gregor Rebel 2010-2019.
 *
 *  Example for universal led controller.
 *
 *  Basic flow of examples with enabled multitasking:
 *  (1) example_ttc_light_gpio_prepare() gets called in single tasking mode from
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates all required tasks
 *      - it returns immediately (if multitasking should be started)
 *  (2) Scheduler is started.
 *  (3) Tasks being created by example_ttc_light_gpio_prepare() get executed.
 *
 *  Basic flow of examples with disabled multitasking:
 *  (1) example_ttc_light_gpio_prepare() gets called in single tasking mode from
 *      ttc_extensions.c:ttc_extensions_start() at system boot
 *      - it initializes all required devices
 *      - it initializes its global and locat static variables
 *      - it creates first task
 *  (2) Every registered task function is called one after another in an endless loop
 *      from ttc_task_start_scheduler().
 *
 *  Created from template example_ttc_device.h revision 19 at 20200104 17:59:03 UTC
 *
 *  Authors: <AUTHOR>
 *
 *
 *  Description of example_ttc_light
 *
 *  ToDo: Document this example!
}*/
//{ Includes *************************************************************

// Include all header files being required to include this file.
// Includes being required to compile example_ttc_light.c only do not belong here!
// This is why header files typically include *_types.h files only (avoids nasty include loops).

#include "../ttc-lib/ttc_basic_types.h"    // basic datatypes for current microcontroller architecture
#include "../ttc-lib/ttc_gpio_types.h"     // allows to store gpio pins in structures 
#include "../ttc-lib/ttc_systick_types.h"  // allows to store task delay data in structures
#include "../ttc-lib/ttc_light_types.h"    // datatypes of light devices

//}Includes
//{ Defines/ TypeDefs ****************************************************

//}Defines
//{ Structures/ Enums ****************************************************
typedef enum {   // statemachine states
    E_example_ttc_light_state_Reset = 0,            // default state at reset
    E_example_ttc_light_state_Initialized,          // task has finished first initialization
    E_example_ttc_light_state_Blinking_Manual1,
    E_example_ttc_light_state_Blinking_Manual2,
    E_example_ttc_light_state_Blinking_ManualEnd,   // end of manual blinking

    E_example_ttc_light_state_Blinking_Automatic1,  // prepare alol leds for automatic blinking
    E_example_ttc_light_state_Blinking_Automatic2,  // wait while all leds are blinking automatically
    E_example_ttc_light_state_Blinking_Automatic3,  // remove autommatic blinking from all leds
    E_example_ttc_light_state_Blinking_Automatic4,  // wait while all leds are off
    E_example_ttc_light_state_Blinking_AutomaticEnd,// end of automatic blinking

    E_example_ttc_light_state_Invalid               // from here, all values are invalid
} e_example_ttc_light_state;
typedef struct { // example_ttc_light_t
    e_example_ttc_light_state    State;
    t_ttc_systick_delay    Delay;             // this type of delay is usable in single- and multitasking setup
    t_u16                  Counter;           // generic counter

    // more argument s...
} t_example_ttc_light_data;

//}Structures/ Enums
//{ Function prototypes **************************************************

/** start point of this example
 *
 * - initializes data
 * - spawns all threads
 * - returns immediately
 */
void example_ttc_light_prepare();


//}Function prototypes

#endif //EXAMPLE_LIGHT_H
