/*{ TemplateName::main.c *************************************************
 *                        TheToolChain
 *
 * Project   TemplateName
 * Author:   TemplateAuthor
 * Template: main.c rev 12
 *
 * This code is part of a project for the TheToolChain framework for
 * microcontrollers with ARM CortexM3 CPU core.
 * TheToolChain is a free of charge operating system and development environment
 * for the before mentioned uCs. No additional closed source software is required.
 *
 * TheToolChain has been written 2010-2019 by Gregor Rebel
 *
 *
 * Support for external tools
 *
 * + QtCreator
 *   The subfolder QtCreator/ contains an automatically updated creator-project.
 *   This project can be loaded into QtCreator. Using this IDE seems to be more comfortable
 *   for many developers. Just issue this script to spawn it:
 *   ./qtcreator.sh
 *
 * + Eclipse
 *   It is possible to use Eclipse for ToolChain project.
 *   Unfortunately, no-one has build a script to create an Eclipse project up to now.
 *
 * + DDD
 *   The Data Display Debugger can be used for graphical live debugging of your code.
 *   See chapter Debug below!
 *
 *
 * Prerequisites
 *
 * + TheToolChain (http://TheToolChain.com)
 *   The required release should be downloaded and installed automatically by issuing ./compile.sh
 *
 *
 * Design flow
 *
 * + Modify activate_project.sh to configure it according to your current board and programmer
 *   - uncomment provideFeature line for your board
 *   - uncomment your programmer in rank 350
 *   - uncomment life example in rank 600
 *
 * + clean up project, compile + flash it onto uC:
 *   ./clean.sh; ./compile.sh
 *
 * + Browse example code
 *   The source code of activated extensions should show up in QtCreator after successfull compilation.
 *   You may modify existing examples and use them as starting point to develop your own application.
 *
 *
 * Debug
 *
 *   ./debug.sh     # see readme.TheToolChain for details about console debugging
 *   _/automate.pl  # supports textual and graphical debugging
 *
 *
 * Source-Code Organisation
 *
 *  + main.c (this file)
 *    Ideally, the main.c file contains only some function calls of names like
 *    init_XXX() or start_XXX(). The intention of this file is to show an overview
 *    of your project and how to use your code.
 *    Place real algorithms and implementations into extra .c and .h files and include
 *    them from here. This makes reusing your code in other projects much easier.
 *
 *  + Add new or existing source files to your project
 *    Use commandline tool _/source.pl in your project folder to add source files to your project:
 *    cd ~/Source/Projects/MY_PROJECT_FOLDER
 *    _/source.pl add foo.c foo.h
 *
 *    The basic main.c created for this project has been designed to
 *    support different usage scenarios out of the box:
 *
 *    1) Single Tasking
 *       ttc_task_start_scheduler() calls all registered task functions one after another in an endless loop.
 *
 *       Pros:
 *       - Each statemachine function can use the complete stack memory.
 *       - No inter task synchronization required.
 *
 *       Cons:
 *       - Statemachines quickly get lengthy and difficult to read.
 *         => see _/source.pl below!
 *
 *    2) Multi Tasking
 *       A central multitasking scheduler (currently FreeRTOS) starts all registered task functions
 *       and switches CPU from one to another.
 *
 *       Pros:
 *       - No need to implement code as statemachine.
 *       - Tasks can sleep while waiting for events (e.TemplateAuthor for a data receival in a port)
 *
 *       Cons:
 *       - Each task requires its own stack.
 *       - Size of task stacks is difficult to determine and often increases overall RAM usage.
 *       - Race conditions can lead to deadlocks and are difficult to debuTemplateAuthor
 *
 *    3) Direct use of activated example frameworks
 *       Examples are prepared and run automatically.
 *       No changes to main.c are required.
 *
 *  + extensions.active/makefile
 *    Dynamically compiled file containing makefile fragments of all activated extensions.
 *    Do not edit this file! Changes will be lost at next compile run.
 *
 *  + extensions.local/makefile.700_extra_settings
 *    Edit this file to add lines to the makefile of your project.
 *
 *  + config/FreeRTOSConfiTemplateAuthorh
 *    Configures behaviour and features of FeeRTOS (if multitasking scheduler FreeRTOS is used).
 *
 *  + _/source.pl
 *    Versatile source code generator:
 *    - Creates new source code files for your project.
 *    - Adds existing source code files to your project.
 *    - Creates public or private functions in existing source files.
 *    - Creates special feature functions for multi-architecture ttc-drivers.
 *    - Creates complete statemachines based on ttc_states.
 *    Execute _/source.pl in your project folder to see its documentation.
 *
 *  + _/automate.pl
 *    Automated compilation, flashing and debugging using multiple debug interfaces.
 *    Can insert additional constant definitions for next compile run.
 *    Execute _/automate.pl in your project folder to see its documentation.
 *
 *
 * Documentation
 * - http://thetoolchain.com/readme.TheToolChain
 * - http://thetoolchain.com/documentation/
 * - ~/Source/TheToolChain/Documentation/  (after installing The ToolChain)
}*/

//{ Includes *************************************************************

#include "ttc-lib/ttc_extensions.h"  // will include activated extensions
//InsertIncludes

//}Includes
//{ Defines/ TypeDefs ****************************************************


//}Defines
//{ Datatypes ************************************************************

typedef struct { // transports any amount of arguments to taskMain()
    t_u16 Unused;  // unused (structs may not be empty)!

    // add more data fields here
} t_main_arguments;

//}Datatypes
//{ Global Variables *****************************************************

//}Global Variables
//{ Function prototypes **************************************************

//InsertPrivateFunctionDeclarations above (DO NOT DELETE THIS LINE!)

//}Function prototypes
//{ Function definitions *************************************************

void main_task( void* Argument ) {         // example task will be run when multitasking scheduler (e.TemplateAuthor FreeRTOS) has been started
    Assert_Writable( Argument, E_ttc_assert_origin_auto );  // always check pointer arguments with Assert() before dereferencing them!
    t_main_arguments* TaskData = ( t_main_arguments* ) Argument;    // we're expecting this pointer type

    do {
        ( void ) TaskData->Unused; // this line can be removed safely

        // implement task statemachine here...

        ttc_task_yield(); // give cpu to other processes
    }
    while ( ttc_task_is_switching_available() ); // Tasks run endlessly if multitasking is enabled and quit immediately otherwise
}
void main_prepare() {                      // prepares your application and creates tasks
    /** This the last single task function before multitasking scheduler is started.
     *  Use it to create tasks, initialize devices and allocate memories.
     *  If it does not return, multitasking will not be available.
     *  See implementation of startExtensions() for details.
     *
     * IMPORTANT: DO NOT DEVELOP YOUR APPLICATION IN THIS FILE!
     *            This file contains the main() function. This makes it impossible to
     *            include it from other files. So nobody can reuse what you code here.
     *            main.c takes includes and function calls to run your application.
     *            Develop your application in a separate MY_APPLICATION.c/.h pair.
     *            You may simply create such like this (type into shell within your project folder):
     *            _/source.pl add MY_APPLICATION
     *            Then recompile and see that MY_APPLICATION.c/.h has been added to your project.
     */

    // call preparation of your application from here...

    if ( 0 ) { // example task creation (enable if you want to use main_task() !)

        // initialize some data to be used by taskMain()
        static t_main_arguments TaskData1; // must be static or global to allocate memory globally!
        TaskData1.Unused = 0xc001;         // just an example value (Delete if you like)

        ttc_task_create(
            main_task,         // function to start as thread
            "tMain",          // thread name (just for debugging)
            256,              // stack size (multitasking only: Increase if main_task() needs more to avoid memory corruption!)
            &TaskData1,       // passed as argument to main_task()
            ( t_base ) 1,     // task priority (multitasking only: higher values mean more process time)
            NULL              // can return a handle to created task
        );
    }
}
void main( void ) {                        // start of program flow

    // Do NOT put any code in this function. Use taskMain() below or modify code of activated examples!

    // ttc_extensions_start() will
    // 1) Run prepare functions of all activated extensions.
    // 2) Call main_prepare() to initialize hardware and create tasks.
    // 3) Start multitasking scheduler to run all created tasks.
    ttc_extensions_start();

    // Program flow will never reach here..
}

//InsertPrivateFunctionDefinitions above (DO NOT DELETE THIS LINE!)

//}Function definitions
