#!/bin/bash

if [ "$USER" == "root" ]; then
  echo "$0 - ERROR: Must not run this script as user root! - TERMINATING"
  exit 15  
fi
PWD_OK=""
if [ -d ".git" ]; then #{ check if we have been called from correct folder
  if [ -e ".readme.TheToolChain" ]; then
    PWD_OK="1"
  fi
fi #}
if [ "$PWD_OK" == "" ]; then #{ script was started in wrong working directory
  echo "$0 - ERROR: This script must be started from base of TheToolChain git repository! (PWD=`pwd`)"
  exit 16
fi #}

Version="$1"; shift
if [ "$1" == "b" ]; then
  Beta="b"; shift
fi
if [ "$1" == "a" ]; then
  Beta="a"; shift
fi
SSH_SERVER="$1"; shift
SSH_PATH="$1"; shift

echo "SSH_SERVER='$SSH_SERVER' SSH_PATH='$SSH_PATH'" #D
FullVersion=${Version}${Beta}
if [ "$Version" == "" ]; then #{ print help + exit
  cat <<END_OF_HELP
$0 <VERSION> [a|b] [<USER>@<SERVER> [<PATH>]]
    <VERSION> version number to append (x.x.xx e.g.: 1.0.35)
    a  an alpha release automatically overwrites existing alpha release of same version
    b  a beta release automatically overwrites existing beta release of same version
    <USER>    Username on remote ssh server.
    <SERVER>  Fully qualfied domain name or IP-address of ssh server where to upload new release to.
    <PATH>    Absolute or hom-folder relative directory path on ssh-server where to copy release files to.
    [ ]       Argument is optional.
    
Examples:
   $0 1.0.54
   $0 1.0.54 b
   $0 1.0.54 gregor@thetoolchain.com www/thetoolchain
   
END_OF_HELP
  exit 5
fi #}

Archive="TheToolChain_${FullVersion}.tar.bz"

RELEASE_DIR="`pwd`/../TTC_Releases"
if [ ! -d ${RELEASE_DIR} ]; then #{ create missing folder
  mkdir -v ${RELEASE_DIR}
fi #}
if  [ "$Beta" == "" ]; then #{ exit if non-beta archive already exists
  if [ -e ${RELEASE_DIR}/$Archive ]; then
    echo "$0 ERROR: ${RELEASE_DIR}/$Archive already exists!"
    exit 6
  fi
fi #}

function createLink() {      # Source,Link,User,QUIET  creates symbolic link (removes old one before)
  cl_Source="$1"
  cl_Link="$2"
  cl_User="$3"
  cl_Quiet="$4"
  
  cl_Quiet="" #D
  if [ "$cl_Quiet" = "" ]; then
    Verbose="-v"
    echo -n "`pwd`> "
  else
    Verbose=""
  fi
  if [ ! -e "$cl_Source" ]; then
    MissingFiles="$MissingFiles $cl_Source"
  fi
  
  rm -f "$cl_Link"
  if [ "$cl_User" != "" ]; then
    sudo -u$cl_User ln $Verbose -sf "$cl_Source" "$cl_Link"
  else
    ln $Verbose -sf "$cl_Source" "$cl_Link"
  fi
}

#? chmod 0700 $HOME/.ssh/id_rsa

if [ "1" == "1" ]; then #{ compile readme.History
  PreviousVersion=`grep -m1 'commit ' readme.History | awk '{print $2}'`
  if [ "$PreviousVersion" != "" ]; then
    mv readme.History readme.History_old
    echo "$FullVersion @`date`" >readme.History
    echo ">git fetch"
    git fetch
    echo ">git log"
    git log ${PreviousVersion}.. >>readme.History
    echo "---------------------------------------------------------"  >>readme.History
    echo "" >>readme.History
    cat readme.History_old >>readme.History
    rm readme.History_old
  fi
fi #}

echo ">git pull"
git pull
echo $FullVersion >Version
echo $FullVersion >InstallData/scripts/Version

bin/compile_ReadMe.TheToolChain.pl .readme.TheToolChain
#cp readme.* .

echo ">git commmit"
git commit -a -m "released Package v${FullVersion}"
echo ">git tag"
git tag -f "Package_v${FullVersion}"
echo ">git push --tags"
git push --tags -f
echo ">git push"
git push

cd ..
echo "creating archive $Archive.."

mv TheToolChain/.git .
mv TheToolChain TheToolChain_${FullVersion}
createLink TheToolChain_${FullVersion} TheToolChain
#? if [ "$Beta" != "" ]; then
#?   rm ${RELEASE_DIR}/*${Beta}.tar*
#? fi
tar cjf ${RELEASE_DIR}/${Archive} TheToolChain_${FullVersion} TheToolChain
rm TheToolChain
mv TheToolChain_${FullVersion} TheToolChain
mv .git TheToolChain/
cp -v TheToolChain/InstallData/deployment/install_TheToolChain* ${RELEASE_DIR}/

if [ "$SSH_SERVER" != "" ]; then #{ copy release onto ssh server
  echo "$0 - sending to ssh server $SSH_SERVER"
  ssh $SSH_SERVER <<SSH_SERVER
if [ ! -d ${SSH_PATH}/archive ]; then
  mkdir ${SSH_PATH}/archive
fi
SSH_SERVER

  scp ${RELEASE_DIR}/${Archive} ${SSH_SERVER}:${SSH_PATH}/archive/
  scp TheToolChain/readme\.* ${SSH_SERVER}:${SSH_PATH}/archive/

  ssh $SSH_SERVER <<SSH_SERVER
cd ${SSH_PATH}
ln -sfv archive/${Archive}          TheToolChain_current.tar.bz
ln -sfv archive/readme.History      readme.History.txt
ln -sfv archive/readme.TheToolChain readme.TheToolChain.txt
SSH_SERVER
  echo "$0 - Done sending release."
fi #}
  
#?createLink ${RELEASE_DIR}/${Archive} $Archive

#{ ToDo
# ln -s archive/TheToolChain_1.0.54.tar.bz TheToolChain_current.tar.bz
# ln -s TheToolChain.Contents_devel.tar.bz2 TheToolChain.Contents_1.0.54.tar.bz2
# scp readme.* toterbriefkasten.de:www/thetoolchain/
#}